/*
ssc (static site checker)
Copyright (c) 2020-2022 Dylan Harris
https://dylanharris.org/

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public Licence as published by
the Free Software Foundation, either version 3 of the Licence, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public Licence for more details.

You should have received a copy of the GNU General Public
Licence along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
*/

#include "main/standard.h"
#include "schema/schema_hierarchy.h"
#include "main/context.h"

struct microdata_hierachy
{   schema_version from_;
    schema_version to_;
    e_schema_type general_;
    e_schema_type specific_; };

microdata_hierachy schema_hierarchy [] =
{   { { s_faux, 1, 0 }, { 0, 0 }, anything, owl_thing },
    { { s_faux, 1, 0 }, { 0, 0 }, anything, sch_thing },
    { { s_faux, 1, 0 }, { 0, 0 }, anything, xsd_anytype },

    // activity streams etc.
    { { s_as, 2, 0 }, { 0, 0 }, asc_activity, asc_accept },
    { { s_as, 2, 0 }, { 0, 0 }, asc_activity, asc_add },
    { { s_as, 2, 0 }, { 0, 0 }, asc_activity, asc_announce },
    { { s_as, 2, 0 }, { 0, 0 }, asc_activity, asc_create },
    { { s_as, 2, 0 }, { 0, 0 }, asc_activity, asc_delete },
    { { s_as, 2, 0 }, { 0, 0 }, asc_activity, asc_dislike },
    { { s_as, 2, 0 }, { 0, 0 }, asc_activity, asc_flag },
    { { s_as, 2, 0 }, { 0, 0 }, asc_activity, asc_follow },
    { { s_as, 2, 0 }, { 0, 0 }, asc_activity, asc_ignore },
    { { s_as, 2, 0 }, { 0, 0 }, asc_activity, asc_intransitiveactivity },
    { { s_as, 2, 0 }, { 0, 0 }, asc_activity, asc_join },
    { { s_as, 2, 0 }, { 0, 0 }, asc_activity, asc_leave },
    { { s_as, 2, 0 }, { 0, 0 }, asc_activity, asc_like },
    { { s_as, 2, 0 }, { 0, 0 }, asc_activity, asc_listen },
    { { s_as, 2, 0 }, { 0, 0 }, asc_activity, asc_move },
    { { s_as, 2, 0 }, { 0, 0 }, asc_activity, asc_offer },
    { { s_as, 2, 0 }, { 0, 0 }, asc_activity, asc_reject },
    { { s_as, 2, 0 }, { 0, 0 }, asc_activity, asc_read },
    { { s_as, 2, 0 }, { 0, 0 }, asc_activity, asc_remove },
    { { s_as, 2, 0 }, { 0, 0 }, asc_activity, asc_tentativeaccept },
    { { s_as, 2, 0 }, { 0, 0 }, asc_activity, asc_undo },
    { { s_as, 2, 0 }, { 0, 0 }, asc_activity, asc_update },
    { { s_as, 2, 0 }, { 0, 0 }, asc_activity, asc_view },

    { { s_as, 2, 0 }, { 0, 0 }, asc_collection, asc_orderedcollection },
    { { s_as, 2, 0 }, { 0, 0 }, asc_collection, asc_collectionpage },

    { { s_as, 2, 0 }, { 0, 0 }, asc_collectionpage, asc_orderedcollectionpage },

    { { s_as, 2, 0 }, { 0, 0 }, asc_document, asc_audio },
    { { s_as, 2, 0 }, { 0, 0 }, asc_document, asc_image },
    { { s_as, 2, 0 }, { 0, 0 }, asc_document, asc_page },
    { { s_as, 2, 0 }, { 0, 0 }, asc_document, asc_video },

    { { s_as, 2, 0 }, { 0, 0 }, asc_ignore, asc_block },

    { { s_as, 2, 0 }, { 0, 0 }, asc_intransitiveactivity, asc_arrive },
    { { s_as, 2, 0 }, { 0, 0 }, asc_intransitiveactivity, asc_question },
    { { s_as, 2, 0 }, { 0, 0 }, asc_intransitiveactivity, asc_travel },

    { { s_as, 2, 0 }, { 0, 0 }, asc_link, asc_image },
    { { s_as, 2, 0 }, { 0, 0 }, asc_link, asc_mention },

    { { s_as, 2, 0 }, { 0, 0 }, asc_object, asc_activity },
    { { s_as, 2, 0 }, { 0, 0 }, asc_object, asc_application },
    { { s_as, 2, 0 }, { 0, 0 }, asc_object, asc_article },
    { { s_as, 2, 0 }, { 0, 0 }, asc_object, asc_collection },
    { { s_as, 2, 0 }, { 0, 0 }, asc_object, asc_document },
    { { s_as, 2, 0 }, { 0, 0 }, asc_object, asc_event },
    { { s_as, 2, 0 }, { 0, 0 }, asc_object, asc_group },
    { { s_as, 2, 0 }, { 0, 0 }, asc_object, asc_note },
    { { s_as, 2, 0 }, { 0, 0 }, asc_object, asc_organisation },
    { { s_as, 2, 0 }, { 0, 0 }, asc_object, asc_person },
    { { s_as, 2, 0 }, { 0, 0 }, asc_object, asc_place },
    { { s_as, 2, 0 }, { 0, 0 }, asc_object, asc_profile },
    { { s_as, 2, 0 }, { 0, 0 }, asc_object, asc_relationship },
    { { s_as, 2, 0 }, { 0, 0 }, asc_object, asc_service },
    { { s_as, 2, 0 }, { 0, 0 }, asc_object, asc_tombstone },

    { { s_as, 2, 0 }, { 0, 0 }, asc_offer, asc_invite },

    { { s_oa, 1, 0 }, { 0, 0 }, asc_orderedcollection, oa_choice },
    { { s_as, 2, 0 }, { 0, 0 }, asc_orderedcollection, asc_orderedcollectionpage },

    { { s_as, 2, 0 }, { 0, 0 }, asc_reject, asc_tentativereject },

    // bibo
    { { s_bibo, 1, 3 }, { 0, 0 }, bibo_article, bibo_academicarticle },

    { { s_bibo, 1, 3 }, { 0, 0 }, bibo_audiovisualdocument, bibo_film },

    { { s_bibo, 1, 3 }, { 0, 0 }, bibo_book, bibo_proceedings },

    { { s_bibo, 1, 3 }, { 0, 0 }, bibo_booksection, bibo_chapter },

    { { s_bibo, 1, 3 }, { 0, 0 }, bibo_collecteddocument, bibo_editedbook },
    { { s_bibo, 1, 3 }, { 0, 0 }, bibo_collecteddocument, bibo_issue },

    { { s_bibo, 1, 3 }, { 0, 0 }, bibo_collection, bibo_collection },
    { { s_bibo, 1, 3 }, { 0, 0 }, bibo_collection, bibo_multivolumebook },
    { { s_bibo, 1, 3 }, { 0, 0 }, bibo_collection, bibo_periodical },
    { { s_bibo, 1, 3 }, { 0, 0 }, bibo_collection, bibo_series },
    { { s_bibo, 1, 3 }, { 0, 0 }, bibo_collection, bibo_website },

    { { s_bibo, 1, 3 }, { 0, 0 }, bibo_document, bibo_article },
    { { s_bibo, 1, 3 }, { 0, 0 }, bibo_document, bibo_audiodocument },
    { { s_bibo, 1, 3 }, { 0, 0 }, bibo_document, bibo_audiovisualdocument },
    { { s_bibo, 1, 3 }, { 0, 0 }, bibo_document, bibo_book },
    { { s_bibo, 1, 3 }, { 0, 0 }, bibo_document, bibo_collecteddocument },
    { { s_bibo, 1, 3 }, { 0, 0 }, bibo_document, bibo_collection },
    { { s_bibo, 1, 3 }, { 0, 0 }, bibo_document, bibo_documentpart },
    { { s_bibo, 1, 3 }, { 0, 0 }, bibo_document, bibo_image },
    { { s_bibo, 1, 3 }, { 0, 0 }, bibo_document, bibo_legaldocument },
    { { s_bibo, 1, 3 }, { 0, 0 }, bibo_document, bibo_manual },
    { { s_bibo, 1, 3 }, { 0, 0 }, bibo_document, bibo_manuscript },
    { { s_bibo, 1, 3 }, { 0, 0 }, bibo_document, bibo_personalcommunicationdocument },
    { { s_bibo, 1, 3 }, { 0, 0 }, bibo_document, bibo_referencesource },
    { { s_bibo, 1, 3 }, { 0, 0 }, bibo_document, bibo_report },
    { { s_bibo, 1, 3 }, { 0, 0 }, bibo_document, bibo_slideshow },
    { { s_bibo, 1, 3 }, { 0, 0 }, bibo_document, bibo_standard },
    { { s_bibo, 1, 3 }, { 0, 0 }, bibo_document, bibo_thesis },
    { { s_bibo, 1, 3 }, { 0, 0 }, bibo_document, bibo_webpage },

    { { s_bibo, 1, 3 }, { 0, 0 }, bibo_documentpart, bibo_booksection },
    { { s_bibo, 1, 3 }, { 0, 0 }, bibo_documentpart, bibo_excerpt },
    { { s_bibo, 1, 3 }, { 0, 0 }, bibo_documentpart, bibo_slide },

    { { s_bibo, 1, 3 }, { 0, 0 }, bibo_event, bibo_conference },
    { { s_bibo, 1, 3 }, { 0, 0 }, bibo_event, bibo_hearing },
    { { s_bibo, 1, 3 }, { 0, 0 }, bibo_event, bibo_interview },
    { { s_bibo, 1, 3 }, { 0, 0 }, bibo_event, bibo_performance },
    { { s_bibo, 1, 3 }, { 0, 0 }, bibo_event, bibo_personalcommunication },
    { { s_bibo, 1, 3 }, { 0, 0 }, bibo_event, bibo_workshop },

    { { s_bibo, 1, 3 }, { 0, 0 }, bibo_excerpt, bibo_quote },

    { { s_bibo, 1, 3 }, { 0, 0 }, bibo_image, bibo_map },

    { { s_bibo, 1, 3 }, { 0, 0 }, bibo_legalcasedocument, bibo_brief },
    { { s_bibo, 1, 3 }, { 0, 0 }, bibo_legalcasedocument, bibo_legaldecision },

    { { s_bibo, 1, 3 }, { 0, 0 }, bibo_legaldocument, bibo_legalcasedocument },
    { { s_bibo, 1, 3 }, { 0, 0 }, bibo_legaldocument, bibo_legislation },

    { { s_bibo, 1, 3 }, { 0, 0 }, bibo_legislation, bibo_bill },
    { { s_bibo, 1, 3 }, { 0, 0 }, bibo_legislation, bibo_statute },

    { { s_bibo, 1, 3 }, { 0, 0 }, bibo_periodical, bibo_code },
    { { s_bibo, 1, 3 }, { 0, 0 }, bibo_periodical, bibo_courtreporter },
    { { s_bibo, 1, 3 }, { 0, 0 }, bibo_periodical, bibo_journal },
    { { s_bibo, 1, 3 }, { 0, 0 }, bibo_periodical, bibo_magazine },
    { { s_bibo, 1, 3 }, { 0, 0 }, bibo_periodical, bibo_newspaper },
    { { s_bibo, 1, 3 }, { 0, 0 }, bibo_periodical, bibo_note },
    { { s_bibo, 1, 3 }, { 0, 0 }, bibo_periodical, bibo_patent },

    { { s_bibo, 1, 3 }, { 0, 0 }, bibo_personalcommunicationdocument, bibo_email },
    { { s_bibo, 1, 3 }, { 0, 0 }, bibo_personalcommunicationdocument, bibo_letter },

    // common tag
    { { s_ctag, 1, 0 }, { 0, 0 }, ctag_tag, ctag_author },
    { { s_ctag, 1, 0 }, { 0, 0 }, ctag_tag, ctag_auto },
    { { s_ctag, 1, 0 }, { 0, 0 }, ctag_tag, ctag_reader },

    // data catalogue
    { { s_dcat, 1, 0 }, { 0, 0 }, dca_dataset, dca_catalogue },

    { { s_dcat, 1, 0 }, { s_dcat, 1, 0 }, dcmi_dataset, dca_dataset },
    { { s_void, 1, 0 }, { 0, 0 }, dcmi_dataset, void_dataset },
    { { s_void, 1, 0 }, { 0, 0 }, dcmi_dataset, void_linkset },

    { { s_dcat, 2, 0 }, { 0, 0 }, dca_resource, dca_dataservice },
    { { s_dcat, 2, 0 }, { 0, 0 }, dca_resource, dca_dataset },

    { { s_dcat, 2, 0 }, { 0, 0 }, dcmi_service, dca_dataservice },

    // dcterms
    { { s_wdrs, 1, 0 }, { 0, 0 }, dct_agent, wdrs_processor },

    // doap
    { { s_doap, 1, 0 }, { 0, 0 }, doap_repository, doap_archrepository },
    { { s_doap, 1, 0 }, { 0, 0 }, doap_repository, doap_bkrepository },
    { { s_doap, 1, 0 }, { 0, 0 }, doap_repository, doap_cvsrepository },
    { { s_doap, 1, 0 }, { 0, 0 }, doap_repository, doap_darcsrepository },
    { { s_doap, 1, 0 }, { 0, 0 }, doap_repository, doap_gitrepository },
    { { s_doap, 1, 0 }, { 0, 0 }, doap_repository, doap_hgrepository },
    { { s_doap, 1, 0 }, { 0, 0 }, doap_repository, doap_svnrepository },

    // data quality
    { { s_dqv, 1, 0 }, { 0, 0 }, dqv_qualityannotation, dqv_qualitycertificate },
    { { s_dqv, 1, 0 }, { 0, 0 }, dqv_qualityannotation, dqv_userqualityfeedback },

    // duv
    { { s_duv, 1, 0 }, { 0, 0 }, duv_userfeedback, duv_ratingfeedback },
    { { s_dqv, 1, 0 }, { 0, 0 }, duv_userfeedback, dqv_userqualityfeedback },

    // earl
    { { s_earl, 1, 0 }, { 0, 0 }, earl_assertor, foaf_agent },
    { { s_earl, 1, 0 }, { 0, 0 }, earl_assertor, foaf_group },
    { { s_earl, 1, 0 }, { 0, 0 }, earl_assertor, foaf_organisation },
    { { s_earl, 1, 0 }, { 0, 0 }, earl_assertor, foaf_person },
    { { s_earl, 1, 0 }, { 0, 0 }, earl_assertor, earl_software },

    { { s_earl, 1, 0 }, { 0, 0 }, earl_cannottell, earl_canttell },

    { { s_earl, 1, 0 }, { 0, 0 }, earl_fail, earl_failed },

    { { s_earl, 1, 0 }, { 0, 0 }, earl_pass, earl_passed },

    { { s_earl, 1, 0 }, { 0, 0 }, earl_notapplicable, earl_inapplicable },

    { { s_earl, 1, 0 }, { 0, 0 }, earl_nottested, earl_untested },

    { { s_earl, 1, 0 }, { 0, 0 }, earl_outcomevalue, earl_cannottell },
    { { s_earl, 1, 0 }, { 0, 0 }, earl_outcomevalue, earl_fail },
    { { s_earl, 1, 0 }, { 0, 0 }, earl_outcomevalue, earl_notapplicable },
    { { s_earl, 1, 0 }, { 0, 0 }, earl_outcomevalue, earl_nottested },
    { { s_earl, 1, 0 }, { 0, 0 }, earl_outcomevalue, earl_outcomevalue },
    { { s_earl, 1, 0 }, { 0, 0 }, earl_outcomevalue, earl_pass },

    { { s_earl, 1, 0 }, { 0, 0 }, earl_testcriterion, earl_testcase },
    { { s_earl, 1, 0 }, { 0, 0 }, earl_testcriterion, earl_testrequirement },

    { { s_earl, 1, 0 }, { 0, 0 }, earl_testmode, earl_automatic },
    { { s_earl, 1, 0 }, { 0, 0 }, earl_testmode, earl_manual },
    { { s_earl, 1, 0 }, { 0, 0 }, earl_testmode, earl_semiauto },
    { { s_earl, 1, 0 }, { 0, 0 }, earl_testmode, earl_undisclosed },
    { { s_earl, 1, 0 }, { 0, 0 }, earl_testmode, earl_unknownmode },

//    { { s_earl, 1, 0 }, { 0, 0 }, earl_testsubject, cnt_content },
    { { s_earl, 1, 0 }, { 0, 0 }, earl_testsubject, foaf_document },
//    { { s_earl, 1, 0 }, { 0, 0 }, earl_testsubject, http_response },
    { { s_earl, 1, 0 }, { 0, 0 }, earl_testsubject, earl_software },

    // event
    { { s_bibo, 1, 3 }, { 0, 0 }, event_event, bibo_event },

    // foaf
    { { s_foaf, 0, 99 }, { 0, 0 }, foaf_agent, foaf_group },
    { { s_odrl, 1, 0 }, { 0, 0 }, foaf_agent, odrl_party },
    { { s_foaf, 0, 99 }, { 0, 0 }, foaf_agent, foaf_person },
    { { s_foaf, 0, 99 }, { 0, 0 }, foaf_agent, foaf_organisation },
    { { s_org, 1, 0 }, { 0, 0 }, foaf_agent, org_organisation },

    { { s_void, 1, 0 }, { 0, 0 }, foaf_document, void_datasetdescription },
    { { s_foaf, 0, 99 }, { 0, 0 }, foaf_document, foaf_image },
    { { s_foaf, 0, 99 }, { 0, 0 }, foaf_document, foaf_personalprofiledocument },
    { { s_foaf, 0, 99 }, { 0, 0 }, foaf_document, sioc_item },

    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_onlineaccount, foaf_onlinechataccount,  },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_onlineaccount, foaf_onlineecommerceaccount },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_onlineaccount, foaf_onlinegamingaccount },
    { { s_foaf, 0, 99 }, { 0, 0 }, foaf_onlineaccount, sioc_user },

    { { s_odrl, 1, 0 }, { 0, 0 }, foaf_organisation, odrl_party },
    { { s_org, 1, 0 }, { 0, 0 }, foaf_organisation, org_formalorganisation },

    { { s_odrl, 1, 0 }, { 0, 0 }, foaf_person, odrl_party },

    { { s_doap, 1, 0 }, { 0, 0 }, foaf_project, doap_project },

    // frbr core
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_endeavour, fs_expression },
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_endeavour, fs_item },
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_endeavour, fs_manifestation },
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_endeavour, fs_object },
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_endeavour, fs_person },
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_endeavour, fs_work },

    { { s_frbr, 1, 0 }, { 0, 0 }, fs_responsibleentity, fs_corporatebody },

    { { s_frbr, 1, 0 }, { 0, 0 }, fs_subject, fs_event },
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_subject, fs_concept },
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_subject, fs_person },
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_subject, fs_place },

    // geo
//    { { s_geo, 1, 0 }, { 0, 0 }, fs_spatialthing, fs_object },
//    { { s_geo, 1, 0 }, { 0, 0 }, fs_spatialthing, fs_place },


    // good relations
    { { s_gr, 1, 0 }, { 0, 0 }, gr_businessentitytype, gr_business },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_businessentitytype, gr_enduser },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_businessentitytype, gr_publicinstitution },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_businessentitytype, gr_reseller },

    { { s_gr, 1, 0 }, { 0, 0 }, gr_businessfunction, gr_constructioninstallation },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_businessfunction, gr_dispose },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_businessfunction, gr_leaseout },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_businessfunction, gr_maintain },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_businessfunction, gr_provideservice },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_businessfunction, gr_repair },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_businessfunction, gr_sell },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_businessfunction, gr_buy },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_businessfunction, gr_licence },

    { { s_gr, 1, 0 }, { 0, 0 }, gr_dayofweek, gr_monday },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_dayofweek, gr_tuesday },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_dayofweek, gr_wednesday },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_dayofweek, gr_thursday },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_dayofweek, gr_friday },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_dayofweek, gr_saturday },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_dayofweek, gr_sunday },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_dayofweek, gr_publicholidays },

    { { s_gr, 1, 0 }, { 0, 0 }, gr_deliverymethod, gr_deliverymodeparcelservice },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_deliverymethod, gr_deliverymodedirectdownload },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_deliverymethod, gr_deliverymodefreight },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_deliverymethod, gr_deliverymodemail },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_deliverymethod, gr_deliverymodeownfleet },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_deliverymethod, gr_deliverymodepickup },

    { { s_gr, 1, 0 }, { 0, 0 }, gr_deliverymodeparcelservice, gr_dhl },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_deliverymodeparcelservice, gr_federalexpress },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_deliverymodeparcelservice, gr_ups },

    { { s_gr, 1, 0 }, { 0, 0 }, gr_paymentmethod, gr_bybanktransferinadvance },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_paymentmethod, gr_byinvoice },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_paymentmethod, gr_cash },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_paymentmethod, gr_checkinadvance },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_paymentmethod, gr_cod },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_paymentmethod, gr_directdebit },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_paymentmethod, gr_googlecheckout },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_paymentmethod, gr_paypal },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_paymentmethod, gr_payswarm },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_paymentmethod, gr_paymentmethodcreditcard },

    { { s_gr, 1, 0 }, { 0, 0 }, gr_paymentmethodcreditcard, gr_americanexpress },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_paymentmethodcreditcard, gr_dinersclub },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_paymentmethodcreditcard, gr_discover },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_paymentmethodcreditcard, gr_jcb },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_paymentmethodcreditcard, gr_mastercard },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_paymentmethodcreditcard, gr_visa },

    { { s_gr, 1, 0 }, { 0, 0 }, gr_pricespecification, gr_deliverychargespecification },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_pricespecification, gr_paymentchargespecification },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_pricespecification, gr_unitpricespecification },

    { { s_gr, 1, 0 }, { 0, 0 }, gr_productorservice, gr_individual },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_productorservice, gr_productorservicemodel },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_productorservice, gr_someitems },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_productorservice, gr_productorservicemodel },

    { { s_gr, 1, 0 }, { 0, 0 }, gr_quantitativevalue, gr_quantitativevaluefloat },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_quantitativevalue, gr_quantitativevalueinteger },

    { { s_gr, 1, 0 }, { 0, 0 }, gr_warrantyscope, gr_labour_bringin },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_warrantyscope, gr_partsandlabour_bringin },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_warrantyscope, gr_partsandlabour_pickup },

    // s_gs1
    { { s_gs1, 1, 0 }, { 0, 0 }, gs1s_levelofcontainmentcode, gs1s_levelofcontainmentcodecontains },
    { { s_gs1, 1, 0 }, { 0, 0 }, gs1s_levelofcontainmentcode, gs1s_levelofcontainmentcodefreefrom },
    { { s_gs1, 1, 0 }, { 0, 0 }, gs1s_levelofcontainmentcode, gs1s_levelofcontainmentcodemaycontain },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_13_bis_24_diaminophenoxypropane },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_1_naphthol },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_26_dimethoxy_35_pyridinediamine_hcl },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_2_hydroxyethyl_picramic_acid },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_2_methyl_5_hydroxyethylaminophenol },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_3_amino_24_dichlorophenol },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_3_aminophenol },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_4_amino_3_nitrophenol },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_4_hydroxy_propylamino_3_nitrophenol },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_abalone },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_almonds },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_alpha_isomethyl_ionone },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_amylcinnamyl_alcohol },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_amyl_cinnamal },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_anchovy },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_anise_alcohol },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_barley },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_barnacle },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_basa },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_bass },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_beech_nuts },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_beef },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_benzyl_alcohol },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_benzyl_benzoate },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_benzyl_cinnamate },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_benzyl_salicylate },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_bluefish },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_brazil_nuts },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_bream },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_butternuts },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_butylphenyl_methylpropionate },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_carp },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_carrots },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_cashew_nuts },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_catfish },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_celery },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_cereals_containing_gluten },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_char },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_chestnuts },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_chicken_meat },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_chinquapins },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_chub },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_cinnamal },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_cinnamyl_alcohol },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_cisco },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_citral },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_citronellol },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_clam },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_cockle },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_cocoa },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_coconuts },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_cod },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_conch },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_coriander },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_corn },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_cotton_seeds },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_coumarin },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_crab },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_crawfish },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_crustaceans },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_d_limonene },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_diaminophenols },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_eel },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_eggs },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_eugenol },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_evernia_furfuracea },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_evernia_prunastri },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_farnesol },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_fish },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_flounder },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_geraniol },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_ginkgo_nuts },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_glutamate },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_gluten },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_grouper },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_haddock },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_hake },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_halibut },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_hazelnuts },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_hc_blue_no_11 },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_hc_blue_no_12 },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_herring },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_hexyl_cinnamal },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_hickory_nuts },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_hydroxybenzomorpholine },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_hydroxycitronellal },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_hydroxyethyl_2_nitro_p_toluidine },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_hydroxyisohexyl_3_cyclohexene_carboxaldehyde_isoeugenol_limonene_linal },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_hydroxypropyl_bisn_hydroxyethyl_p_phenyldiamine_hcl },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_isoeugenol },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_kamut },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_krill },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_lactose },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_land },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_lichee_nuts },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_limpets },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_linalool },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_lobster },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_lupine },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_macadamia_nuts },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_mackerel },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_mahi_mahi },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_marlin },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_melatonin },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_methyl_2_octynoate },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_methyl_heptin_carbonate },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_milk },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_molluscs },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_monkfish_anglerfish_lotte },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_mussels },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_mustard },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_no_declared_allergens },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_oat },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_octopus },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_orange_roughy },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_oysters },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_p_methylaminophenol },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_p_phenylenediamine },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_peanuts },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_peas },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_pecan_nuts },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_perch },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_periwinkle },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_pike },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_pili_nuts },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_pine_nuts },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_pistachios },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_plaice },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_pod_fruits },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_pollock },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_pompano },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_poppy_seeds },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_porgy },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_pork },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_prawns },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_pulses },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_quahaugs },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_queensland_nuts },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_rockfish },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_rye },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_salicylate },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_salmon },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_sardine },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_scallops },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_seed_products },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_sesame_seeds },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_shark },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_shea_nuts },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_shellfish },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_shrimp },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_smelt },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_snapper },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_sole },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_soybeans },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_spelt },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_squid_calamari },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_sturgeon },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_sulphur_dioxide },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_sunflower_seeds },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_swordfish },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_tilapia },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_toluene_25_diamine },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_tree_nuts },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_tree_nut_traces },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_triticale },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_trout },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_tuna },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_turbot },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_walleye },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_walnuts },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_wheat },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_whelks },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_whitefish },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergentypecode, gs1s_allergentypecode_whiting },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_anatomicalformcode, gs1s_anatomicalformcode_belly },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_anatomicalformcode, gs1s_anatomicalformcode_blood },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_anatomicalformcode, gs1s_anatomicalformcode_bone },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_anatomicalformcode, gs1s_anatomicalformcode_brain },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_anatomicalformcode, gs1s_anatomicalformcode_carcass_nonpoultry },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_anatomicalformcode, gs1s_anatomicalformcode_diaphram },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_anatomicalformcode, gs1s_anatomicalformcode_esophagus },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_anatomicalformcode, gs1s_anatomicalformcode_fat },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_anatomicalformcode, gs1s_anatomicalformcode_feet },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_anatomicalformcode, gs1s_anatomicalformcode_forequarter },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_anatomicalformcode, gs1s_anatomicalformcode_gizzard },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_anatomicalformcode, gs1s_anatomicalformcode_head },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_anatomicalformcode, gs1s_anatomicalformcode_headmeat_cheekmeat },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_anatomicalformcode, gs1s_anatomicalformcode_heart },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_anatomicalformcode, gs1s_anatomicalformcode_hindquarter },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_anatomicalformcode, gs1s_anatomicalformcode_horn },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_anatomicalformcode, gs1s_anatomicalformcode_intestine_large },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_anatomicalformcode, gs1s_anatomicalformcode_intestine_small },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_anatomicalformcode, gs1s_anatomicalformcode_kidney },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_anatomicalformcode, gs1s_anatomicalformcode_lips },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_anatomicalformcode, gs1s_anatomicalformcode_liver },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_anatomicalformcode, gs1s_anatomicalformcode_lungs },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_anatomicalformcode, gs1s_anatomicalformcode_mixed_offal },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_anatomicalformcode, gs1s_anatomicalformcode_mixed_parts_for_grounding_mincing },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_anatomicalformcode, gs1s_anatomicalformcode_neck },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_anatomicalformcode, gs1s_anatomicalformcode_pancreas },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_anatomicalformcode, gs1s_anatomicalformcode_skin },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_anatomicalformcode, gs1s_anatomicalformcode_stomach },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_anatomicalformcode, gs1s_anatomicalformcode_tail },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_anatomicalformcode, gs1s_anatomicalformcode_tendons },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_anatomicalformcode, gs1s_anatomicalformcode_testicles },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_anatomicalformcode, gs1s_anatomicalformcode_thymus },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_anatomicalformcode, gs1s_anatomicalformcode_tongue },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_anatomicalformcode, gs1s_anatomicalformcode_tripe },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_anatomicalformcode, gs1s_anatomicalformcode_unclassified },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_anatomicalformcode, gs1s_anatomicalformcode_unidentified },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_anatomicalformcode, gs1s_anatomicalformcode_whole_bird },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_anatomicalformcode, gs1s_anatomicalformcode_whole_muscle_nonpoultry_meatpart_piece },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_anatomicalformcode, gs1s_anatomicalformcode_whole_muscle_poultry_part },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_anatomicalformcode, gs1s_anatomicalformcode_whole_muscle_primal },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_anatomicalformcode, gs1s_anatomicalformcode_whole_muscle_ribcut },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_anatomicalformcode, gs1s_anatomicalformcode_whole_muscle_roastcut },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_anatomicalformcode, gs1s_anatomicalformcode_whole_muscle_slice_chopcut },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_anatomicalformcode, gs1s_anatomicalformcode_whole_muscle_slice_cutlet },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_anatomicalformcode, gs1s_anatomicalformcode_whole_muscle_steak_chopcut },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_anatomicalformcode, gs1s_anatomicalformcode_whole_muscle_subprimal },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_cheesefirmnesscode, gs1s_cheesefirmnesscode_extra_hard },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_cheesefirmnesscode, gs1s_cheesefirmnesscode_firm_semihard },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_cheesefirmnesscode, gs1s_cheesefirmnesscode_hard },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_cheesefirmnesscode, gs1s_cheesefirmnesscode_soft },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_cheesefirmnesscode, gs1s_cheesefirmnesscode_unidentified },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_colourcodelistcode, gs1s_colourcodelist_buyer },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_colourcodelistcode, gs1s_colourcodelist_ifps },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_colourcodelistcode, gs1s_colourcodelist_ncs },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_colourcodelistcode, gs1s_colourcodelist_nrf },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_colourcodelistcode, gs1s_colourcodelist_pantone_hexachrome },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_colourcodelistcode, gs1s_colourcodelist_pantone_matching },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_colourcodelistcode, gs1s_colourcodelist_pantone_process_colour },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_colourcodelistcode, gs1s_colourcodelist_pantone_textile },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_colourcodelistcode, gs1s_colourcodelist_ral },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_colourcodelistcode, gs1s_colourcodelist_seller },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_colourcodelistcode, gs1s_colourcodelist_wws },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_consumerlifestagecode, gs1s_consumerlifestagecode_adult },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_consumerlifestagecode, gs1s_consumerlifestagecode_all_ages },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_consumerlifestagecode, gs1s_consumerlifestagecode_baby_infant },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_consumerlifestagecode, gs1s_consumerlifestagecode_child_1_to_2_years },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_consumerlifestagecode, gs1s_consumerlifestagecode_child_2_years_onwards },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_consumerlifestagecode, gs1s_consumerlifestagecode_unclassified },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_consumerlifestagecode, gs1s_consumerlifestagecode_unidentified },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_consumersalesconditionscode, gs1s_consumersalesconditionscode_btc },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_consumersalesconditionscode, gs1s_consumersalesconditionscode_otc },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_consumersalesconditionscode, gs1s_consumersalesconditionscode_prescription_required },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_consumersalesconditionscode, gs1s_consumersalesconditionscode_restricted_to_sell_16 },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_consumersalesconditionscode, gs1s_consumersalesconditionscode_restricted_to_sell_18 },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_consumersalesconditionscode, gs1s_consumersalesconditionscode_restricted_to_sell_21 },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_diettypecode, gs1s_diettypecode_coeliac },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_diettypecode, gs1s_diettypecode_free_from_gluten },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_diettypecode, gs1s_diettypecode_halal },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_diettypecode, gs1s_diettypecode_kosher },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_diettypecode, gs1s_diettypecode_organic },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_diettypecode, gs1s_diettypecode_vegan },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_diettypecode, gs1s_diettypecode_vegetarian },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_diettypecode, gs1s_diettypecode_without_beef },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_diettypecode, gs1s_diettypecode_without_pork },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_discounttypecode, gs1s_discounttypecode_bogo },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_discounttypecode, gs1s_discounttypecode_discounted_item },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_discounttypecode, gs1s_discounttypecode_free_gift },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_discounttypecode, gs1s_discounttypecode_free_shipping },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_discounttypecode, gs1s_discounttypecode_other },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_discounttypecode, gs1s_discounttypecode_percentage_off },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragerefrigerationclaimcode, gs1s_foodbeveragerefrigerationclaimcode_can_be_refrigerated },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragerefrigerationclaimcode, gs1s_foodbeveragerefrigerationclaimcode_must_be_refrigerated },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragerefrigerationclaimcode, gs1s_foodbeveragerefrigerationclaimcode_shelf_stable },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragerefrigerationclaimcode, gs1s_foodbeveragerefrigerationclaimcode_unidentified },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetargetusecode, gs1s_foodbeveragetargetusecode_any_meal },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetargetusecode, gs1s_foodbeveragetargetusecode_breakfast },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetargetusecode, gs1s_foodbeveragetargetusecode_main_meal },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetargetusecode, gs1s_foodbeveragetargetusecode_portable_meal },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetargetusecode, gs1s_foodbeveragetargetusecode_snack },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetargetusecode, gs1s_foodbeveragetargetusecode_unclassified },

    { { s_gs1, 1, 0 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1s_beverage },
    { { s_gs1, 1, 0 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1s_fruitsvegetables },
    { { s_gs1, 1, 0 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1s_meatpoultry },
    { { s_gs1, 1, 0 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1s_milkbuttercreamyogurtcheeseeggssubstitutes },
    { { s_gs1, 1, 0 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1s_seafood },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_footwearfasteningtypecode, gs1s_footwearfasteningtypecode_multiple_fastening },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_footwearfasteningtypecode, gs1s_footwearfasteningtypecode_shoe_lace },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_footwearfasteningtypecode, gs1s_footwearfasteningtypecode_slip_on_without_elastic },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_footwearfasteningtypecode, gs1s_footwearfasteningtypecode_slip_on_with_elastic },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_footwearfasteningtypecode, gs1s_footwearfasteningtypecode_strap },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_footwearfasteningtypecode, gs1s_footwearfasteningtypecode_unclassified },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_footwearfasteningtypecode, gs1s_footwearfasteningtypecode_unidentified },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_footwearfasteningtypecode, gs1s_footwearfasteningtypecode_velcro },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_freshorseawaterfarmedcode, gs1s_freshorseawaterfarmedcode_freshwater_farmed },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_freshorseawaterfarmedcode, gs1s_freshorseawaterfarmedcode_seawater_farmed },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_freshorseawaterfarmedcode, gs1s_freshorseawaterfarmedcode_unclassified },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_freshorseawaterfarmedcode, gs1s_freshorseawaterfarmedcode_unidentified },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_growingmethodcode, gs1s_growingmethodcode_cage_free },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_growingmethodcode, gs1s_growingmethodcode_cloned_foods },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_growingmethodcode, gs1s_growingmethodcode_conventional },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_growingmethodcode, gs1s_growingmethodcode_farm_raised },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_growingmethodcode, gs1s_growingmethodcode_field_grown },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_growingmethodcode, gs1s_growingmethodcode_fly_free },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_growingmethodcode, gs1s_growingmethodcode_free_range },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_growingmethodcode, gs1s_growingmethodcode_grass_fed },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_growingmethodcode, gs1s_growingmethodcode_greenhouse },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_growingmethodcode, gs1s_growingmethodcode_humanely_raised },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_growingmethodcode, gs1s_growingmethodcode_hydroponic },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_growingmethodcode, gs1s_growingmethodcode_integrated_pest_management },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_growingmethodcode, gs1s_growingmethodcode_organic },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_growingmethodcode, gs1s_growingmethodcode_shade_grown },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_growingmethodcode, gs1s_growingmethodcode_sustainable },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_growingmethodcode, gs1s_growingmethodcode_wild },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_levelofcontainmentcode, gs1s_levelofcontainmentcodecontains },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_levelofcontainmentcode, gs1s_levelofcontainmentcodefreefrom },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_levelofcontainmentcode, gs1s_levelofcontainmentcodemaycontain },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_maturationmethodcode, gs1s_maturationmethodcode_jet_fresh },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_maturationmethodcode, gs1s_maturationmethodcode_preconditioned },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_maturationmethodcode, gs1s_maturationmethodcode_tree_vine_ripe },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_measurementprecisioncode, gs1s_measurementprecisioncode_approximately },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_measurementprecisioncode, gs1s_measurementprecisioncode_exact },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_measurementprecisioncode, gs1s_measurementprecisioncode_less_than },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_nonbinarylogiccode, gs1s_nonbinarylogiccode_false },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_nonbinarylogiccode, gs1s_nonbinarylogiccode_na },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_nonbinarylogiccode, gs1s_nonbinarylogiccode_true },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_nonbinarylogiccode, gs1s_nonbinarylogiccode_unspecified },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_nutrientbasisquantitycode, gs1s_nutrientbasisquantitycode_by_measure },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_nutrientbasisquantitycode, gs1s_nutrientbasisquantitycode_by_serving },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_nutritionalclaimtypecode, gs1s_nutritionalclaimtypecode_additive_free },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_nutritionalclaimtypecode, gs1s_nutritionalclaimtypecode_artificially_sweetened },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_nutritionalclaimtypecode, gs1s_nutritionalclaimtypecode_cholesterol_free },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_nutritionalclaimtypecode, gs1s_nutritionalclaimtypecode_colouring_agent_free },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_nutritionalclaimtypecode, gs1s_nutritionalclaimtypecode_contains_glyzyrrhizin },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_nutritionalclaimtypecode, gs1s_nutritionalclaimtypecode_contains_liquorice },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_nutritionalclaimtypecode, gs1s_nutritionalclaimtypecode_contains_soy },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_nutritionalclaimtypecode, gs1s_nutritionalclaimtypecode_egg_free },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_nutritionalclaimtypecode, gs1s_nutritionalclaimtypecode_energy_free },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_nutritionalclaimtypecode, gs1s_nutritionalclaimtypecode_energy_reduced },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_nutritionalclaimtypecode, gs1s_nutritionalclaimtypecode_enriched_or_fortified_in_vitamins_and_or_minerals },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_nutritionalclaimtypecode, gs1s_nutritionalclaimtypecode_fat_free },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_nutritionalclaimtypecode, gs1s_nutritionalclaimtypecode_free_from_gluten },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_nutritionalclaimtypecode, gs1s_nutritionalclaimtypecode_guaranteed_lactose_free },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_nutritionalclaimtypecode, gs1s_nutritionalclaimtypecode_high_fibre },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_nutritionalclaimtypecode, gs1s_nutritionalclaimtypecode_high_protein },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_nutritionalclaimtypecode, gs1s_nutritionalclaimtypecode_high_vitamins_and_or_minerals },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_nutritionalclaimtypecode, gs1s_nutritionalclaimtypecode_lactose_free },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_nutritionalclaimtypecode, gs1s_nutritionalclaimtypecode_light_lite },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_nutritionalclaimtypecode, gs1s_nutritionalclaimtypecode_low_energy },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_nutritionalclaimtypecode, gs1s_nutritionalclaimtypecode_low_fat },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_nutritionalclaimtypecode, gs1s_nutritionalclaimtypecode_low_lactose },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_nutritionalclaimtypecode, gs1s_nutritionalclaimtypecode_low_protein },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_nutritionalclaimtypecode, gs1s_nutritionalclaimtypecode_low_saturated_fat },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_nutritionalclaimtypecode, gs1s_nutritionalclaimtypecode_low_sodium_salt },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_nutritionalclaimtypecode, gs1s_nutritionalclaimtypecode_low_sugars },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_nutritionalclaimtypecode, gs1s_nutritionalclaimtypecode_milk_free },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_nutritionalclaimtypecode, gs1s_nutritionalclaimtypecode_milk_protein_free },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_nutritionalclaimtypecode, gs1s_nutritionalclaimtypecode_natural_source_of_vitamins_and_or_minerals },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_nutritionalclaimtypecode, gs1s_nutritionalclaimtypecode_non_alcoholic },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_nutritionalclaimtypecode, gs1s_nutritionalclaimtypecode_nut_free },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_nutritionalclaimtypecode, gs1s_nutritionalclaimtypecode_peanut_free },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_nutritionalclaimtypecode, gs1s_nutritionalclaimtypecode_preservative_free },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_nutritionalclaimtypecode, gs1s_nutritionalclaimtypecode_saturated_fat_free },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_nutritionalclaimtypecode, gs1s_nutritionalclaimtypecode_sodium_free_or_salt_free },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_nutritionalclaimtypecode, gs1s_nutritionalclaimtypecode_source_of_fibre },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_nutritionalclaimtypecode, gs1s_nutritionalclaimtypecode_source_of_protein },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_nutritionalclaimtypecode, gs1s_nutritionalclaimtypecode_soy_free },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_nutritionalclaimtypecode, gs1s_nutritionalclaimtypecode_strongly_salted },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_nutritionalclaimtypecode, gs1s_nutritionalclaimtypecode_sugars_free },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_nutritionalclaimtypecode, gs1s_nutritionalclaimtypecode_sweetened_with_agave_syrup },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_nutritionalclaimtypecode, gs1s_nutritionalclaimtypecode_sweetened_with_cane_sugar },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_nutritionalclaimtypecode, gs1s_nutritionalclaimtypecode_sweetened_with_corn_syrup },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_nutritionalclaimtypecode, gs1s_nutritionalclaimtypecode_sweetened_with_fructose },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_nutritionalclaimtypecode, gs1s_nutritionalclaimtypecode_sweetened_with_fruit_juice },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_nutritionalclaimtypecode, gs1s_nutritionalclaimtypecode_sweetened_with_fruit_syrup },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_nutritionalclaimtypecode, gs1s_nutritionalclaimtypecode_sweetened_with_honey },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_nutritionalclaimtypecode, gs1s_nutritionalclaimtypecode_sweetened_with_malt },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_nutritionalclaimtypecode, gs1s_nutritionalclaimtypecode_sweetened_with_raw_beet_sugar },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_nutritionalclaimtypecode, gs1s_nutritionalclaimtypecode_sweetened_with_white_sugar },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_nutritionalclaimtypecode, gs1s_nutritionalclaimtypecode_very_low_gluten },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_nutritionalclaimtypecode, gs1s_nutritionalclaimtypecode_very_low_sodium_salt },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_nutritionalclaimtypecode, gs1s_nutritionalclaimtypecode_wheat_free },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_nutritionalclaimtypecode, gs1s_nutritionalclaimtypecode_with_no_added_sugars },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_offerredemptiontypecode, gs1s_offerredemptiontypecode_coupon_code },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_offerredemptiontypecode, gs1s_offerredemptiontypecode_gift_card },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_offerredemptiontypecode, gs1s_offerredemptiontypecode_rebate },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_offerredemptiontypecode, gs1s_offerredemptiontypecode_sale },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_offerredemptiontypecode, gs1s_offerredemptiontypecode_single_use_code },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organicclaimagencycode, gs1s_organicclaimagencycode_bfa },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organicclaimagencycode, gs1s_organicclaimagencycode_bioland },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organicclaimagencycode, gs1s_organicclaimagencycode_demeter_bund },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organicclaimagencycode, gs1s_organicclaimagencycode_demeter_international },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organicclaimagencycode, gs1s_organicclaimagencycode_ecoland },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organicclaimagencycode, gs1s_organicclaimagencycode_ecovin },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organicclaimagencycode, gs1s_organicclaimagencycode_epa },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organicclaimagencycode, gs1s_organicclaimagencycode_fda },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organicclaimagencycode, gs1s_organicclaimagencycode_fsa },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organicclaimagencycode, gs1s_organicclaimagencycode_gaa },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organicclaimagencycode, gs1s_organicclaimagencycode_ifoa },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organicclaimagencycode, gs1s_organicclaimagencycode_ifoam },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organicclaimagencycode, gs1s_organicclaimagencycode_naturland },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organicclaimagencycode, gs1s_organicclaimagencycode_qai },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organicclaimagencycode, gs1s_organicclaimagencycode_suditirol },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organicclaimagencycode, gs1s_organicclaimagencycode_usda },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_accepting_party },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_bill_of_lading_recipient },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_bill_to },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_brand_owner },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_breeder },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_broker_agent },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_buyer },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_buyers_agent_representative },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_carrier },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_central_payment_service },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_checking_party },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_cheque_order },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_consignee },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_consolidator },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_consumer },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_corporate_identity },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_customs },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_customs_broker },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_cutter },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_declarants_agent_representative },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_delivery_party },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_designer },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_despatch_party },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_distributor },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_dock_door },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_empty_equipment_despatch_party },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_empty_equipment_return_party },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_equipment_owner },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_exporter },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_factor },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_fattener },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_freight_forwarder },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_goods_owner },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_grower },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_head_office },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_importer },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_information_provider },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_insurer },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_intermediary_bank_1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_intermediary_bank_2 },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_inventory_controller },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_inventory_reporting_party },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_invoicee },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_issuer_of_invoice },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_logistics_service_provider },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_manufacturer_of_goods },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_manufacturing_plant },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_mark_for },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_message_from },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_message_recipient },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_mincer },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_operating_division },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_operator },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_ordering_party },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_owner_of_equipment },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_owner_of_means_of_transport },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_party_declaring_the_value_added_tax },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_party_for_whom_item_is_ultimately_intended },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_party_receiving_private_data },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_party_recovering_the_value_added_tax },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_party_to_receive_all_documents },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_party_to_receive_commercial_invoice },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_party_to_receive_electronic_memo_of_invoice },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_party_to_receive_freight_bill },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_party_to_receive_refund },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_payee },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_payer },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_point_of_sale },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_price_location_party },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_proxy },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_public_warehouse },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_purchase_order_receiver },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_registered_agent },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_remit_to },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_returnable_asset_service_centre },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_seller },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_service_provider },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_ship_from },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_ship_to },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_slaughterer },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_store },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_substitute_supplier },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_supplier },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_transportation_carrier },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_visiting_address },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_warehouse_and_or_depot },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_warehouse_keeper },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisationroletype, gs1s_organisationroletype_wholesaler },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingfeaturecode, gs1s_packagingfeaturecode_base },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingfeaturecode, gs1s_packagingfeaturecode_beam },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingfeaturecode, gs1s_packagingfeaturecode_bung_seal },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingfeaturecode, gs1s_packagingfeaturecode_cap },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingfeaturecode, gs1s_packagingfeaturecode_carrier },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingfeaturecode, gs1s_packagingfeaturecode_consumption_utensil },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingfeaturecode, gs1s_packagingfeaturecode_core },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingfeaturecode, gs1s_packagingfeaturecode_creel },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingfeaturecode, gs1s_packagingfeaturecode_edge_protection },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingfeaturecode, gs1s_packagingfeaturecode_handle },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingfeaturecode, gs1s_packagingfeaturecode_inner_container },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingfeaturecode, gs1s_packagingfeaturecode_internal_divider },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingfeaturecode, gs1s_packagingfeaturecode_label },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingfeaturecode, gs1s_packagingfeaturecode_lid },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingfeaturecode, gs1s_packagingfeaturecode_liner },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingfeaturecode, gs1s_packagingfeaturecode_lug },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingfeaturecode, gs1s_packagingfeaturecode_nesting_edge },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingfeaturecode, gs1s_packagingfeaturecode_peg },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingfeaturecode, gs1s_packagingfeaturecode_pull_off_tab },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingfeaturecode, gs1s_packagingfeaturecode_ring_holder },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingfeaturecode, gs1s_packagingfeaturecode_rivet },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingfeaturecode, gs1s_packagingfeaturecode_sleeve },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingfeaturecode, gs1s_packagingfeaturecode_spout },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingfeaturecode, gs1s_packagingfeaturecode_tag },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingfeaturecode, gs1s_packagingfeaturecode_wicker_outer_container },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingfeaturecode, gs1s_packagingfeaturecode_wrap },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingfunctioncode, gs1s_packagingfunctioncode_antiseptic },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingfunctioncode, gs1s_packagingfunctioncode_anti_tampering },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingfunctioncode, gs1s_packagingfunctioncode_coated },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingfunctioncode, gs1s_packagingfunctioncode_compressed },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingfunctioncode, gs1s_packagingfunctioncode_dispenser },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingfunctioncode, gs1s_packagingfunctioncode_gift_wrapped },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingfunctioncode, gs1s_packagingfunctioncode_isothermic },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingfunctioncode, gs1s_packagingfunctioncode_modified_atmosphere },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingfunctioncode, gs1s_packagingfunctioncode_oxygen_infused },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingfunctioncode, gs1s_packagingfunctioncode_peel_off },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingfunctioncode, gs1s_packagingfunctioncode_pinpack },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingfunctioncode, gs1s_packagingfunctioncode_protected },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingfunctioncode, gs1s_packagingfunctioncode_reinforced },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingfunctioncode, gs1s_packagingfunctioncode_sift_proof },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingfunctioncode, gs1s_packagingfunctioncode_tamper_evident },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingfunctioncode, gs1s_packagingfunctioncode_water_resistant },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkeddietallergencode, gs1s_packagingmarkeddietallergencode_approved_by_asthma_and_allergy_assoc },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkeddietallergencode, gs1s_packagingmarkeddietallergencode_approved_for_tube_feeding },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkeddietallergencode, gs1s_packagingmarkeddietallergencode_biological },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkeddietallergencode, gs1s_packagingmarkeddietallergencode_calories_per_portion },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkeddietallergencode, gs1s_packagingmarkeddietallergencode_contains_latex },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkeddietallergencode, gs1s_packagingmarkeddietallergencode_contains_pvc },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkeddietallergencode, gs1s_packagingmarkeddietallergencode_contains_pvc_without_phthalates },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkeddietallergencode, gs1s_packagingmarkeddietallergencode_contains_pvc_with_phthalates },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkeddietallergencode, gs1s_packagingmarkeddietallergencode_diet_product_450_800_kcal_per_day },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkeddietallergencode, gs1s_packagingmarkeddietallergencode_diet_product_800_1200_kcal_per_day },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkeddietallergencode, gs1s_packagingmarkeddietallergencode_gezonde_keuze_klavertje },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkeddietallergencode, gs1s_packagingmarkeddietallergencode_halal },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkeddietallergencode, gs1s_packagingmarkeddietallergencode_ik_kies_bewust },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkeddietallergencode, gs1s_packagingmarkeddietallergencode_kosher },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkeddietallergencode, gs1s_packagingmarkeddietallergencode_lactase_enzyme },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkeddietallergencode, gs1s_packagingmarkeddietallergencode_low_on_phenylalanine },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkeddietallergencode, gs1s_packagingmarkeddietallergencode_low_on_sugar },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkeddietallergencode, gs1s_packagingmarkeddietallergencode_mothers_milk_substitute },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkeddietallergencode, gs1s_packagingmarkeddietallergencode_nutrition_supplement },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkeddietallergencode, gs1s_packagingmarkeddietallergencode_nyckelhal_mark },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkeddietallergencode, gs1s_packagingmarkeddietallergencode_vegetarian },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedfreefromcode_free_from_artificial_colouring },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedfreefromcode_free_from_artificial_flavouring },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedfreefromcode_free_from_artificial_preservatives },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedfreefromcode_free_from_cholesterol },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedfreefromcode_free_from_cloned_food },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedfreefromcode_free_from_dairy },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedfreefromcode_free_from_egg },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedfreefromcode_free_from_fish },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedfreefromcode_free_from_gluten },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedfreefromcode_free_from_lactose },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedfreefromcode_free_from_latex },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedfreefromcode_free_from_legume_protein },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedfreefromcode_free_from_milk },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedfreefromcode_free_from_milk_protein },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedfreefromcode_free_from_natural_gluten },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedfreefromcode_free_from_peanuts },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedfreefromcode_free_from_protein },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedfreefromcode_free_from_pvc },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedfreefromcode_free_from_soya },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedfreefromcode_free_from_sugar },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedfreefromcode_free_from_transfat },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedfreefromcode_reduced_lactose },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedfreefromcode_reduced_protein },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedfreefromcode_reduced_transfat },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedfreefromcode_very_low_gluten },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedfreefromcode_without_added_salt },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedfreefromcode_without_added_sugar },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedfreefromcode_without_added_sweetener },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedlabelaccreditationcode_ama_organic_seal },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedlabelaccreditationcode_aus_kaup_estonia },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedlabelaccreditationcode_bdih_logo },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedlabelaccreditationcode_beter_leven_1_ster },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedlabelaccreditationcode_beter_leven_2_ster },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedlabelaccreditationcode_beter_leven_3_ster },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedlabelaccreditationcode_bio_austria_label },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedlabelaccreditationcode_bio_label_german },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedlabelaccreditationcode_bio_suisse_bud_seal },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedlabelaccreditationcode_blue_angel },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedlabelaccreditationcode_bord_bia_approved },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedlabelaccreditationcode_bord_bia_approved_meat },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedlabelaccreditationcode_bra_miljoval_label_swedish },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedlabelaccreditationcode_crossed_grain_symbol },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedlabelaccreditationcode_demeter_label },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedlabelaccreditationcode_ecocert_certificate },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedlabelaccreditationcode_eco_label_ladybug },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedlabelaccreditationcode_ec_national_health_mark },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedlabelaccreditationcode_eesti_okomark },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedlabelaccreditationcode_eesti_parim_toiduaine },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedlabelaccreditationcode_eko },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedlabelaccreditationcode_eu_eco_label },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedlabelaccreditationcode_eu_organic_farming },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedlabelaccreditationcode_fair_trade_mark },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedlabelaccreditationcode_falken },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedlabelaccreditationcode_finnish_heart_symbol },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedlabelaccreditationcode_forest_stewardship_council_label },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedlabelaccreditationcode_gmo_marked },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedlabelaccreditationcode_goods_from_finland_blue_swan },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedlabelaccreditationcode_green_dot },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedlabelaccreditationcode_green_restaurant_association_endorsed },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedlabelaccreditationcode_green_seal_certified },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedlabelaccreditationcode_guaranteed_irish },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedlabelaccreditationcode_krav_mark },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedlabelaccreditationcode_label_of_the_allergy_and_asthma_federation },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedlabelaccreditationcode_leaping_bunny },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedlabelaccreditationcode_loodussobralik_toode_estonia },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedlabelaccreditationcode_love_irish_food },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedlabelaccreditationcode_made_in_finland_flag_with_key },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedlabelaccreditationcode_marine_stewardship_council_label },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedlabelaccreditationcode_max_havelaar },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedlabelaccreditationcode_natrue_label },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedlabelaccreditationcode_nyckelhalet },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedlabelaccreditationcode_oeko_tex_label },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedlabelaccreditationcode_official_eco_label_sun },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedlabelaccreditationcode_pefc },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedlabelaccreditationcode_protected_designation_of_origin },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedlabelaccreditationcode_protected_geographical_indication },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedlabelaccreditationcode_protected_harvest_certified },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedlabelaccreditationcode_quality_mark_ireland },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedlabelaccreditationcode_rainforest_alliance },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedlabelaccreditationcode_scharrelvlees },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedlabelaccreditationcode_soil_association_organic_symbol },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedlabelaccreditationcode_sustainable_palm_oil_rspo },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedlabelaccreditationcode_svanen },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedlabelaccreditationcode_swedish_seal_of_quality },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedlabelaccreditationcode_traditional_specialty_guaranteed },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedlabelaccreditationcode_tunnustatud_eesti_maitse },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedlabelaccreditationcode_tunnustatud_maitse },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedlabelaccreditationcode_uniquely_finnish },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedlabelaccreditationcode_usda },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedlabelaccreditationcode_utz_certified },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedlabelaccreditationcode_vaelg_fuldkorn_forst },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmarkedfreefromcode, gs1s_packagingmarkedlabelaccreditationcode_wwf_panda_label },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialtypecode, gs1s_packagingmaterialtypecode_ceramic },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialtypecode, gs1s_packagingmaterialtypecode_cloth_or_fabric },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialtypecode, gs1s_packagingmaterialtypecode_composite },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialtypecode, gs1s_packagingmaterialtypecode_corrugated_board_double_wall },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialtypecode, gs1s_packagingmaterialtypecode_corrugated_board_other },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialtypecode, gs1s_packagingmaterialtypecode_corrugated_board_single_wall },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialtypecode, gs1s_packagingmaterialtypecode_corrugated_board_triple_wall },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialtypecode, gs1s_packagingmaterialtypecode_fibre_burlap },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialtypecode, gs1s_packagingmaterialtypecode_fibre_cotton },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialtypecode, gs1s_packagingmaterialtypecode_fibre_flax },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialtypecode, gs1s_packagingmaterialtypecode_fibre_hemp },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialtypecode, gs1s_packagingmaterialtypecode_fibre_jute },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialtypecode, gs1s_packagingmaterialtypecode_fibre_other },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialtypecode, gs1s_packagingmaterialtypecode_foam },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialtypecode, gs1s_packagingmaterialtypecode_glass },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialtypecode, gs1s_packagingmaterialtypecode_glass_coloured },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialtypecode, gs1s_packagingmaterialtypecode_laminated_carton },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialtypecode, gs1s_packagingmaterialtypecode_metal_aluminum },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialtypecode, gs1s_packagingmaterialtypecode_metal_brass },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialtypecode, gs1s_packagingmaterialtypecode_metal_iron },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialtypecode, gs1s_packagingmaterialtypecode_metal_lead },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialtypecode, gs1s_packagingmaterialtypecode_metal_other },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialtypecode, gs1s_packagingmaterialtypecode_metal_stainless_steel },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialtypecode, gs1s_packagingmaterialtypecode_metal_steel },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialtypecode, gs1s_packagingmaterialtypecode_metal_tin },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialtypecode, gs1s_packagingmaterialtypecode_mineral_calcium_carbonate },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialtypecode, gs1s_packagingmaterialtypecode_mineral_other },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialtypecode, gs1s_packagingmaterialtypecode_mineral_talc },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialtypecode, gs1s_packagingmaterialtypecode_natural_rubber },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialtypecode, gs1s_packagingmaterialtypecode_other },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialtypecode, gs1s_packagingmaterialtypecode_paper_corrugated },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialtypecode, gs1s_packagingmaterialtypecode_paper_molded_pulp },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialtypecode, gs1s_packagingmaterialtypecode_paper_other },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialtypecode, gs1s_packagingmaterialtypecode_paper_paper },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialtypecode, gs1s_packagingmaterialtypecode_paper_paperboard },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialtypecode, gs1s_packagingmaterialtypecode_paper_rayon },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialtypecode, gs1s_packagingmaterialtypecode_plastic_bioplastic },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialtypecode, gs1s_packagingmaterialtypecode_plastic_other },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialtypecode, gs1s_packagingmaterialtypecode_plastic_thermoplastics },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialtypecode, gs1s_packagingmaterialtypecode_polymer_cellulose_acetate },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialtypecode, gs1s_packagingmaterialtypecode_polymer_epoxy },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialtypecode, gs1s_packagingmaterialtypecode_polymer_eva },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialtypecode, gs1s_packagingmaterialtypecode_polymer_evoh },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialtypecode, gs1s_packagingmaterialtypecode_polymer_hdpe },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialtypecode, gs1s_packagingmaterialtypecode_polymer_ldpe },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialtypecode, gs1s_packagingmaterialtypecode_polymer_lldpe },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialtypecode, gs1s_packagingmaterialtypecode_polymer_mdpe },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialtypecode, gs1s_packagingmaterialtypecode_polymer_nylon },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialtypecode, gs1s_packagingmaterialtypecode_polymer_other },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialtypecode, gs1s_packagingmaterialtypecode_polymer_pan },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialtypecode, gs1s_packagingmaterialtypecode_polymer_pc },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialtypecode, gs1s_packagingmaterialtypecode_polymer_pcl },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialtypecode, gs1s_packagingmaterialtypecode_polymer_pe },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialtypecode, gs1s_packagingmaterialtypecode_polymer_pen },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialtypecode, gs1s_packagingmaterialtypecode_polymer_pet },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialtypecode, gs1s_packagingmaterialtypecode_polymer_pha },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialtypecode, gs1s_packagingmaterialtypecode_polymer_pla },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialtypecode, gs1s_packagingmaterialtypecode_polymer_pp },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialtypecode, gs1s_packagingmaterialtypecode_polymer_ps },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialtypecode, gs1s_packagingmaterialtypecode_polymer_pu },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialtypecode, gs1s_packagingmaterialtypecode_polymer_pva },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialtypecode, gs1s_packagingmaterialtypecode_polymer_pvc },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialtypecode, gs1s_packagingmaterialtypecode_polymer_pvdc },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialtypecode, gs1s_packagingmaterialtypecode_polymer_tps },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingrecyclingprocesstypecode, gs1s_packagingrecyclingprocesstypecode_compostable },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingrecyclingprocesstypecode, gs1s_packagingrecyclingprocesstypecode_energy_recoverable },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingrecyclingprocesstypecode, gs1s_packagingrecyclingprocesstypecode_recyclable },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingrecyclingprocesstypecode, gs1s_packagingrecyclingprocesstypecode_reusable },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingrecyclingschemecode, gs1s_packagingrecyclingschemecode_alko },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingrecyclingschemecode, gs1s_packagingrecyclingschemecode_a_pullo },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingrecyclingschemecode, gs1s_packagingrecyclingschemecode_eko_pullo },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingrecyclingschemecode, gs1s_packagingrecyclingschemecode_palpa },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingshapecode, gs1s_packagingshapecode_bar },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingshapecode, gs1s_packagingshapecode_coil },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingshapecode, gs1s_packagingshapecode_cone },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingshapecode, gs1s_packagingshapecode_cylindrical },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingshapecode, gs1s_packagingshapecode_polygon },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingshapecode, gs1s_packagingshapecode_rectangular },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingshapecode, gs1s_packagingshapecode_spherical },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingshapecode, gs1s_packagingshapecode_tablet },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingshapecode, gs1s_packagingshapecode_unspecified },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_paymentmethod, gs1s_paymentmethod_bankers_draft },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_paymentmethod, gs1s_paymentmethod_bank_cheque },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_paymentmethod, gs1s_paymentmethod_bank_giro },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_paymentmethod, gs1s_paymentmethod_bookentry_credit },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_paymentmethod, gs1s_paymentmethod_bookentry_debit },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_paymentmethod, gs1s_paymentmethod_bop },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_paymentmethod, gs1s_paymentmethod_cash },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_paymentmethod, gs1s_paymentmethod_certified_cheque },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_paymentmethod, gs1s_paymentmethod_cheque },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_paymentmethod, gs1s_paymentmethod_credit_card },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_paymentmethod, gs1s_paymentmethod_credit_card_amex },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_paymentmethod, gs1s_paymentmethod_credit_card_diners_club },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_paymentmethod, gs1s_paymentmethod_credit_card_discover },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_paymentmethod, gs1s_paymentmethod_credit_card_mastercard },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_paymentmethod, gs1s_paymentmethod_credit_card_visa },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_paymentmethod, gs1s_paymentmethod_debit_card },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_paymentmethod, gs1s_paymentmethod_debit_card_maestro },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_paymentmethod, gs1s_paymentmethod_debit_card_mastercard },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_paymentmethod, gs1s_paymentmethod_debit_card_visa },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_paymentmethod, gs1s_paymentmethod_debit_card_visa_electron },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_paymentmethod, gs1s_paymentmethod_electronic_credit_ach },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_paymentmethod, gs1s_paymentmethod_electronic_debit_ach },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_paymentmethod, gs1s_paymentmethod_fed_wire_non_repetitive },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_paymentmethod, gs1s_paymentmethod_fed_wire_repetitive },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_paymentmethod, gs1s_paymentmethod_international_wire },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_paymentmethod, gs1s_paymentmethod_letter_of_credit },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_paymentmethod, gs1s_paymentmethod_online_payment },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_paymentmethod, gs1s_paymentmethod_online_payment_apple_pay },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_paymentmethod, gs1s_paymentmethod_online_payment_google_wallet },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_paymentmethod, gs1s_paymentmethod_online_payment_paypal },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_paymentmethod, gs1s_paymentmethod_postgiro },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_paymentmethod, gs1s_paymentmethod_wire_transfer_credit },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_paymentmethod, gs1s_paymentmethod_wire_transfer_debit },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_preparationtypecode, gs1s_preparationtypecode_as_drained },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_preparationtypecode, gs1s_preparationtypecode_bake },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_preparationtypecode, gs1s_preparationtypecode_barbecue },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_preparationtypecode, gs1s_preparationtypecode_blanch },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_preparationtypecode, gs1s_preparationtypecode_blind_bake },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_preparationtypecode, gs1s_preparationtypecode_boil },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_preparationtypecode, gs1s_preparationtypecode_braise },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_preparationtypecode, gs1s_preparationtypecode_broil },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_preparationtypecode, gs1s_preparationtypecode_deep_fry },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_preparationtypecode, gs1s_preparationtypecode_double_steam },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_preparationtypecode, gs1s_preparationtypecode_freeze },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_preparationtypecode, gs1s_preparationtypecode_fry },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_preparationtypecode, gs1s_preparationtypecode_griddle_fry },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_preparationtypecode, gs1s_preparationtypecode_grill },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_preparationtypecode, gs1s_preparationtypecode_heat_and_serve },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_preparationtypecode, gs1s_preparationtypecode_microwave },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_preparationtypecode, gs1s_preparationtypecode_pan_fry },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_preparationtypecode, gs1s_preparationtypecode_poach },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_preparationtypecode, gs1s_preparationtypecode_prepared },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_preparationtypecode, gs1s_preparationtypecode_pressure_cook },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_preparationtypecode, gs1s_preparationtypecode_pressure_fry },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_preparationtypecode, gs1s_preparationtypecode_ready_to_drink },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_preparationtypecode, gs1s_preparationtypecode_ready_to_eat },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_preparationtypecode, gs1s_preparationtypecode_reconstitute },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_preparationtypecode, gs1s_preparationtypecode_refrigerate },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_preparationtypecode, gs1s_preparationtypecode_roast },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_preparationtypecode, gs1s_preparationtypecode_rotisserie },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_preparationtypecode, gs1s_preparationtypecode_saute },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_preparationtypecode, gs1s_preparationtypecode_sear },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_preparationtypecode, gs1s_preparationtypecode_simmer },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_preparationtypecode, gs1s_preparationtypecode_smoke },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_preparationtypecode, gs1s_preparationtypecode_steam },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_preparationtypecode, gs1s_preparationtypecode_stew },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_preparationtypecode, gs1s_preparationtypecode_stir_fry },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_preparationtypecode, gs1s_preparationtypecode_thaw },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_preparationtypecode, gs1s_preparationtypecode_unprepared },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_preparationtypecode, gs1s_preparationtypecode_unspecified },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_preservationtechniquecode, gs1s_preservationtechniquecode_acidification },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_preservationtechniquecode, gs1s_preservationtechniquecode_alcohol_curing },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_preservationtechniquecode, gs1s_preservationtechniquecode_attested_milk },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_preservationtechniquecode, gs1s_preservationtechniquecode_boiling },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_preservationtechniquecode, gs1s_preservationtechniquecode_brining },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_preservationtechniquecode, gs1s_preservationtechniquecode_canning },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_preservationtechniquecode, gs1s_preservationtechniquecode_conserve },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_preservationtechniquecode, gs1s_preservationtechniquecode_dehydration },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_preservationtechniquecode, gs1s_preservationtechniquecode_drying },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_preservationtechniquecode, gs1s_preservationtechniquecode_fermentation },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_preservationtechniquecode, gs1s_preservationtechniquecode_freeze_drying },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_preservationtechniquecode, gs1s_preservationtechniquecode_freezing },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_preservationtechniquecode, gs1s_preservationtechniquecode_high_temperature_treated_milk },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_preservationtechniquecode, gs1s_preservationtechniquecode_hot_smoke_curing },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_preservationtechniquecode, gs1s_preservationtechniquecode_ionisation },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_preservationtechniquecode, gs1s_preservationtechniquecode_irradiation },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_preservationtechniquecode, gs1s_preservationtechniquecode_pasteurisation },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_preservationtechniquecode, gs1s_preservationtechniquecode_quick_freezing },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_preservationtechniquecode, gs1s_preservationtechniquecode_salt_curing },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_preservationtechniquecode, gs1s_preservationtechniquecode_sous_vide },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_preservationtechniquecode, gs1s_preservationtechniquecode_sterilisation },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_preservationtechniquecode, gs1s_preservationtechniquecode_sugar_curing },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_preservationtechniquecode, gs1s_preservationtechniquecode_ultra_high_temperature },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_preservationtechniquecode, gs1s_preservationtechniquecode_under_modified_atmosphere },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_preservationtechniquecode, gs1s_preservationtechniquecode_vacuum_packed },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1s_foodbeveragetobaccoproduct },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1s_wearableproduct },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_productyieldtypecode, gs1s_productyieldtypecode_after_cooking },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_productyieldtypecode, gs1s_productyieldtypecode_after_dilution },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_productyieldtypecode, gs1s_productyieldtypecode_drained_weight },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_productyieldtypecode, gs1s_productyieldtypecode_unspecified },

    { { s_gs1, 1, 0 }, { 0, 0 }, gs1s_quantitativevalue, gs1s_nutritionmeasurementtype },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_referencedfiletypecode, gs1s_referencedfiletypecode_diet_certificate },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_referencedfiletypecode, gs1s_referencedfiletypecode_document },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_referencedfiletypecode, gs1s_referencedfiletypecode_group_characteristic_sheet },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_referencedfiletypecode, gs1s_referencedfiletypecode_logo },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_referencedfiletypecode, gs1s_referencedfiletypecode_marketing_information },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_referencedfiletypecode, gs1s_referencedfiletypecode_out_of_package_image },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_referencedfiletypecode, gs1s_referencedfiletypecode_planogram },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_referencedfiletypecode, gs1s_referencedfiletypecode_product_label_image },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_referencedfiletypecode, gs1s_referencedfiletypecode_safety_data_sheet },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_referencedfiletypecode, gs1s_referencedfiletypecode_safety_summary_sheet },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_referencedfiletypecode, gs1s_referencedfiletypecode_trade_item_description },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_referencedfiletypecode, gs1s_referencedfiletypecode_video },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_referencedfiletypecode, gs1s_referencedfiletypecode_warranty_information },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_referencedfiletypecode, gs1s_referencedfiletypecode_website },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_seasonparametercode, gs1s_seasonparametercode_all_year },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_seasonparametercode, gs1s_seasonparametercode_autumn },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_seasonparametercode, gs1s_seasonparametercode_autumn_winter },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_seasonparametercode, gs1s_seasonparametercode_spring },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_seasonparametercode, gs1s_seasonparametercode_spring_summer },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_seasonparametercode, gs1s_seasonparametercode_summer },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_seasonparametercode, gs1s_seasonparametercode_winter },

    { { s_gs1, 1, 51 }, { 0, 0 }, gs1s_securitymarking, gs1s_securitymarking_hologram },
    { { s_gs1, 1, 51 }, { 0, 0 }, gs1s_securitymarking, gs1s_securitymarking_microprinting },
    { { s_gs1, 1, 51 }, { 0, 0 }, gs1s_securitymarking, gs1s_securitymarking_uvink },
    { { s_gs1, 1, 51 }, { 0, 0 }, gs1s_securitymarking, gs1s_securitymarking_watermark },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sharpnessofcheesecode, gs1s_sharpnessofcheesecode_extra_extra_sharp },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sharpnessofcheesecode, gs1s_sharpnessofcheesecode_extra_sharp },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sharpnessofcheesecode, gs1s_sharpnessofcheesecode_mild },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sharpnessofcheesecode, gs1s_sharpnessofcheesecode_regular },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sharpnessofcheesecode, gs1s_sharpnessofcheesecode_sharp },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sharpnessofcheesecode, gs1s_sharpnessofcheesecode_unclassified },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sizegroupcode, gs1s_sizegroupcode_boys },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sizegroupcode, gs1s_sizegroupcode_girls },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sizegroupcode, gs1s_sizegroupcode_infants },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sizegroupcode, gs1s_sizegroupcode_juniors },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sizegroupcode, gs1s_sizegroupcode_maternity },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sizegroupcode, gs1s_sizegroupcode_mens },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sizegroupcode, gs1s_sizegroupcode_mens_tall },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sizegroupcode, gs1s_sizegroupcode_misses },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sizegroupcode, gs1s_sizegroupcode_petite },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sizegroupcode, gs1s_sizegroupcode_womens },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sizegroupcode, gs1s_sizegroupcode_womens_tall },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sizesystemcode, gs1s_sizesystemcode_australia },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sizesystemcode, gs1s_sizesystemcode_continental },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sizesystemcode, gs1s_sizesystemcode_europe },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sizesystemcode, gs1s_sizesystemcode_japan },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sizesystemcode, gs1s_sizesystemcode_mexico },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sizesystemcode, gs1s_sizesystemcode_uk },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sizesystemcode, gs1s_sizesystemcode_us },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sizetypecode, gs1s_sizetypecode_back },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sizetypecode, gs1s_sizetypecode_chest_bust },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sizetypecode, gs1s_sizetypecode_collar },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sizetypecode, gs1s_sizetypecode_cup },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sizetypecode, gs1s_sizetypecode_hips },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sizetypecode, gs1s_sizetypecode_inseam },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sizetypecode, gs1s_sizetypecode_length },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sizetypecode, gs1s_sizetypecode_one_dimension },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sizetypecode, gs1s_sizetypecode_outside_leg },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sizetypecode, gs1s_sizetypecode_sleeve },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sizetypecode, gs1s_sizetypecode_waist },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sizetypecode, gs1s_sizetypecode_width },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sourceanimalcode, gs1s_sourceanimalcode_alligator },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sourceanimalcode, gs1s_sourceanimalcode_ass },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sourceanimalcode, gs1s_sourceanimalcode_bear },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sourceanimalcode, gs1s_sourceanimalcode_beef_cow },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sourceanimalcode, gs1s_sourceanimalcode_boar },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sourceanimalcode, gs1s_sourceanimalcode_buffalo },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sourceanimalcode, gs1s_sourceanimalcode_calf },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sourceanimalcode, gs1s_sourceanimalcode_camel },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sourceanimalcode, gs1s_sourceanimalcode_chicken },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sourceanimalcode, gs1s_sourceanimalcode_cockerel },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sourceanimalcode, gs1s_sourceanimalcode_cow },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sourceanimalcode, gs1s_sourceanimalcode_crocodile },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sourceanimalcode, gs1s_sourceanimalcode_deer },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sourceanimalcode, gs1s_sourceanimalcode_dog },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sourceanimalcode, gs1s_sourceanimalcode_duck },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sourceanimalcode, gs1s_sourceanimalcode_duckling },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sourceanimalcode, gs1s_sourceanimalcode_fish },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sourceanimalcode, gs1s_sourceanimalcode_fowl },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sourceanimalcode, gs1s_sourceanimalcode_frog },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sourceanimalcode, gs1s_sourceanimalcode_goat },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sourceanimalcode, gs1s_sourceanimalcode_goose },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sourceanimalcode, gs1s_sourceanimalcode_grouse },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sourceanimalcode, gs1s_sourceanimalcode_guineafowl },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sourceanimalcode, gs1s_sourceanimalcode_hare },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sourceanimalcode, gs1s_sourceanimalcode_horse },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sourceanimalcode, gs1s_sourceanimalcode_invertebrate },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sourceanimalcode, gs1s_sourceanimalcode_lamb },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sourceanimalcode, gs1s_sourceanimalcode_land_snail },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sourceanimalcode, gs1s_sourceanimalcode_milk_cow },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sourceanimalcode, gs1s_sourceanimalcode_moose },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sourceanimalcode, gs1s_sourceanimalcode_ostrich },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sourceanimalcode, gs1s_sourceanimalcode_other },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sourceanimalcode, gs1s_sourceanimalcode_partridge },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sourceanimalcode, gs1s_sourceanimalcode_pheasant },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sourceanimalcode, gs1s_sourceanimalcode_pigeon },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sourceanimalcode, gs1s_sourceanimalcode_pork },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sourceanimalcode, gs1s_sourceanimalcode_poussin },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sourceanimalcode, gs1s_sourceanimalcode_quail },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sourceanimalcode, gs1s_sourceanimalcode_rabbit },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sourceanimalcode, gs1s_sourceanimalcode_rat },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sourceanimalcode, gs1s_sourceanimalcode_reindeer },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sourceanimalcode, gs1s_sourceanimalcode_roe_deer },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sourceanimalcode, gs1s_sourceanimalcode_sheep },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sourceanimalcode, gs1s_sourceanimalcode_shellfish },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sourceanimalcode, gs1s_sourceanimalcode_snake },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sourceanimalcode, gs1s_sourceanimalcode_snipe },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sourceanimalcode, gs1s_sourceanimalcode_sparrow },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sourceanimalcode, gs1s_sourceanimalcode_thrush },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sourceanimalcode, gs1s_sourceanimalcode_turkey },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sourceanimalcode, gs1s_sourceanimalcode_turtle },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sourceanimalcode, gs1s_sourceanimalcode_woodcock },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sourceanimalcode, gs1s_sourceanimalcode_yak },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sportingactivitytypecode, gs1s_sportingactivitytypecode_american_football },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sportingactivitytypecode, gs1s_sportingactivitytypecode_archery },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sportingactivitytypecode, gs1s_sportingactivitytypecode_badminton },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sportingactivitytypecode, gs1s_sportingactivitytypecode_baseball },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sportingactivitytypecode, gs1s_sportingactivitytypecode_basketball },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sportingactivitytypecode, gs1s_sportingactivitytypecode_boxing },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sportingactivitytypecode, gs1s_sportingactivitytypecode_climbing },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sportingactivitytypecode, gs1s_sportingactivitytypecode_cricket },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sportingactivitytypecode, gs1s_sportingactivitytypecode_cycling },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sportingactivitytypecode, gs1s_sportingactivitytypecode_diving },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sportingactivitytypecode, gs1s_sportingactivitytypecode_equestrian },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sportingactivitytypecode, gs1s_sportingactivitytypecode_fencing },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sportingactivitytypecode, gs1s_sportingactivitytypecode_fishing },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sportingactivitytypecode, gs1s_sportingactivitytypecode_football },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sportingactivitytypecode, gs1s_sportingactivitytypecode_golf },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sportingactivitytypecode, gs1s_sportingactivitytypecode_gymnastics },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sportingactivitytypecode, gs1s_sportingactivitytypecode_hockey },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sportingactivitytypecode, gs1s_sportingactivitytypecode_ice_hockey },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sportingactivitytypecode, gs1s_sportingactivitytypecode_ice_skating },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sportingactivitytypecode, gs1s_sportingactivitytypecode_motor_cycling },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sportingactivitytypecode, gs1s_sportingactivitytypecode_rugby },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sportingactivitytypecode, gs1s_sportingactivitytypecode_running },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sportingactivitytypecode, gs1s_sportingactivitytypecode_skiing },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sportingactivitytypecode, gs1s_sportingactivitytypecode_squash },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sportingactivitytypecode, gs1s_sportingactivitytypecode_surfing },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sportingactivitytypecode, gs1s_sportingactivitytypecode_swimming },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sportingactivitytypecode, gs1s_sportingactivitytypecode_tennis },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sportingactivitytypecode, gs1s_sportingactivitytypecode_unclassified },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sportingactivitytypecode, gs1s_sportingactivitytypecode_unidentified },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_targetconsumergendercode, gs1s_targetconsumergendercode_female },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_targetconsumergendercode, gs1s_targetconsumergendercode_male },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_targetconsumergendercode, gs1s_targetconsumergendercode_unisex },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_tradechannelcode, gs1s_tradechannelcode_instore_only },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_tradechannelcode, gs1s_tradechannelcode_omnichannel },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_tradechannelcode, gs1s_tradechannelcode_online_only },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_typecode, gs1s_allergentypecode },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_typecode, gs1s_anatomicalformcode },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_typecode, gs1s_cheesefirmnesscode },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_typecode, gs1s_colourcodelistcode },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_typecode, gs1s_consumerlifestagecode },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_typecode, gs1s_consumersalesconditionscode },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_typecode, gs1s_diettypecode },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_typecode, gs1s_discounttypecode },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_typecode, gs1s_foodbeveragerefrigerationclaimcode },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_typecode, gs1s_foodbeveragetargetusecode },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_typecode, gs1s_footwearfasteningtypecode },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_typecode, gs1s_freshorseawaterfarmedcode },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_typecode, gs1s_growingmethodcode },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_typecode, gs1s_levelofcontainmentcode },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_typecode, gs1s_maturationmethodcode },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_typecode, gs1s_measurementprecisioncode },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_typecode, gs1s_nonbinarylogiccode },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_typecode, gs1s_nutrientbasisquantitycode },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_typecode, gs1s_nutritionalclaimtypecode },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_typecode, gs1s_offerredemptiontypecode },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_typecode, gs1s_organicclaimagencycode },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_typecode, gs1s_organisationroletype },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_typecode, gs1s_packagingfeaturecode },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_typecode, gs1s_packagingfunctioncode },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_typecode, gs1s_packagingmarkeddietallergencode },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_typecode, gs1s_packagingmarkedfreefromcode },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_typecode, gs1s_packagingmarkedlabelaccreditationcode },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_typecode, gs1s_packagingmaterialtypecode },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_typecode, gs1s_packagingrecyclingprocesstypecode },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_typecode, gs1s_packagingrecyclingschemecode },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_typecode, gs1s_packagingshapecode },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_typecode, gs1s_paymentmethod },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_typecode, gs1s_preparationtypecode },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_typecode, gs1s_preservationtechniquecode },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_typecode, gs1s_productyieldtypecode },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_typecode, gs1s_referencedfiletypecode },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_typecode, gs1s_seasonparametercode },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_typecode, gs1s_securitymarking },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_typecode, gs1s_sharpnessofcheesecode },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_typecode, gs1s_sizegroupcode },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_typecode, gs1s_sizesystemcode },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_typecode, gs1s_sizetypecode },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_typecode, gs1s_sourceanimalcode },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_typecode, gs1s_sportingactivitytypecode },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_typecode, gs1s_targetconsumergendercode },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_typecode, gs1s_tradechannelcode },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_typecode, gs1s_uppertypecode },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_uppertypecode, gs1s_uppertypecode_closed_toe_closed_back_open_instep },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_uppertypecode, gs1s_uppertypecode_closed_toe_open_back },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_uppertypecode, gs1s_uppertypecode_closed_toe_strap_back },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_uppertypecode, gs1s_uppertypecode_fully_closed },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_uppertypecode, gs1s_uppertypecode_open_toe_fully_closed_back },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_uppertypecode, gs1s_uppertypecode_open_toe_open_back },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_uppertypecode, gs1s_uppertypecode_open_toe_strap_back },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_uppertypecode, gs1s_uppertypecode_unclassified },

    { { s_gs1, 1, 0 }, { 0, 0 }, gs1s_wearableproduct, gs1s_clothing },
    { { s_gs1, 1, 0 }, { 0, 0 }, gs1s_wearableproduct, gs1s_footwear },

    // ldp
    { { s_ldp, 1, 0 }, { 0, 0 }, ldp_container, ldp_basiccontainer },

    { { s_ldp, 1, 0 }, { 0, 0 }, ldp_container, ldp_directcontainer },

    { { s_ldp, 1, 0 }, { 0, 0 }, ldp_container, ldp_indirectcontainer },

    { { s_ldp, 1, 0 }, { 0, 0 }, ldp_rdfsource, ldp_container },
    { { s_ldp, 1, 0 }, { 0, 0 }, ldp_rdfsource, ldp_container },

    { { s_ldp, 1, 0 }, { 0, 0 }, ldp_resource, ldp_rdfsource },
    { { s_ldp, 1, 0 }, { 0, 0 }, ldp_resource, ldp_nonrdfsource },

    // media resources
    { { s_ma, 1, 0 }, { 0, 0 }, ma_agent, ma_organisation },
    { { s_ma, 1, 0 }, { 0, 0 }, ma_agent, ma_person },

    { { s_ma, 1, 0 }, { 0, 0 }, ma_mediafragment, ma_track },

    { { s_ma, 1, 0 }, { 0, 0 }, ma_mediaresource, ma_image },
    { { s_ma, 1, 0 }, { 0, 0 }, ma_mediaresource, ma_mediafragment },

    { { s_ma, 1, 0 }, { 0, 0 }, ma_track, ma_audiotrack },
    { { s_ma, 1, 0 }, { 0, 0 }, ma_track, ma_datatrack },
    { { s_ma, 1, 0 }, { 0, 0 }, ma_track, ma_videotrack },

    // web annotation
    { { s_dqv, 1, 0 }, { 0, 0 }, oa_annotation, dqv_qualityannotation },

    { { s_oa, 1, 0 }, { 0, 0 }, oa_direction, oa_autodirection },
    { { s_oa, 1, 0 }, { 0, 0 }, oa_direction, oa_ltrdirection },
    { { s_oa, 1, 0 }, { 0, 0 }, oa_direction, oa_rtldirection },

    { { s_oa, 1, 0 }, { 0, 0 }, oa_motivation, oa_assessing },
    { { s_oa, 1, 0 }, { 0, 0 }, oa_motivation, oa_bookmarking },
    { { s_oa, 1, 0 }, { 0, 0 }, oa_motivation, oa_classifying },
    { { s_oa, 1, 0 }, { 0, 0 }, oa_motivation, oa_commenting },
    { { s_oa, 1, 0 }, { 0, 0 }, oa_motivation, oa_describing },
    { { s_oa, 1, 0 }, { 0, 0 }, oa_motivation, oa_editing },
    { { s_oa, 1, 0 }, { 0, 0 }, oa_motivation, oa_highlighting },
    { { s_oa, 1, 0 }, { 0, 0 }, oa_motivation, oa_identifying },
    { { s_oa, 1, 0 }, { 0, 0 }, oa_motivation, oa_linking },
    { { s_oa, 1, 0 }, { 0, 0 }, oa_motivation, oa_moderating },
    { { s_oa, 1, 0 }, { 0, 0 }, oa_motivation, oa_questioning },
    { { s_oa, 1, 0 }, { 0, 0 }, oa_motivation, oa_replying },
    { { s_oa, 1, 0 }, { 0, 0 }, oa_motivation, oa_tagging },

    { { s_oa, 1, 0 }, { 0, 0 }, oa_resourceselection, oa_specificresource },

    { { s_oa, 1, 0 }, { 0, 0 }, oa_selector, oa_cssselector },
    { { s_oa, 1, 0 }, { 0, 0 }, oa_selector, oa_datapositionselector },
    { { s_oa, 1, 0 }, { 0, 0 }, oa_selector, oa_fragmentselector },
    { { s_oa, 1, 0 }, { 0, 0 }, oa_selector, oa_rangeselector },
    { { s_oa, 1, 0 }, { 0, 0 }, oa_selector, oa_svgselector },
    { { s_oa, 1, 0 }, { 0, 0 }, oa_selector, oa_textpositionselector },
    { { s_oa, 1, 0 }, { 0, 0 }, oa_selector, oa_textquoteselector },
    { { s_oa, 1, 0 }, { 0, 0 }, oa_selector, oa_xpathselector },

    { { s_oa, 1, 0 }, { 0, 0 }, oa_state, oa_httprequeststate },
    { { s_oa, 1, 0 }, { 0, 0 }, oa_state, oa_timestate },

    { { s_oa, 1, 0 }, { 0, 0 }, oa_style, oa_cssstyle },

    // odrl
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_accepttracking },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_adhocshare },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_aggregate },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_annotate },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_anonymize },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_append },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_appendto },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_archive },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_attachpolicy },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_attachsource },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_attribute },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_attribution },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_commercialize },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_commericaluse },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_compensate },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_concurrentuse },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_copy },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_delete },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_derivativeworks },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_derive },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_digitize },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_display },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_distribute },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_distribution },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_ensureexclusivity },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_execute },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_export },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_extract },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_extractchar },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_extractpage },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_extractword },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_give },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_grantuse },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_include },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_index },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_inform },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_install },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_lease },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_lend },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_license },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_modify },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_move },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_nextpolicy },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_notice },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_obtainconsent },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_pay },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_play },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_present },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_preview },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_print },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_read },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_reproduce },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_reproduction },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_reviewpolicy },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_secondaryuse },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_sell },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_share },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_sharealike },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_sharing },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_sourcecode },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_stream },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_synchronize },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_texttospeech },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_transfer },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_transform },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_translate },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_uninstall },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_use },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_watermark },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_write },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrl_writeto },

    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_asset, odrl_assetcollection },

    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_conflictterm, odrl_invalid },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_conflictterm, odrl_perm },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_conflictterm, odrl_prohibit },

    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_leftoperand, odrl_absoluteposition },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_leftoperand, odrl_absolutesize },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_leftoperand, odrl_absolutespatialposition },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_leftoperand, odrl_absolutetemporalposition },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_leftoperand, odrl_count },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_leftoperand, odrl_datetime },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_leftoperand, odrl_delayperiod },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_leftoperand, odrl_deliverychannel },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_leftoperand, odrl_device },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_leftoperand, odrl_elapsedtime },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_leftoperand, odrl_event },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_leftoperand, odrl_fileformat },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_leftoperand, odrl_industry },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_leftoperand, odrl_language },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_leftoperand, odrl_media },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_leftoperand, odrl_meteredtime },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_leftoperand, odrl_payamount },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_leftoperand, odrl_percentage },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_leftoperand, odrl_product },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_leftoperand, odrl_purpose },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_leftoperand, odrl_recipient },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_leftoperand, odrl_relativeposition },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_leftoperand, odrl_relativesize },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_leftoperand, odrl_relativespatialposition },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_leftoperand, odrl_relativetemporalposition },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_leftoperand, odrl_resolution },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_leftoperand, odrl_spatial },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_leftoperand, odrl_spatialcoordinates },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_leftoperand, odrl_system },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_leftoperand, odrl_systemdevice },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_leftoperand, odrl_timeinterval },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_leftoperand, odrl_unitofcount },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_leftoperand, odrl_version },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_leftoperand, odrl_virtuallocation },

    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_operator, odrl_eq },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_operator, odrl_gt },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_operator, odrl_gteq },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_operator, odrl_haspart },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_operator, odrl_isa },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_operator, odrl_isallof },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_operator, odrl_isanyof },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_operator, odrl_isnoneof },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_operator, odrl_ispartof },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_operator, odrl_lt },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_operator, odrl_lteq },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_operator, odrl_neq },

    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_party, odrl_partycollection },

    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_policy, odrl_agreement },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_policy, odrl_assertion },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_policy, odrl_offer },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_policy, odrl_privacy },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_policy, odrl_request },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_policy, odrl_set },

    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_rule, odrl_duty },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_rule, odrl_permission },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_rule, odrl_prohibition },

    // org
    { { s_gr, 1, 0 }, { 0, 0 }, org_formalorganisation, gr_businessentity },

    { { s_org, 1, 0 }, { 0, 0 }, org_organisation, org_formalorganisation },
    { { s_org, 1, 0 }, { 0, 0 }, org_organisation, org_organisationalunit },
    { { s_org, 1, 0 }, { 0, 0 }, org_organisation, org_organisationalcollaboration },

    { { s_org, 1, 0 }, { 0, 0 }, org_role, org_head },

    // owl
    { { s_owl, 1, 0 }, { 0, 0 }, owl_class, owl_restriction },

    { { s_owl, 1, 0 }, { 0, 0 }, owl_objectproperty, owl_asymmetricproperty },
    { { s_owl, 1, 0 }, { 0, 0 }, owl_objectproperty, owl_inversefunctionalproperty },
    { { s_owl, 1, 0 }, { 0, 0 }, owl_objectproperty, owl_irreflexiveproperty },
    { { s_owl, 1, 0 }, { 0, 0 }, owl_objectproperty, owl_reflexiveproperty },
    { { s_owl, 1, 0 }, { 0, 0 }, owl_objectproperty, owl_symmetricproperty },
    { { s_owl, 1, 0 }, { 0, 0 }, owl_objectproperty, owl_transitiveproperty },

    { { s_wdrs, 1, 0 }, { 0, 0 }, owl_ontology, wdrs_document },

    { { s_gs1, 1, 1 }, { 0, 0 }, owl_thing, gs1s_additionalproductclassificationdetails },
    { { s_gs1, 1, 1 }, { 0, 0 }, owl_thing, gs1s_additivedetails },
    { { s_gs1, 1, 3 }, { 0, 0 }, owl_thing, gs1s_ai }, // ai is currently undefined, so this is a best guess
    { { s_gs1, 1, 1 }, { 0, 0 }, owl_thing, gs1s_allergendetails },
    { { s_gs1, 1, 51 }, { 0, 0 }, owl_thing, gs1s_authenticitydetails },
    { { s_gs1, 1, 1 }, { 0, 0 }, owl_thing, gs1s_awardprizedetails },
    { { s_gs1, 1, 1 }, { 0, 0 }, owl_thing, gs1s_brand },
    { { s_gs1, 1, 1 }, { 0, 0 }, owl_thing, gs1s_certificationdetails },
    { { s_gs1, 1, 1 }, { 0, 0 }, owl_thing, gs1s_colourcodedetails },
    { { s_gs1, 1, 1 }, { 0, 0 }, owl_thing, gs1s_contactpoint },
    { { s_gs1, 1, 1 }, { 0, 0 }, owl_thing, gs1s_country },
    { { s_gs1, 1, 1 }, { 0, 0 }, owl_thing, gs1s_diettypecodedetails },
    { { s_gs1, 1, 1 }, { 0, 0 }, owl_thing, gs1s_foodandbeveragepreparationinformation },
    { { s_gs1, 1, 1 }, { 0, 0 }, owl_thing, gs1s_foodbeveragetobaccoingredientdetails },
    { { s_gs1, 1, 1 }, { 0, 0 }, owl_thing, gs1s_geocoordinates },
    { { s_gs1, 1, 1 }, { 0, 0 }, owl_thing, gs1s_offer },
    { { s_gs1, 1, 1 }, { 0, 0 }, owl_thing, gs1s_organicclaimdetails },
    { { s_gs1, 1, 1 }, { 0, 0 }, owl_thing, gs1s_organisation },
    { { s_gs1, 1, 1 }, { 0, 0 }, owl_thing, gs1s_packagingdetails },
    { { s_gs1, 1, 1 }, { 0, 0 }, owl_thing, gs1s_packagingmaterialdetails },
    { { s_gs1, 1, 1 }, { 0, 0 }, owl_thing, gs1s_place },
    { { s_gs1, 1, 1 }, { 0, 0 }, owl_thing, gs1s_postaladdress },
    { { s_gs1, 1, 1 }, { 0, 0 }, owl_thing, gs1s_pricespecification },
    { { s_gs1, 1, 1 }, { 0, 0 }, owl_thing, gs1s_product },
    { { s_gs1, 1, 1 }, { 0, 0 }, owl_thing, gs1s_productyielddetails },
    { { s_gs1, 1, 1 }, { 0, 0 }, owl_thing, gs1s_quantitativevalue },
    { { s_gs1, 1, 1 }, { 0, 0 }, owl_thing, gs1s_referencedfiledetails },
    { { s_gs1, 1, 1 }, { 0, 0 }, owl_thing, gs1s_returnablepackagedepositdetails },
    { { s_gs1, 1, 1 }, { 0, 0 }, owl_thing, gs1s_sizecodedetails },
    { { s_gs1, 1, 1 }, { 0, 0 }, owl_thing, gs1s_sizedetails },
    { { s_gs1, 1, 1 }, { 0, 0 }, owl_thing, gs1s_targetmarketdetails },
    { { s_gs1, 1, 1 }, { 0, 0 }, owl_thing, gs1s_textilematerialdetails },
    { { s_gs1, 1, 1 }, { 0, 0 }, owl_thing, gs1s_typecode },
    { { s_gs1, 1, 1 }, { 0, 0 }, owl_thing, gs1s_warrantypromise },

    { { s_foaf, 0, 1 }, { 0, 0 }, owl_thing, foaf_onlineaccount },
    { { s_owl, 1, 0 }, { 0, 0 }, owl_thing, owl_namedindividual },
    { { s_owl, 1, 0 }, { 0, 0 }, owl_thing, owl_nothing },
    { { s_time, 1, 0 }, { 0, 0 }, owl_thing, time_temporalentity },

//    { { s_frbr, 1, 0 }, { 0, 0 }, owltime_event, fs_event },


    // poetry
    { { s_poetry, 1, 0 }, { 0, 0 }, poetry_poem, poetry_stanza },

    // prov
    { { s_prov, 1, 0 }, { 0, 0 }, prov_activity, org_changeevent },

    { { s_prov, 1, 0 }, { 0, 0 }, prov_agent, prov_organisation },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_agent, prov_person },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_agent, prov_softwareagent },

    { { s_prov, 1, 0 }, { 0, 0 }, prov_activityinfluence, prov_communication },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_activityinfluence, prov_generation },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_activityinfluence, prov_invalidation },

    { { s_prov, 1, 0 }, { 0, 0 }, prov_agentinfluence, prov_association },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_agentinfluence, prov_attribution },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_agentinfluence, prov_delegation },

    { { s_prov, 1, 0 }, { 0, 0 }, prov_collection, prov_emptycollection },

    { { s_prov, 1, 0 }, { 0, 0 }, prov_derivation, prov_primarysource },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_derivation, prov_quotation },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_derivation, prov_revision },

    { { s_prov, 1, 0 }, { 0, 0 }, prov_entity, prov_bundle },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_entity, prov_collection },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_entity, prov_plan },

    { { s_prov, 1, 0 }, { 0, 0 }, prov_entityinfluence, prov_derivation },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_entityinfluence, prov_end },
    { { s_dcat, 2, 0 }, { 0, 0 }, prov_entityinfluence, dca_relationship },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_entityinfluence, prov_start },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_entityinfluence, prov_usage },

    { { s_prov, 1, 0 }, { 0, 0 }, prov_influence, prov_activityinfluence },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_influence, prov_agentinfluence },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_influence, prov_entityinfluence },

    { { s_prov, 1, 0 }, { 0, 0 }, prov_instantaneousevent, prov_end },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_instantaneousevent, prov_generation },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_instantaneousevent, prov_invalidation },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_instantaneousevent, prov_start },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_instantaneousevent, prov_usage },

    // ptr
    { { s_ptr, 1, 0 }, { 0, 0 }, ptr_content, ptr_contentasbase64 },
    { { s_ptr, 1, 0 }, { 0, 0 }, ptr_content, ptr_contentastext },
    { { s_ptr, 1, 0 }, { 0, 0 }, ptr_content, ptr_contentasxml },

    // data cube
    { { s_qb, 1, 0 }, { 0, 0 }, anything, qb_componentset },

    { { s_qb, 1, 0 }, { 0, 0 }, qb_attachable, qb_dataset },
    { { s_qb, 1, 0 }, { 0, 0 }, qb_attachable, qb_observation },
    { { s_qb, 1, 0 }, { 0, 0 }, qb_attachable, qb_slice },

    // unnecessary since its inherited from qb_componentproperty, but ...
    { { s_qb, 1, 0 }, { 0, 0 }, qb_codedproperty, qb_dimensionproperty },

    { { s_qb, 1, 0 }, { 0, 0 }, qb_codelist, qb_hierarchicalcodelist },

    { { s_qb, 1, 0 }, { 0, 0 }, qb_componentproperty, qb_attributeproperty },
    { { s_qb, 1, 0 }, { 0, 0 }, qb_componentproperty, qb_codedproperty },
    { { s_qb, 1, 0 }, { 0, 0 }, qb_componentproperty, qb_dimensionproperty },
    { { s_qb, 1, 0 }, { 0, 0 }, qb_componentproperty, qb_measureproperty },

    { { s_qb, 1, 0 }, { 0, 0 }, qb_componentset, qb_componentproperty },
    { { s_qb, 1, 0 }, { 0, 0 }, qb_componentset, qb_componentspecification },
    { { s_qb, 1, 0 }, { 0, 0 }, qb_componentset, qb_datastructuredefinition },
    { { s_qb, 1, 0 }, { 0, 0 }, qb_componentset, qb_slicekey },

    { { s_dqv, 1, 0 }, { 0, 0 }, qb_dataset, dqv_qualitymeasurementdataset },

    { { s_dqv, 1, 0 }, { 0, 0 }, qb_observation, dqv_qualitymeasurement },

    { { s_qb, 1, 0 }, { 0, 0 }, qb_observationgroup, qb_slice },

    // RDF
    { { s_qb, 1, 0 }, { 0, 0 }, rdf_property, qb_componentproperty },
    { { s_rdf, 1, 0 }, { 0, 0 }, rdf_property, rdfs_containermembershipproperty },
    { { s_owl, 1, 0 }, { 0, 0 }, rdf_property, owl_annotationproperty },
    { { s_owl, 1, 0 }, { 0, 0 }, rdf_property, owl_datatypeproperty },
    { { s_owl, 1, 0 }, { 0, 0 }, rdf_property, owl_deprecatedproperty },
    { { s_owl, 1, 0 }, { 0, 0 }, rdf_property, owl_functionalproperty },
    { { s_owl, 1, 0 }, { 0, 0 }, rdf_property, owl_objectproperty },
    { { s_owl, 1, 0 }, { 0, 0 }, rdf_property, owl_ontologyproperty },

    // RDFg
    { { s_rdfg, 1, 0 }, { 0, 0 }, rdfg_graph, dqv_qualitymetadata },

    // RDFs
    { { s_owl, 1, 0 }, { 0, 0 }, rdfs_class, owl_class },
    { { s_rdfs, 1, 0 }, { 0, 0 }, rdfs_class, rdfs_datatype },
    { { s_owl, 1, 0 }, { 0, 0 }, rdfs_class, owl_deprecatedclass },

    { { s_rdfs, 1, 0 }, { 0, 0 }, rdfs_container, rdf_alt },
    { { s_rdfs, 1, 0 }, { 0, 0 }, rdfs_container, rdf_bag },
    { { s_rdfs, 1, 0 }, { 0, 0 }, rdfs_container, rdf_seq },

    { { s_owl, 1, 0 }, { 0, 0 }, rdfs_datatype, owl_datarange },
    { { s_rdfs, 1, 0 }, { 0, 0 }, rdfs_datatype, rdf_html },
    { { s_rdfs, 1, 0 }, { 0, 0 }, rdfs_datatype, rdf_langstring },
    { { s_rdfs, 1, 0 }, { 0, 0 }, rdfs_datatype, rdf_plainliteral },

    { { s_rdfs, 1, 0 }, { 0, 0 }, rdfs_literal, rdf_compoundliteral },
    { { s_rdfs, 1, 0 }, { 0, 0 }, rdfs_literal, rdf_json },
    { { s_rdfs, 1, 0 }, { 0, 0 }, rdfs_literal, rdf_xmlliteral },

    { { s_owl, 1, 0 }, { 0, 0 }, rdfs_resource, owl_alldisjointclasses },
    { { s_owl, 1, 0 }, { 0, 0 }, rdfs_resource, owl_alldisjointproperties },
    { { s_owl, 1, 0 }, { 0, 0 }, rdfs_resource, owl_annotation },
    { { s_owl, 1, 0 }, { 0, 0 }, rdfs_resource, owl_axiom },
    { { s_rdfs, 1, 0 }, { 0, 0 }, rdfs_resource, rdfs_class },
    { { s_rdfs, 1, 0 }, { 0, 0 }, rdfs_resource, rdfs_container },
    { { s_rdfs, 1, 0 }, { 0, 0 }, rdfs_resource, rdf_list },
    { { s_rdfs, 1, 0 }, { 0, 0 }, rdfs_resource, rdfs_literal },
    { { s_owl, 1, 0 }, { 0, 0 }, rdfs_resource, owl_negativepropertyassertion },
    { { s_owl, 1, 0 }, { 0, 0 }, rdfs_resource, owl_ontology },
    { { s_oa, 1, 0 }, { 0, 0 }, rdfs_resource, oa_prefercontaineddescriptions },
    { { s_oa, 1, 0 }, { 0, 0 }, rdfs_resource, oa_prefercontainediris },
    { { s_rdfs, 1, 0 }, { 0, 0 }, rdfs_resource, rdf_property },
    { { s_doap, 1, 0 }, { 0, 0 }, rdfs_resource, doap_specification },
    { { s_rdfs, 1, 0 }, { 0, 0 }, rdfs_resource, rdf_statement },
    { { s_doap, 1, 0 }, { 0, 0 }, rdfs_resource, doap_version },

    // schema.org
    { { 3, 1 }, { 0, 0 }, sch_accommodation, sch_apartment },
    { { 3, 1 }, { 0, 0 }, sch_accommodation, sch_campingpitch },
    { { 3, 1 }, { 0, 0 }, sch_accommodation, sch_house },
    { { 3, 1 }, { 0, 0 }, sch_accommodation, sch_room },
    { { 3, 1 }, { 0, 0 }, sch_accommodation, sch_suite },
    { { 1, 2 }, { 0, 0 }, sch_achieveaction, sch_loseaction },
    { { 1, 2 }, { 0, 0 }, sch_achieveaction, sch_tieaction },
    { { 1, 2 }, { 0, 0 }, sch_achieveaction, sch_winaction },
    { { s_odrl, 1, 0 }, { 0, 0 }, sch_action, odrl_action },
    { { 1, 2 }, { 0, 0 }, sch_action, sch_achieveaction },
    { { 1, 2 }, { 0, 0 }, sch_action, sch_assessaction },
    { { 1, 2 }, { 0, 0 }, sch_action, sch_consumeaction },
    { { 1, 92 }, { 0, 0 }, sch_action, sch_controlaction },
    { { 1, 2 }, { 0, 0 }, sch_action, sch_createaction },
    { { 1, 2 }, { 0, 0 }, sch_action, sch_findaction },
    { { 1, 2 }, { 0, 0 }, sch_action, sch_interactaction },
    { { 1, 2 }, { 0, 0 }, sch_action, sch_moveaction },
    { { 1, 2 }, { 0, 0 }, sch_action, sch_organiseaction },
    { { 1, 2 }, { 0, 0 }, sch_action, sch_playaction },
    { { 1, 2 }, { 0, 0 }, sch_action, sch_searchaction },
    { { 11, 0 }, { 0, 0 }, sch_action, sch_seektoaction },
    { { 11, 0 }, { 0, 0 }, sch_action, sch_solvemathaction },
    { { 1, 2 }, { 0, 0 }, sch_action, sch_tradeaction },
    { { 1, 2 }, { 0, 0 }, sch_action, sch_transferaction },
    { { 1, 2 }, { 0, 0 }, sch_action, sch_updateaction },
    { { 1, 20 }, { 0, 0 }, sch_actionstatustype, sch_activeactionstatus },
    { { 1, 20 }, { 0, 0 }, sch_actionstatustype, sch_completedactionstatus },
    { { 1, 20 }, { 0, 0 }, sch_actionstatustype, sch_failedactionstatus },
    { { 1, 20 }, { 0, 0 }, sch_actionstatustype, sch_potentialactionstatus },
    { { 1, 2 }, { 0, 0 }, sch_addaction, sch_insertaction },
    { { 0, 10 }, { 0, 0 }, sch_administrativearea, sch_city },
    { { 0, 10 }, { 0, 0 }, sch_administrativearea, sch_country },
    { { 7, 1 }, { 0, 0 }, sch_administrativearea, sch_schooldistrict },
    { { 0, 10 }, { 0, 0 }, sch_administrativearea, sch_state },
    { { 14, 0 }, { 0, 0 }, sch_adultorientatedenumeration, sch_alcoholconsideration },
    { { 14, 0 }, { 0, 0 }, sch_adultorientatedenumeration, sch_dangerousgoodconsideration },
    { { 14, 0 }, { 0, 0 }, sch_adultorientatedenumeration, sch_healthcareconsideration },
    { { 14, 0 }, { 0, 0 }, sch_adultorientatedenumeration, sch_narcoticconsideration },
    { { 14, 0 }, { 0, 0 }, sch_adultorientatedenumeration, sch_reducedrelevanceforchildrenconsideration },
    { { 14, 0 }, { 0, 0 }, sch_adultorientatedenumeration, sch_sexualcontentconsideration },
    { { 14, 0 }, { 0, 0 }, sch_adultorientatedenumeration, sch_tobacconicotineconsideration },
    { { 14, 0 }, { 0, 0 }, sch_adultorientatedenumeration, sch_unclassifiedadultconsideration },
    { { 14, 0 }, { 0, 0 }, sch_adultorientatedenumeration, sch_violenceconsideration },
    { { 14, 0 }, { 0, 0 }, sch_adultorientatedenumeration, sch_weaponconsideration },
    { { 2, 0 }, { 0, 0 }, sch_aggregaterating, sch_employeraggregaterating },
    { { 1, 2 }, { 0, 0 }, sch_allocateaction, sch_acceptaction },
    { { 1, 2 }, { 0, 0 }, sch_allocateaction, sch_assignaction },
    { { 1, 2 }, { 0, 0 }, sch_allocateaction, sch_authoriseaction },
    { { 1, 2 }, { 0, 0 }, sch_allocateaction, sch_rejectaction },
    { { 0, 95 }, { 0, 0 }, sch_anatomicalstructure, sch_bone },
    { { 0, 95 }, { 0, 0 }, sch_anatomicalstructure, sch_brainstructure },
    { { 0, 95 }, { 0, 0 }, sch_anatomicalstructure, sch_joint },
    { { 0, 95 }, { 0, 0 }, sch_anatomicalstructure, sch_ligament },
    { { 0, 95 }, { 0, 0 }, sch_anatomicalstructure, sch_muscle },
    { { 0, 95 }, { 0, 0 }, sch_anatomicalstructure, sch_nerve },
    { { 0, 95 }, { 0, 0 }, sch_anatomicalstructure, sch_vessel },
    { { 3, 5 }, { 0, 0 }, sch_article, sch_advertisercontentarticle },
    { { 0, 10 }, { 2, 0 }, sch_article, sch_blogposting },
    { { 3, 0 }, { 3, 0 }, sch_article, sch_blogposting },
    { { 2, 0 }, { 0, 0 }, sch_article, sch_medicalscholarlyarticle },
    { { 0, 10 }, { 0, 0 }, sch_article, sch_newsarticle },
    { { 2, 1 }, { 0, 0 }, sch_article, sch_report },
    { { 3, 5 }, { 0, 0 }, sch_article, sch_satiricalarticle },
    { { 0, 10 }, { 0, 0 }, sch_article, sch_scholarlyarticle },
    { { 2, 1 }, { 0, 0 }, sch_article, sch_socialmediaposting },
    { { 1, 0 }, { 0, 0 }, sch_article, sch_techarticle },
    { { 1, 2 }, { 0, 0 }, sch_assessaction, sch_chooseaction },
    { { 1, 2 }, { 0, 0 }, sch_assessaction, sch_ignoreaction },
    { { 1, 2 }, { 0, 0 }, sch_assessaction, sch_reactaction },
    { { 1, 2 }, { 0, 0 }, sch_assessaction, sch_reviewaction },
    { { 1, 3 }, { 0, 0 }, sch_audience, sch_businessaudience },
    { { 1, 0 }, { 0, 0 }, sch_audience, sch_educationaudience },
    { { 0, 95 }, { 2, 1 }, sch_audience, sch_medicalaudience },
    { { 3, 5 }, { 0, 0 }, sch_audience, sch_medicalaudience },
    { { 1, 0 }, { 0, 0 }, sch_audience, sch_peopleaudience },
    { { 3, 5 }, { 0, 0 }, sch_audioobject, sch_audiobook },
    { { 13, 0 }, { 0, 0 }, sch_audioobject, sch_audioobjectsnapshot },
    { { 0, 10 }, { 0, 0 }, sch_automotivebusiness, sch_autobodyshop },
    { { 0, 10 }, { 0, 0 }, sch_automotivebusiness, sch_autodealer },
    { { 0, 10 }, { 0, 0 }, sch_automotivebusiness, sch_carpartsshop },
    { { 0, 10 }, { 0, 0 }, sch_automotivebusiness, sch_autorental },
    { { 0, 10 }, { 0, 0 }, sch_automotivebusiness, sch_autorepair },
    { { 0, 10 }, { 0, 0 }, sch_automotivebusiness, sch_autowash },
    { { 0, 10 }, { 0, 0 }, sch_automotivebusiness, sch_petrolstation },
    { { 0, 10 }, { 0, 0 }, sch_automotivebusiness, sch_motorcycledealer },
    { { 0, 10 }, { 0, 0 }, sch_automotivebusiness, sch_motorcyclerepair },
    { { 3, 0 }, { 0, 0 }, sch_bankaccount, sch_depositaccount },
    { { 13, 0 }, { 0, 0 }, sch_biochementity, sch_chemicalsubstance },
    { { 13, 0 }, { 0, 0 }, sch_biochementity, sch_gene },
    { { 13, 0 }, { 0, 0 }, sch_biochementity, sch_molecularentity },
    { { 13, 0 }, { 0, 0 }, sch_biochementity, sch_protein },
    { { 2, 1 }, { 0, 0 }, sch_blogposting, sch_liveblogposting },
    { { 3, 5 }, { 0, 0 }, sch_boardingpolicytype, sch_groupboardingpolicy },
    { { 3, 5 }, { 0, 0 }, sch_boardingpolicytype, sch_zoneboardingpolicy },
    { { 12, 0 }, { 0, 0 }, sch_bodymeasurementtypeenumeration, sch_bodymeasurementarm },
    { { 12, 0 }, { 0, 0 }, sch_bodymeasurementtypeenumeration, sch_bodymeasurementbust },
    { { 12, 0 }, { 0, 0 }, sch_bodymeasurementtypeenumeration, sch_bodymeasurementchest },
    { { 12, 0 }, { 0, 0 }, sch_bodymeasurementtypeenumeration, sch_bodymeasurementfoot },
    { { 12, 0 }, { 0, 0 }, sch_bodymeasurementtypeenumeration, sch_bodymeasurementhand },
    { { 12, 0 }, { 0, 0 }, sch_bodymeasurementtypeenumeration, sch_bodymeasurementhead },
    { { 12, 0 }, { 0, 0 }, sch_bodymeasurementtypeenumeration, sch_bodymeasurementheight },
    { { 12, 0 }, { 0, 0 }, sch_bodymeasurementtypeenumeration, sch_bodymeasurementhips },
    { { 12, 0 }, { 0, 0 }, sch_bodymeasurementtypeenumeration, sch_bodymeasurementinsideleg },
    { { 12, 0 }, { 0, 0 }, sch_bodymeasurementtypeenumeration, sch_bodymeasurementneck },
    { { 12, 0 }, { 0, 0 }, sch_bodymeasurementtypeenumeration, sch_bodymeasurementunderbust },
    { { 12, 0 }, { 0, 0 }, sch_bodymeasurementtypeenumeration, sch_bodymeasurementwaist },
    { { 12, 0 }, { 0, 0 }, sch_bodymeasurementtypeenumeration, sch_bodymeasurementweight },
    { { 0, 10 }, { 0, 0 }, sch_bodyofwater, sch_canal },
    { { 0, 10 }, { 0, 0 }, sch_bodyofwater, sch_lakebodyofwater },
    { { 0, 10 }, { 0, 0 }, sch_bodyofwater, sch_oceanbodyofwater },
    { { 0, 10 }, { 0, 0 }, sch_bodyofwater, sch_pond },
    { { 0, 10 }, { 0, 0 }, sch_bodyofwater, sch_reservoir },
    { { 0, 10 }, { 0, 0 }, sch_bodyofwater, sch_riverbodyofwater },
    { { 0, 10 }, { 0, 0 }, sch_bodyofwater, sch_seabodyofwater },
    { { 0, 10 }, { 0, 0 }, sch_bodyofwater, sch_waterfall },
    { { 1, 92 }, { 0, 0 }, sch_boolean, sch_false },
    { { 1, 92 }, { 0, 0 }, sch_boolean, sch_true },
    { { 3, 5 }, { 0, 0 }, sch_book, sch_audiobook },
    { { 3, 5 }, { 0, 0 }, sch_bookformattype, sch_audiobookformat },
    { { 0, 10 }, { 0, 0 }, sch_bookformattype, sch_ebook },
    { { 3, 5 }, { 0, 0 }, sch_bookformattype, sch_graphicnovel },
    { { 0, 10 }, { 0, 0 }, sch_bookformattype, sch_hardcover },
    { { 0, 10 }, { 0, 0 }, sch_bookformattype, sch_paperback },
    { { 2, 0 }, { 0, 0 }, sch_broadcastchannel, sch_radiochannel },
    { { 2, 0 }, { 0, 0 }, sch_broadcastchannel, sch_televisionchannel },
    { { 3, 5 }, { 0, 0 }, sch_broadcastservice, sch_radiobroadcastservice },
    { { 1, 92 }, { 0, 0 }, sch_businessentitytype, gr_business },
    { { 1, 92 }, { 0, 0 }, sch_businessentitytype, gr_enduser },
    { { 1, 92 }, { 0, 0 }, sch_businessentitytype, gr_publicinstitution },
    { { 1, 92 }, { 0, 0 }, sch_businessentitytype, gr_reseller },
    { { 1, 92 }, { 0, 0 }, sch_businessfunction, gr_constructioninstallation },
    { { 1, 92 }, { 0, 0 }, sch_businessfunction, gr_dispose },
    { { 1, 92 }, { 0, 0 }, sch_businessfunction, gr_leaseout },
    { { 1, 92 }, { 0, 0 }, sch_businessfunction, gr_maintain },
    { { 1, 92 }, { 0, 0 }, sch_businessfunction, gr_provideservice },
    { { 1, 92 }, { 0, 0 }, sch_businessfunction, gr_repair },
    { { 1, 92 }, { 0, 0 }, sch_businessfunction, gr_sell },
    { { 1, 92 }, { 0, 0 }, sch_businessfunction, gr_buy },
    { { 1, 92 }, { 0, 0 }, sch_businessfunction, gr_licence },
    { { 3, 6 }, { 5, 0 }, sch_carusagetype, sch_drivingschoolvehicleusage },
    { { 3, 6 }, { 5, 0 }, sch_carusagetype, sch_rentalvehicleusage },
    { { 3, 6 }, { 5, 0 }, sch_carusagetype, sch_taxivehicleusage },
    { { 3, 5 }, { 0, 0 }, sch_categorycode, sch_medicalcode },
    { { 1, 2 }, { 0, 0 }, sch_chooseaction, sch_voteaction },
    { { 3, 6 }, { 0, 0 }, sch_church, sch_catholicchurch },
    { { 0, 10 }, { 0, 0 }, sch_civicstructure, sch_airport },
    { { 0, 10 }, { 0, 0 }, sch_civicstructure, sch_aquarium },
    { { 0, 10 }, { 0, 0 }, sch_civicstructure, sch_beach },
    { { 9, 0 }, { 0, 0 }, sch_civicstructure, sch_boatterminal },
    { { 2, 1 }, { 0, 0 }, sch_civicstructure, sch_bridge },
    { { 0, 10 }, { 0, 0 }, sch_civicstructure, sch_busstation },
    { { 0, 10 }, { 0, 0 }, sch_civicstructure, sch_busstop },
    { { 0, 10 }, { 0, 0 }, sch_civicstructure, sch_campground },
    { { 0, 10 }, { 0, 0 }, sch_civicstructure, sch_cemetery },
    { { 0, 10 }, { 0, 0 }, sch_civicstructure, sch_crematorium },
    { { 7, 4 }, { 0, 0 }, sch_civicstructure, sch_educationorganisation },
    { { 0, 10 }, { 0, 0 }, sch_civicstructure, sch_eventvenue },
    { { 0, 10 }, { 0, 0 }, sch_civicstructure, sch_firestation },
    { { 0, 10 }, { 0, 0 }, sch_civicstructure, sch_governmentbuilding },
    { { 0, 10 }, { 0, 0 }, sch_civicstructure, sch_hospital },
    { { 0, 10 }, { 0, 0 }, sch_civicstructure, sch_movietheatre },
    { { 0, 10 }, { 0, 0 }, sch_civicstructure, sch_museum },
    { { 0, 10 }, { 0, 0 }, sch_civicstructure, sch_musicvenue },
    { { 0, 10 }, { 0, 0 }, sch_civicstructure, sch_park },
    { { 0, 10 }, { 0, 0 }, sch_civicstructure, sch_parkingfacility },
    { { 0, 10 }, { 0, 0 }, sch_civicstructure, sch_performingartstheatre },
    { { 0, 10 }, { 0, 0 }, sch_civicstructure, sch_placeofworship },
    { { 0, 10 }, { 0, 0 }, sch_civicstructure, sch_playground },
    { { 0, 10 }, { 0, 0 }, sch_civicstructure, sch_policestation },
    { { 3, 5 }, { 0, 0 }, sch_civicstructure, sch_publictoilet },
    { { 0, 10 }, { 0, 0 }, sch_civicstructure, sch_rvpark },
    { { 0, 10 }, { 0, 0 }, sch_civicstructure, sch_stadiumorarena },
    { { 0, 10 }, { 0, 0 }, sch_civicstructure, sch_metrostation },
    { { 0, 10 }, { 0, 0 }, sch_civicstructure, sch_taxistand },
    { { 0, 10 }, { 0, 0 }, sch_civicstructure, sch_trainstation },
    { { 0, 10 }, { 0, 0 }, sch_civicstructure, sch_zoo },
    { { 2, 0 }, { 0, 0 }, sch_clip, sch_movieclip },
    { { 1, 3 }, { 0, 0 }, sch_clip, sch_radioclip },
    { { 1, 3 }, { 0, 0 }, sch_clip, sch_tvclip },
    { { 2, 0 }, { 0, 0 }, sch_clip, sch_videogameclip },
    { { 0, 10 }, { 5, 0 }, sch_collectionpage, sch_imagegallery },
    { { 6, 0 }, { 0, 0 }, sch_collectionpage, sch_mediagallery },
    { { 0, 10 }, { 5, 0 }, sch_collectionpage, sch_videogallery },
    { { 3, 5 }, { 0, 0 }, sch_comicstory, sch_comiccoverart },
    { { 2, 0 }, { 0, 0 }, sch_comment, sch_answer },
    { { 3, 5 }, { 0, 0 }, sch_comment, sch_correctioncomment },
    { { 10, 0 }, { 0, 0 }, sch_comment, sch_question },
    { { 1, 2 }, { 0, 0 }, sch_communicateaction, sch_askaction },
    { { 1, 2 }, { 0, 0 }, sch_communicateaction, sch_checkinaction },
    { { 1, 2 }, { 0, 0 }, sch_communicateaction, sch_checkoutaction },
    { { 1, 2 }, { 0, 0 }, sch_communicateaction, sch_commentaction },
    { { 1, 2 }, { 0, 0 }, sch_communicateaction, sch_informaction },
    { { 1, 2 }, { 0, 0 }, sch_communicateaction, sch_inviteaction },
    { { 1, 2 }, { 0, 0 }, sch_communicateaction, sch_replyaction },
    { { 1, 2 }, { 0, 0 }, sch_communicateaction, sch_shareaction },
    { { 1, 2 }, { 0, 0 }, sch_consumeaction, sch_drinkaction },
    { { 1, 2 }, { 0, 0 }, sch_consumeaction, sch_eataction },
    { { 14, 0 }, { 0, 0 }, sch_consumeaction, sch_playgameaction },
    { { 1, 2 }, { 0, 0 }, sch_consumeaction, sch_installaction },
    { { 1, 2 }, { 0, 0 }, sch_consumeaction, sch_listenaction },
    { { 1, 2 }, { 0, 0 }, sch_consumeaction, sch_readaction },
    { { 1, 2 }, { 0, 0 }, sch_consumeaction, sch_useaction },
    { { 1, 2 }, { 0, 0 }, sch_consumeaction, sch_viewaction },
    { { 1, 2 }, { 0, 0 }, sch_consumeaction, sch_watchaction },
    { { 0, 10 }, { 0, 0 }, sch_contactpoint, sch_postaladdress },
    { { 1, 3 }, { 0, 0 }, sch_contactpointoption, sch_hearingimpairedsupported },
    { { 1, 3 }, { 0, 0 }, sch_contactpointoption, sch_tollfree },
    { { 1, 92 }, { 0, 0 }, sch_controlaction, sch_activateaction },
    { { 1, 92 }, { 0, 0 }, sch_controlaction, sch_deactivateaction },
    { { 1, 92 }, { 0, 0 }, sch_controlaction, sch_resumeaction },
    { { 1, 92 }, { 0, 0 }, sch_controlaction, sch_suspendaction },
    { { 3, 5 }, { 0, 0 }, sch_coverart, sch_comiccoverart },
    { { 1, 2 }, { 0, 0 }, sch_createaction, sch_cookaction },
    { { 1, 2 }, { 0, 0 }, sch_createaction, sch_drawaction },
    { { 1, 2 }, { 0, 0 }, sch_createaction, sch_filmaction },
    { { 1, 2 }, { 0, 0 }, sch_createaction, sch_paintaction },
    { { 1, 2 }, { 0, 0 }, sch_createaction, sch_photographaction },
    { { 1, 2 }, { 0, 0 }, sch_createaction, sch_writeaction },
    { { 3, 5 }, { 7, 2 }, sch_creativework, sch_3dmodel },
    { { 11, 0 }, { 0, 0 }, sch_creativework, sch_ampstory },
    { { 1, 20 }, { 1, 93 }, sch_creativework, sch_answer },
    { { 3, 5 }, { 0, 0 }, sch_creativework, sch_archivecomponent },
    { { 0, 10 }, { 0, 0 }, sch_creativework, sch_article },
    { { 3, 5 }, { 0, 0 }, sch_creativework, sch_atlas },
    { { 0, 10 }, { 1, 80 }, sch_creativework, sch_blog },
    { { 0, 10 }, { 0, 0 }, sch_creativework, sch_book },
    { { 3, 5 }, { 0, 0 }, sch_creativework, sch_chapter },
    { { 3, 5 }, { 0, 0 }, sch_creativework, sch_claim },
    { { 1, 3 }, { 0, 0 }, sch_creativework, sch_clip },
    { { 1, 0 }, { 2, 1 }, sch_creativework, sch_code },
    { { 3, 5 }, { 0, 0 }, sch_creativework, sch_collection },
    { { 3, 5 }, { 0, 0 }, sch_creativework, sch_comicstory },
    { { 0, 95 }, { 0, 0 }, sch_creativework, sch_comment },
    { { 3, 0 }, { 0, 0 }, sch_creativework, sch_conversation },
    { { 3, 2 }, { 0, 0 }, sch_creativework, sch_course },
    { { 2, 0 }, { 0, 0 }, sch_creativework, sch_creativeworkseason },
    { { 2, 0 }, { 0, 0 }, sch_creativework, sch_creativeworkseries },
    { { 1, 0 }, { 0, 0 }, sch_creativework, sch_datacatalogue },
    { { 1, 0 }, { 0, 0 }, sch_creativework, sch_dataset },
    { { 3, 5 }, { 0, 0 }, sch_creativework, sch_definedtermset },
    { { 0, 95 }, { 2, 1 }, sch_creativework, sch_diet },
    { { 3, 5 }, { 0, 0 }, sch_creativework, sch_diet },
    { { 3, 0 }, { 0, 0 }, sch_creativework, sch_digitaldocument },
    { { 3, 5 }, { 0, 0 }, sch_creativework, sch_drawing },
    { { 1, 10 }, { 2, 1 }, sch_creativework, sch_emailmessage },
    { { 3, 5 }, { 0, 0 }, sch_creativework, sch_educationoccupationcredential },
    { { 1, 3 }, { 0, 0 }, sch_creativework, sch_episode },
    { { 0, 95 }, { 2, 1 }, sch_creativework, sch_exerciseplan },
    { { 3, 5 }, { 0, 0 }, sch_creativework, sch_exerciseplan },
    { { 13, 0 }, { 0, 0 }, sch_creativework, poetry_form },
    { { 2, 0 }, { 0, 0 }, sch_creativework, sch_game },
    { { 6, 0 }, { 0, 0 }, sch_creativework, sch_guide },
    { { 3, 3 }, { 0, 0 }, sch_creativework, sch_howto },
    { { 3, 4 }, { 0, 0 }, sch_creativework, sch_howtodirection },
    { { 3, 4 }, { 0, 0 }, sch_creativework, sch_howtosection },
    { { 3, 4 }, { 0, 0 }, sch_creativework, sch_howtostep },
    { { 3, 4 }, { 0, 0 }, sch_creativework, sch_howtotip },
    { { 11, 0 }, { 0, 0 }, sch_creativework, sch_hypertoc },
    { { 11, 0 }, { 0, 0 }, sch_creativework, sch_hypertocentry },
    { { 0, 10 }, { 1, 91 }, sch_creativework, sch_itemlist },
    { { 9, 0 }, { 0, 0 }, sch_creativework, sch_learningresource },
    { { 3, 5 }, { 0, 0 }, sch_creativework, sch_legislation },
    { { 3, 5 }, { 0, 0 }, sch_creativework, sch_manuscript },
    { { 0, 10 }, { 0, 0 }, sch_creativework, sch_map },
    { { 11, 0 }, { 0, 0 }, sch_creativework, sch_mathsolver },
    { { 0, 10 }, { 0, 0 }, sch_creativework, sch_mediaobject },
    { { 13, 0 }, { 0, 0 }, sch_creativework, sch_mediareviewitem },
    { { 3, 2 }, { 0, 0 }, sch_creativework, sch_menu },
    { { 3, 2 }, { 0, 0 }, sch_creativework, sch_menusection },
    { { 3, 0 }, { 0, 0 }, sch_creativework, sch_message },
    { { 0, 10 }, { 0, 0 }, sch_creativework, sch_movie },
    { { 2, 0 }, { 0, 0 }, sch_creativework, sch_musiccomposition },
    { { 0, 10 }, { 0, 0 }, sch_creativework, sch_musicplaylist },
    { { 0, 10 }, { 0, 0 }, sch_creativework, sch_musicrecording },
    { { 0, 10 }, { 0, 0 }, sch_creativework, sch_painting },
    { { 1, 90 }, { 0, 0 }, sch_creativework, sch_periodical },
    { { 0, 10 }, { 0, 0 }, sch_creativework, sch_photograph },
    { { 3, 5 }, { 0, 0 }, sch_creativework, sch_play },
    { { 13, 0 }, { 0, 0 }, sch_creativework, poetry_poem },
    { { 3, 5 }, { 0, 0 }, sch_creativework, sch_poster },
    { { 1, 90 }, { 0, 0 }, sch_creativework, sch_publicationissue },
    { { 1, 90 }, { 0, 0 }, sch_creativework, sch_publicationvolume },
    { { 1, 20 }, { 9, 0 }, sch_creativework, sch_question },
    { { 3, 5 }, { 0, 0 }, sch_creativework, sch_quotation },
    { { 0, 10 }, { 3, 2 }, sch_creativework, sch_recipe },
    { { 0, 10 }, { 0, 0 }, sch_creativework, sch_review },
    { { 0, 10 }, { 0, 0 }, sch_creativework, sch_sculpture },
    { { 3, 5 }, { 0, 0 }, sch_creativework, sch_sheetmusic },
    { { 3, 5 }, { 0, 0 }, sch_creativework, sch_shortstory },
    { { 1, 3 }, { 2, 1 }, sch_creativework, sch_season },
    { { 1, 3 }, { 3, 3 }, sch_creativework, sch_series },
    { { 0, 95 }, { 0, 0 }, sch_creativework, sch_softwareapplication },
    { { 2, 0 }, { 0, 0 }, sch_creativework, sch_softwaresourcecode },
    { { 7, 0 }, { 0, 0 }, sch_creativework, sch_specialannouncement },
    { { 13, 0 }, { 0, 0 }, sch_creativework, sch_statement },
    { { 0, 10 }, { 1, 2 }, sch_creativework, sch_tvepisode},
    { { 0, 10 }, { 1, 93 }, sch_creativework, sch_tvseason },
    { { 0, 10 }, { 0, 0 }, sch_creativework, sch_tvseries },
    { { 3, 5 }, { 0, 0 }, sch_creativework, sch_thesis },
    { { 1, 93 }, { 0, 0 }, sch_creativework, sch_visualartwork },
    { { 5, 0 }, { 0, 0 }, sch_creativework, sch_webcontent },
    { { 0, 10 }, { 0, 0 }, sch_creativework, sch_webpage },
    { { 0, 10 }, { 0, 0 }, sch_creativework, sch_webpageelement },
    { { 2, 0 }, { 0, 0 }, sch_creativework, sch_website },
    { { 4, 0 }, { 0, 0 }, sch_creativeworkseason, sch_podcastseason },
    { { 2, 0 }, { 0, 0 }, sch_creativeworkseason, sch_radioseason },
    { { 2, 0 }, { 0, 0 }, sch_creativeworkseason, sch_tvseason },
    { { 2, 0 }, { 0, 0 }, sch_creativeworkseries, sch_bookseries },
    { { 2, 0 }, { 0, 0 }, sch_creativeworkseries, sch_movieseries },
    { { 2, 0 }, { 0, 0 }, sch_creativeworkseries, sch_periodical },
    { { 4, 0 }, { 0, 0 }, sch_creativeworkseries, sch_podcastseries },
    { { 2, 0 }, { 0, 0 }, sch_creativeworkseries, sch_radioseries },
    { { 2, 0 }, { 0, 0 }, sch_creativeworkseries, sch_tvseries },
    { { 2, 0 }, { 0, 0 }, sch_creativeworkseries, sch_videogameseries },
    { { 0, 99 }, { 0, 0 }, sch_creditcard, gr_americanexpress },
    { { 0, 99 }, { 0, 0 }, sch_creditcard, gr_dinersclub },
    { { 0, 99 }, { 0, 0 }, sch_creditcard, gr_discover },
    { { 0, 99 }, { 0, 0 }, sch_creditcard, gr_jcb },
    { { 0, 99 }, { 0, 0 }, sch_creditcard, gr_mastercard },
    { { 0, 99 }, { 0, 0 }, sch_creditcard, gr_visa },
    { { 3, 5 }, { 0, 0 }, sch_criticreview, sch_reviewnewsarticle },
    { { 3, 5 }, { 0, 0 }, sch_datafeed, sch_completedatafeed },
    { { 2, 2 }, { 0, 0 }, sch_dataset, sch_datafeed },
    { { 0, 10 }, { 0, 0 }, sch_datatype, sch_boolean },
    { { 0, 10 }, { 0, 0 }, sch_datatype, sch_date },
    { { 0, 95 }, { 0, 0 }, sch_datatype, sch_datetime },
    { { 0, 10 }, { 0, 0 }, sch_datatype, sch_number },
    { { 0, 10 }, { 0, 0 }, sch_datatype, sch_text },
    { { 0, 95 }, { 0, 0 }, sch_datatype, sch_time },
    { { 0, 99 }, { 2, 2 }, sch_dayofweek, gr_friday },
    { { 0, 99 }, { 2, 2 }, sch_dayofweek, gr_monday },
    { { 0, 99 }, { 2, 2 }, sch_dayofweek, gr_publicholidays },
    { { 0, 99 }, { 2, 2 }, sch_dayofweek, gr_saturday },
    { { 0, 99 }, { 2, 2 }, sch_dayofweek, gr_sunday },
    { { 0, 99 }, { 2, 2 }, sch_dayofweek, gr_thursday },
    { { 0, 99 }, { 2, 2 }, sch_dayofweek, gr_tuesday },
    { { 0, 99 }, { 2, 2 }, sch_dayofweek, gr_wednesday },
    { { 3, 0 }, { 0, 0 }, sch_dayofweek, sch_friday },
    { { 3, 0 }, { 0, 0 }, sch_dayofweek, sch_monday },
    { { 3, 0 }, { 0, 0 }, sch_dayofweek, sch_publicholidays },
    { { 3, 0 }, { 0, 0 }, sch_dayofweek, sch_saturday },
    { { 3, 0 }, { 0, 0 }, sch_dayofweek, sch_sunday },
    { { 3, 0 }, { 0, 0 }, sch_dayofweek, sch_thursday },
    { { 3, 0 }, { 0, 0 }, sch_dayofweek, sch_tuesday },
    { { 3, 0 }, { 0, 0 }, sch_dayofweek, sch_wednesday },
    { { 3, 5 }, { 0, 0 }, sch_definedterm, sch_categorycode },
    { { 3, 5 }, { 0, 0 }, sch_definedtermset, sch_categorycodeset },
    { { 1, 3 }, { 0, 0 }, sch_deliverymethod, sch_lockerdelivery },
    { { 1, 3 }, { 1, 91 }, sch_deliverymethod, sch_onsitepickup },
    { { 3, 5 }, { 0, 0 }, sch_deliverymethod, sch_onsitepickup },
    { { 0, 99 }, { 0, 0 }, sch_deliverymethod, sch_parcelservice },
    { { 3, 0 }, { 0, 0 }, sch_digitaldocument, sch_notedigitaldocument },
    { { 3, 0 }, { 0, 0 }, sch_digitaldocument, sch_presentationdigitaldocument },
    { { 3, 0 }, { 0, 0 }, sch_digitaldocument, sch_spreadsheetdigitaldocument },
    { { 3, 0 }, { 0, 0 }, sch_digitaldocument, sch_textdigitaldocument },
    { { 3, 5 }, { 0, 0 }, sch_digitaldocumentpermissiontype, sch_commentpermission },
    { { 3, 5 }, { 0, 0 }, sch_digitaldocumentpermissiontype, sch_readpermission },
    { { 3, 5 }, { 0, 0 }, sch_digitaldocumentpermissiontype, sch_writepermission },
    { { 14, 0 }, { 0, 0 }, sch_digitalplatformenumeration, sch_androidplatform },
    { { 14, 0 }, { 0, 0 }, sch_digitalplatformenumeration, sch_desktopwebplatform },
    { { 14, 0 }, { 0, 0 }, sch_digitalplatformenumeration, sch_genericwebplatform },
    { { 14, 0 }, { 0, 0 }, sch_digitalplatformenumeration, sch_iosplatform },
    { { 14, 0 }, { 0, 0 }, sch_digitalplatformenumeration, sch_mobilewebplatform },
    { { 0, 95 }, { 0, 0 }, sch_doseschedule, sch_maximumdoseschedule },
    { { 0, 95 }, { 0, 0 }, sch_doseschedule, sch_recommendeddoseschedule },
    { { 0, 95 }, { 0, 0 }, sch_doseschedule, sch_reporteddoseschedule },
    { { 3, 6 }, { 5, 0 }, sch_drivewheelconfigurationvalue, sch_allwheeldriveconfiguration },
    { { 3, 6 }, { 5, 0 }, sch_drivewheelconfigurationvalue, sch_fourwheeldriveconfiguration },
    { { 3, 6 }, { 5, 0 }, sch_drivewheelconfigurationvalue, sch_frontwheeldriveconfiguration },
    { { 3, 6 }, { 5, 0 }, sch_drivewheelconfigurationvalue, sch_rearwheeldriveconfiguration },
    { { 9, 0 }, { 0, 0 }, sch_drivewheelconfigurationvalue, sch_allwheeldriveconfiguration },
    { { 9, 0 }, { 0, 0 }, sch_drivewheelconfigurationvalue, sch_fourwheeldriveconfiguration },
    { { 9, 0 }, { 0, 0 }, sch_drivewheelconfigurationvalue, sch_frontwheeldriveconfiguration },
    { { 9, 0 }, { 0, 0 }, sch_drivewheelconfigurationvalue, sch_rearwheeldriveconfiguration },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_drugcostcategory, sch_reimbursementcap },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_drugcostcategory, sch_retail },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_drugcostcategory, sch_wholesale },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_drugpregnancycategory, sch_fdacategorya },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_drugpregnancycategory, sch_fdacategoryb },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_drugpregnancycategory, sch_fdacategoryc },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_drugpregnancycategory, sch_fdacategoryd },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_drugpregnancycategory, sch_fdacategoryx },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_drugpregnancycategory, sch_fdanotevaluated },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_drugprescriptionstatus, sch_otc },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_drugprescriptionstatus, sch_prescriptiononly },
    { { 3, 9 }, { 0, 0 }, sch_educationoccupationprogramme, sch_workbasedprogramme },
    { { 0, 10 }, { 0, 0 }, sch_educationorganisation, sch_collegeoruniversity },
    { { 0, 10 }, { 0, 0 }, sch_educationorganisation, sch_elementaryschool },
    { { 0, 10 }, { 0, 0 }, sch_educationorganisation, sch_highschool },
    { { 0, 10 }, { 0, 0 }, sch_educationorganisation, sch_middleschool },
    { { 0, 10 }, { 0, 0 }, sch_educationorganisation, sch_preschool },
    { { 0, 10 }, { 0, 0 }, sch_educationorganisation, sch_school },
    { { 0, 10 }, { 0, 0 }, sch_emergencyservice, sch_firestation },
    { { 0, 10 }, { 0, 0 }, sch_emergencyservice, sch_hospital },
    { { 0, 10 }, { 0, 0 }, sch_emergencyservice, sch_policestation },
    { { 10, 0 }, { 0, 0 }, sch_energyefficiencyenumeration, sch_energystarcertified },
    { { 10, 0 }, { 0, 0 }, sch_energyefficiencyenumeration, sch_euenergyefficiencyenumeration },
    { { 0, 10 }, { 0, 0 }, sch_entertainmentbusiness, sch_adultentertainment },
    { { 0, 10 }, { 0, 0 }, sch_entertainmentbusiness, sch_amusementpark },
    { { 0, 10 }, { 0, 0 }, sch_entertainmentbusiness, sch_artgallery },
    { { 0, 10 }, { 0, 0 }, sch_entertainmentbusiness, sch_casino },
    { { 0, 10 }, { 0, 0 }, sch_entertainmentbusiness, sch_comedyclub },
    { { 0, 10 }, { 0, 0 }, sch_entertainmentbusiness, sch_movietheatre },
    { { 0, 10 }, { 0, 0 }, sch_entertainmentbusiness, sch_nightclub },
    { { 1, 20 }, { 8, 0 }, sch_enumeration, sch_actionstatustype },
    { { 14, 0 }, { 0, 0 }, sch_enumeration, sch_adultorientatedenumeration },
    { { 2, 0 }, { 0, 0 }, sch_enumeration, sch_boardingpolicytype },
    { { 0, 10 }, { 0, 0 }, sch_enumeration, sch_bookformattype },
    { { 0, 99 }, { 0, 0 }, sch_enumeration, sch_businessentitytype },
    { { 0, 99 }, { 0, 0 }, sch_enumeration, sch_businessfunction },
    { { 9, 0 }, { 0, 0 }, sch_enumeration, sch_carusagetype },
    { { 1, 3 }, { 0, 0 }, sch_enumeration, sch_contactpointoption },
    { { 0, 99 }, { 0, 0 }, sch_enumeration, sch_dayofweek }, 
    { { 0, 99 }, { 0, 0 }, sch_enumeration, sch_deliverymethod },
    { { 1, 92 }, { 2, 1 }, sch_enumeration, sch_drugcostcategory },
    { { 1, 92 }, { 2, 1 }, sch_enumeration, sch_drugpregnancycategory },
    { { 1, 92 }, { 2, 1 }, sch_enumeration, sch_drugprescriptionstatus },
    { { 3, 0 }, { 0, 0 }, sch_enumeration, sch_digitaldocumentpermissiontype },
    { { 14, 0 }, { 0, 0 }, sch_enumeration, sch_digitalplatformenumeration },
    { { 10, 0 }, { 0, 0 }, sch_enumeration, sch_energyefficiencyenumeration },
    { { 7, 0 }, { 0, 0 }, sch_enumeration, sch_eventattendancemodeenumeration },
    { { 1, 3 }, { 8, 0 }, sch_enumeration, sch_eventstatustype },
    { { 14, 0 }, { 0, 0 }, sch_enumeration, sch_gameavailabilityenumeration },
    { { 1, 92 }, { 0, 0 }, sch_enumeration, sch_gameplaymode },
    { { 1, 92 }, { 8, 0 }, sch_enumeration, sch_gameserverstatus },
    { { 3, 0 }, { 0, 0 }, sch_enumeration, sch_gendertype },
    { { 9, 0 }, { 0, 0 }, sch_enumeration, sch_governmentbenefitstype },
    { { 5, 0 }, { 0, 0 }, sch_enumeration, sch_healthaspectenumeration },
    { { 1, 92 }, { 2, 1 }, sch_enumeration, sch_infectiousagentclass },
    { { 0, 10 }, { 0, 0 }, sch_enumeration, sch_itemavailability },
    { { 1, 92 }, { 0, 0 }, sch_enumeration, sch_itemlistordertype },
    { { 3, 5 }, { 8, 0 }, sch_enumeration, sch_legalforcestatus },
    { { 3, 5 }, { 0, 0 }, sch_enumeration, sch_legalvaluelevel },
    { { 1, 92 }, { 0, 0 }, sch_enumeration, sch_mapcategorytype },
    { { 7, 0 }, { 0, 0 }, sch_enumeration, sch_mediamanipulationratingenumeration },
    { { 1, 92 }, { 2, 1 }, sch_enumeration, sch_medicalaudience },
    { { 1, 92 }, { 2, 1 }, sch_enumeration, sch_medicaldevicepurpose },
    { { 1, 92 }, { 2, 1 }, sch_enumeration, sch_medicalenumeration },
    { { 3, 5 }, { 0, 0 }, sch_enumeration, sch_medicalenumeration },
    { { 1, 92 }, { 2, 1 }, sch_enumeration, sch_medicalevidencelevel },
    { { 1, 92 }, { 2, 1 }, sch_enumeration, sch_medicalimagingtechnique },
    { { 1, 92 }, { 2, 1 }, sch_enumeration, sch_medicalobservationalstudydesign },
    { { 1, 92 }, { 2, 1 }, sch_enumeration, sch_medicalproceduretype },
    { { 1, 92 }, { 2, 1 }, sch_enumeration, sch_medicalspeciality },
    { { 1, 92 }, { 2, 1 }, sch_enumeration, sch_medicalstudystatus },
    { { 1, 92 }, { 2, 1 }, sch_enumeration, sch_medicaltrialdesign },
    { { 3, 5 }, { 0, 0 }, sch_enumeration, sch_medicaltrialdesign },
    { { 1, 92 }, { 2, 1 }, sch_enumeration, sch_medicinesystem },
    { { 6, 0 }, { 0, 0 }, sch_enumeration, sch_merchantreturnenumeration },
    { { 1, 92 }, { 0, 0 }, sch_enumeration, sch_musicalbumproductiontype },
    { { 1, 92 }, { 0, 0 }, sch_enumeration, sch_musicalbumreleasetype },
    { { 1, 92 }, { 0, 0 }, sch_enumeration, sch_musicreleaseformattype },
    { { 8, 0 }, { 0, 0 }, sch_enumeration, sch_nonprofittype },
    { { 0, 10 }, { 0, 0 }, sch_enumeration, sch_offeritemcondition },
    { { 1, 92 }, { 8, 0 }, sch_enumeration, sch_orderstatus },
    { { 0, 99 }, { 0, 0 }, sch_enumeration, sch_paymentmethod },
    { { 2, 1 }, { 8, 0 }, sch_enumeration, sch_paymentstatustype },
    { { 1, 92 }, { 2, 1 }, sch_enumeration, sch_physicalactivitycategory },
    { { 3, 5 }, { 0, 0 }, sch_enumeration, sch_physicalactivitycategory },
    { { 1, 92 }, { 2, 1 }, sch_enumeration, sch_physicalexam },
    { { 3, 8 }, { 5, 0 }, sch_enumeration, sch_productreturnenumeration },
    { { 0, 99 }, { 0, 0 }, sch_enumeration, sch_qualitativevalue },
    { { 3, 8 }, { 0, 0 }, sch_enumeration, sch_refundtypeenumeration },
    { { 1, 92 }, { 0, 0 }, sch_enumeration, sch_reservationstatustype },
    { { 3, 0 }, { 0, 0 }, sch_enumeration, sch_restricteddiet },
    { { 3, 8 }, { 0, 0 }, sch_enumeration, sch_returnfeesenumeration },
    { { 1, 92 }, { 0, 0 }, sch_enumeration, sch_rsvpresponsetype },
    { { 12, 0 }, { 0, 0 }, sch_enumeration, sch_sizegroupenumeration },
    { { 12, 0 }, { 0, 0 }, sch_enumeration, sch_sizesystemenumeration },
    { { 0, 95 }, { 0, 0 }, sch_enumeration, sch_speciality },
    { { 9, 0 }, { 0, 0 }, sch_enumeration, sch_statusenumeration },
    { { 0, 99 }, { 0, 0 }, sch_enumeration, sch_warrantyscope },
    { { 12, 0 }, { 0, 0 }, sch_enumeration, sch_wearablesizesystemenumeration },
    { { 4, 0 }, { 0, 0 }, sch_episode, sch_podcastepisode },
    { { 1, 3 }, { 0, 0 }, sch_episode, sch_radioepisode },
    { { 1, 3 }, { 0, 0 }, sch_episode, sch_tvepisode },
    { { 10, 0 }, { 0, 0 }, sch_euenergyefficiencyenumeration, sch_euenergyefficiencycategorya },
    { { 10, 0 }, { 0, 0 }, sch_euenergyefficiencyenumeration, sch_euenergyefficiencycategorya1plus },
    { { 10, 0 }, { 0, 0 }, sch_euenergyefficiencyenumeration, sch_euenergyefficiencycategorya2plus },
    { { 10, 0 }, { 0, 0 }, sch_euenergyefficiencyenumeration, sch_euenergyefficiencycategorya3plus },
    { { 10, 0 }, { 0, 0 }, sch_euenergyefficiencyenumeration, sch_euenergyefficiencycategoryb },
    { { 10, 0 }, { 0, 0 }, sch_euenergyefficiencyenumeration, sch_euenergyefficiencycategoryc },
    { { 10, 0 }, { 0, 0 }, sch_euenergyefficiencyenumeration, sch_euenergyefficiencycategoryd },
    { { 10, 0 }, { 0, 0 }, sch_euenergyefficiencyenumeration, sch_euenergyefficiencycategorye },
    { { 10, 0 }, { 0, 0 }, sch_euenergyefficiencyenumeration, sch_euenergyefficiencycategoryf },
    { { 10, 0 }, { 0, 0 }, sch_euenergyefficiencyenumeration, sch_euenergyefficiencycategoryg },
    { { 0, 10 }, { 0, 0 }, sch_event, sch_businessevent },
    { { 0, 10 }, { 0, 0 }, sch_event, sch_childrensevent },
    { { 0, 10 }, { 0, 0 }, sch_event, sch_comedyevent },
    { { 3, 1 }, { 0, 0 }, sch_event, sch_courseinstance },
    { { 0, 10 }, { 0, 0 }, sch_event, sch_danceevent },
    { { 1, 4 }, { 0, 0 }, sch_event, sch_deliveryevent },
    { { 0, 10 }, { 0, 0 }, sch_event, sch_educationevent },
    { { 3, 5 }, { 0, 0 }, sch_event, sch_eventseries },
    { { 2, 2 }, { 0, 0 }, sch_event, sch_exhibitionevent },
    { { 0, 10 }, { 0, 0 }, sch_event, sch_festival },
    { { 0, 10 }, { 0, 0 }, sch_event, sch_foodevent },
    { { 8, 0 }, { 0, 0 }, sch_event, sch_hackathon },
    { { 0, 10 }, { 0, 0 }, sch_event, sch_literaryevent },
    { { 0, 10 }, { 0, 0 }, sch_event, sch_musicevent },
    { { 1, 3 }, { 0, 0 }, sch_event, sch_publicationevent },
    { { 0, 10 }, { 0, 0 }, sch_event, sch_saleevent },
    { { 2, 0 }, { 0, 0 }, sch_event, sch_screeningevent },
    { { 0, 10 }, { 0, 0 }, sch_event, sch_socialevent },
    { { 0, 10 }, { 0, 0 }, sch_event, sch_sportsevent },
    { { 0, 10 }, { 0, 0 }, sch_event, sch_theatreevent },
    { { 0, 10, SV_DEP_3034 }, { 3, 4 }, sch_event, sch_userinteraction },
    { { 0, 10 }, { 0, 0 }, sch_event, sch_visualartsevent },
    { { 7, 0 }, { 0, 0 }, sch_eventattendancemodeenumeration, sch_mixedeventattendancemode },
    { { 7, 0 }, { 0, 0 }, sch_eventattendancemodeenumeration, sch_offlineeventattendancemode },
    { { 7, 0 }, { 0, 0 }, sch_eventattendancemodeenumeration, sch_onlineeventattendancemode },
    { { 3, 5 }, { 0, 0 }, sch_eventstatustype, sch_eventcancelled },
    { { 7, 0 }, { 0, 0 }, sch_eventstatustype, sch_eventmovedonline },
    { { 3, 5 }, { 0, 0 }, sch_eventstatustype, sch_eventpostponed },
    { { 3, 5 }, { 0, 0 }, sch_eventstatustype, sch_eventrescheduled },
    { { 3, 5 }, { 0, 0 }, sch_eventstatustype, sch_eventscheduled },
    { { 3, 0 }, { 0, 0 }, sch_financialproduct, sch_bankaccount },
    { { 3, 1 }, { 0, 0 }, sch_financialproduct, sch_currencyconversionservice },
    { { 3, 1 }, { 0, 0 }, sch_financialproduct, sch_investmentordeposit },
    { { 3, 0 }, { 0, 0 }, sch_financialproduct, sch_loanorcredit },
    { { 3, 0 }, { 0, 0 }, sch_financialproduct, sch_paymentcard },
    { { 3, 1 }, { 0, 0 }, sch_financialproduct, sch_paymentservice },
    { { 0, 10 }, { 0, 0 }, sch_financialservice, sch_accountingservice },
    { { 0, 10 }, { 0, 0 }, sch_financialservice, sch_automatedteller },
    { { 0, 10 }, { 0, 0 }, sch_financialservice, sch_bankorcreditunion },
    { { 0, 10 }, { 0, 0 }, sch_financialservice, sch_insuranceagency },
    { { 1, 2 }, { 0, 0 }, sch_findaction, sch_checkaction },
    { { 1, 2 }, { 0, 0 }, sch_findaction, sch_discoveraction },
    { { 1, 2 }, { 0, 0 }, sch_findaction, sch_trackaction },
    { { 0, 10 }, { 0, 0 }, sch_foodestablishment, sch_bakery },
    { { 0, 10 }, { 0, 0 }, sch_foodestablishment, sch_barorpub },
    { { 0, 10 }, { 0, 0 }, sch_foodestablishment, sch_brewery },
    { { 0, 10 }, { 0, 0 }, sch_foodestablishment, sch_cafeorcoffeeshop },
    { { 0, 10 }, { 0, 0 }, sch_foodestablishment, sch_fastfoodrestaurant },
    { { 0, 10 }, { 0, 0 }, sch_foodestablishment, sch_icecreamshop },
    { { 0, 10 }, { 0, 0 }, sch_foodestablishment, sch_restaurant },
    { { 0, 10 }, { 0, 0 }, sch_foodestablishment, sch_winery },
    { { 2, 0 }, { 0, 0 }, sch_game, sch_videogame },
    { { 14, 0 }, { 0, 0 }, sch_gameavailabilityenumeration, sch_demogameavailability },
    { { 14, 0 }, { 0, 0 }, sch_gameavailabilityenumeration, sch_fullgameavailability },
    { { 1, 92 }, { 0, 0 }, sch_gameplaymode, sch_coop },
    { { 1, 92 }, { 0, 0 }, sch_gameplaymode, sch_multiplayer },
    { { 1, 92 }, { 0, 0 }, sch_gameplaymode, sch_singleplayer },
    { { 1, 92 }, { 0, 0 }, sch_gameserverstatus, sch_offlinepermanently },
    { { 1, 92 }, { 0, 0 }, sch_gameserverstatus, sch_offlinetemporarily },
    { { 1, 92 }, { 0, 0 }, sch_gameserverstatus, sch_online },
    { { 1, 92 }, { 0, 0 }, sch_gameserverstatus, sch_onlinefull },
    { { 3, 5 }, { 0, 0 }, sch_gendertype, sch_female },
    { { 3, 5 }, { 0, 0 }, sch_gendertype, sch_male },
    { { 2, 2 }, { 0, 0 }, sch_geoshape, sch_geocircle },
    { { 9, 0 }, { 0, 0 }, sch_governmentbenefitstype, sch_basicincome },
    { { 9, 0 }, { 0, 0 }, sch_governmentbenefitstype, sch_businesssupport },
    { { 9, 0 }, { 0, 0 }, sch_governmentbenefitstype, sch_disabilitysupport },
    { { 9, 0 }, { 0, 0 }, sch_governmentbenefitstype, sch_healthcare },
    { { 9, 0 }, { 0, 0 }, sch_governmentbenefitstype, sch_onetimepayments },
    { { 9, 0 }, { 0, 0 }, sch_governmentbenefitstype, sch_paidleave },
    { { 9, 0 }, { 0, 0 }, sch_governmentbenefitstype, sch_parentalsupport },
    { { 9, 0 }, { 0, 0 }, sch_governmentbenefitstype, sch_unemploymentsupport },
    { { 0, 10 }, { 0, 0 }, sch_governmentbuilding, sch_townhall },
    { { 0, 10 }, { 0, 0 }, sch_governmentbuilding, sch_courthouse },
    { { 0, 10 }, { 0, 0 }, sch_governmentbuilding, sch_defenceestablishment },
    { { 0, 10 }, { 0, 0 }, sch_governmentbuilding, sch_embassy },
    { { 0, 10 }, { 0, 0 }, sch_governmentbuilding, sch_legislativebuilding },
    { { 0, 10 }, { 0, 0 }, sch_governmentoffice, sch_postoffice },
    { { 3, 5 }, { 0, 0 }, sch_grant, sch_monetarygrant },
    { { 0, 10 }, { 0, 0 }, sch_healthandbeautybusiness, sch_beautysalon },
    { { 0, 10 }, { 0, 0 }, sch_healthandbeautybusiness, sch_dayspa },
    { { 0, 10 }, { 0, 0 }, sch_healthandbeautybusiness, sch_hairsalon },
    { { 0, 10 }, { 0, 0 }, sch_healthandbeautybusiness, sch_healthclub },
    { { 0, 10 }, { 0, 0 }, sch_healthandbeautybusiness, sch_nailsalon },
    { { 0, 10 }, { 0, 0 }, sch_healthandbeautybusiness, sch_tattooparlour },
    { { 5, 0 }, { 0, 0 }, sch_healthaspectenumeration, sch_benefitshealthaspect },
    { { 5, 0 }, { 0, 0 }, sch_healthaspectenumeration, sch_causeshealthaspect },
    { { 5, 0 }, { 0, 0 }, sch_healthaspectenumeration, sch_contagiousnesshealthaspect },
    { { 12, 0 }, { 0, 0 }, sch_healthaspectenumeration, sch_allergieshealthaspect },
    { { 12, 0 }, { 0, 0 }, sch_healthaspectenumeration, sch_effectivenesshealthaspect },
    { { 12, 0 }, { 0, 0 }, sch_healthaspectenumeration, sch_gettingaccesshealthaspect },
    { { 12, 0 }, { 0, 0 }, sch_healthaspectenumeration, sch_howitworkshealthaspect },
    { { 5, 0 }, { 0, 0 }, sch_healthaspectenumeration, sch_howorwherehealthaspect },
    { { 12, 0 }, { 0, 0 }, sch_healthaspectenumeration, sch_pregnancyhealthaspect },
    { { 5, 0 }, { 0, 0 }, sch_healthaspectenumeration, sch_ingredientshealthaspect },
    { { 5, 0 }, { 0, 0 }, sch_healthaspectenumeration, sch_maytreathealthaspect },
    { { 5, 0 }, { 0, 0 }, sch_healthaspectenumeration, sch_misconceptionshealthaspect },
    { { 5, 0 }, { 0, 0 }, sch_healthaspectenumeration, sch_overviewhealthaspect },
    { { 5, 0 }, { 0, 0 }, sch_healthaspectenumeration, sch_patientexperiencehealthaspect },
    { { 12, 0 }, { 0, 0 }, sch_healthaspectenumeration, sch_pregnancyhealthaspect },
    { { 5, 0 }, { 0, 0 }, sch_healthaspectenumeration, sch_preventionhealthaspect },
    { { 5, 0 }, { 0, 0 }, sch_healthaspectenumeration, sch_prognosishealthaspect },
    { { 5, 0 }, { 0, 0 }, sch_healthaspectenumeration, sch_relatedtopicshealthaspect },
    { { 5, 0 }, { 0, 0 }, sch_healthaspectenumeration, sch_risksorcomplicationshealthaspect },
    { { 12, 0 }, { 0, 0 }, sch_healthaspectenumeration, sch_safetyhealthaspect },
    { { 5, 0 }, { 0, 0 }, sch_healthaspectenumeration, sch_screeninghealthaspect },
    { { 5, 0 }, { 0, 0 }, sch_healthaspectenumeration, sch_seedoctorhealthaspect },
    { { 5, 0 }, { 0, 0 }, sch_healthaspectenumeration, sch_selfcarehealthaspect },
    { { 5, 0 }, { 0, 0 }, sch_healthaspectenumeration, sch_sideeffectshealthaspect },
    { { 5, 0 }, { 0, 0 }, sch_healthaspectenumeration, sch_stageshealthaspect },
    { { 5, 0 }, { 0, 0 }, sch_healthaspectenumeration, sch_symptomshealthaspect },
    { { 5, 0 }, { 0, 0 }, sch_healthaspectenumeration, sch_treatmentshealthaspect },
    { { 5, 0 }, { 0, 0 }, sch_healthaspectenumeration, sch_typeshealthaspect },
    { { 5, 0 }, { 0, 0 }, sch_healthaspectenumeration, sch_usageorschedulehealthaspect },
    { { 0, 10 }, { 0, 0 }, sch_homeandconstructionbusiness, sch_electrician },
    { { 0, 10 }, { 0, 0 }, sch_homeandconstructionbusiness, sch_generalcontractor },
    { { 0, 10 }, { 0, 0 }, sch_homeandconstructionbusiness, sch_hvacbusiness },
    { { 0, 10 }, { 0, 0 }, sch_homeandconstructionbusiness, sch_housepainter },
    { { 0, 10 }, { 0, 0 }, sch_homeandconstructionbusiness, sch_locksmith },
    { { 0, 10 }, { 0, 0 }, sch_homeandconstructionbusiness, sch_removers },
    { { 0, 10 }, { 0, 0 }, sch_homeandconstructionbusiness, sch_plumber },
    { { 0, 10 }, { 0, 0 }, sch_homeandconstructionbusiness, sch_roofingcontractor },
    { { 3, 3 }, { 0, 0 }, sch_howto, sch_recipe },
    { { 3, 3 }, { 0, 0 }, sch_howtoitem, sch_howtosupply },
    { { 3, 3 }, { 0, 0 }, sch_howtoitem, sch_howtotool },
    { { 3, 1 }, { 0, 0 }, sch_house, sch_singlefamilyresidence },
    { { 2, 2 }, { 0, 0 }, sch_imageobject, sch_barcode },
    { { 13, 0 }, { 0, 0 }, sch_imageobject, sch_imageobjectsnapshot },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_infectiousagentclass, sch_bacteria },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_infectiousagentclass, sch_fungus },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_infectiousagentclass, sch_multicellularparasite },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_infectiousagentclass, sch_prion },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_infectiousagentclass, sch_protozoa },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_infectiousagentclass, sch_virus },
    { { 1, 2 }, { 0, 0 }, sch_informaction, sch_confirmaction },
    { { 1, 2 }, { 0, 0 }, sch_informaction, sch_rsvpaction },
    { { 1, 2 }, { 0, 0 }, sch_insertaction, sch_appendaction },
    { { 1, 2 }, { 0, 0 }, sch_insertaction, sch_prependaction },
    { { 3, 5 }, { 0, 0 }, sch_intangible, sch_actionaccessspecification },
    { { 1, 0 }, { 0, 0 }, sch_intangible, sch_alignmentobject },
    { { 0, 95 }, { 0, 0 }, sch_intangible, sch_audience },
    { { 3, 1 }, { 0, 0 }, sch_intangible, sch_beddetails },
    { { 0, 99 }, { 0, 0 }, sch_intangible, sch_brand },
    { { 2, 0 }, { 0, 0 }, sch_intangible, sch_broadcastchannel },
    { { 1, 20 }, { 3, 4 }, sch_intangible, sch_bustrip },
    { { 3, 5 }, { 0, 0 }, sch_intangible, sch_broadcastfrequencyspecification },
    { { 1, 92 }, { 2, 1 }, sch_intangible, sch_class },
    { { 3, 5 }, { 0, 0 }, sch_intangible, sch_class },
    { { 3, 0 }, { 0, 0 }, sch_intangible, sch_computerlanguage },
    { { 2, 2 }, { 0, 0 }, sch_intangible, sch_datafeeditem },
    { { 3, 5 }, { 0, 0 }, sch_intangible, sch_definedterm },
    { { 0, 99 }, { 0, 0 }, sch_intangible, sch_demand },
    { { 3, 0 }, { 0, 0 }, sch_intangible, sch_digitaldocumentpermission },
    { { 10, 0 }, { 0, 0 }, sch_intangible, sch_energyconsumptiondetails },
    { { 1, 20 }, { 0, 0 }, sch_intangible, sch_entrypoint },
    { { 3, 9 }, { 0, 0 }, sch_intangible, sch_educationoccupationprogramme },
    { { 0, 10 }, { 0, 0 }, sch_intangible, sch_enumeration },
    { { 1, 20 }, { 3, 3 }, sch_intangible, sch_flight },
    { { 6, 0 }, { 0, 0 }, sch_intangible, sch_floorplan },
    { { 1, 92 }, { 0, 0 }, sch_intangible, sch_gameserver },
    { { 3, 5 }, { 0, 0 }, sch_intangible, sch_geospatialgeometry },
    { { 3, 5 }, { 0, 0 }, sch_intangible, sch_grant },
    { { 3, 5 }, { 0, 0 }, sch_intangible, sch_dystopianhealthinsuranceplan },
    { { 3, 5 }, { 0, 0 }, sch_intangible, sch_dystopianhealthplancostsharingspecification },
    { { 3, 5 }, { 0, 0 }, sch_intangible, sch_dystopianhealthplanformulary },
    { { 3, 5 }, { 0, 0 }, sch_intangible, sch_dystopianhealthplannetwork },
    { { 1, 93 }, { 0, 0 }, sch_intangible, sch_invoice },
    { { 1, 92 }, { 0, 0 }, sch_intangible, sch_itemlist },
    { { 0, 60 }, { 0, 0 }, sch_intangible, sch_jobposting },
    { { 0, 10 }, { 0, 0 }, sch_intangible, sch_language },
    { { 1, 92 }, { 0, 0 }, sch_intangible, sch_listitem },
    { { 3, 5 }, { 0, 0 }, sch_intangible, sch_mediasubscription },
    { { 3, 2 }, { 0, 0 }, sch_intangible, sch_menuitem },
    { { 6, 0 }, { 0, 0 }, sch_intangible, sch_merchantreturnpolicy },
    { { 13, 0 }, { 0, 0 }, sch_intangible, sch_merchantreturnpolicyseasonaloverride },
    { { 3, 9 }, { 0, 0 }, sch_intangible, sch_observation },
    { { 3, 5 }, { 0, 0 }, sch_intangible, sch_occupation },
    { { 0, 10 }, { 0, 0 }, sch_intangible, sch_offer },
    { { 7, 2 }, { 7, 3 }, sch_intangible, sch_offershippingdetails },
    { { 1, 4 }, { 0, 0 }, sch_intangible, sch_order },
    { { 2, 0 }, { 0, 0 }, sch_intangible, sch_orderitem },
    { { 1, 4 }, { 0, 0 }, sch_intangible, sch_parceldelivery },
    { { 13, 0 }, { 0, 0 }, sch_intangible, poetry_period },
    { { 1, 4 }, { 0, 0 }, sch_intangible, sch_permit },
    { { 3, 8 }, { 5, 0 }, sch_intangible, sch_productreturnpolicy },
    { { 1, 20 }, { 0, 0 }, sch_intangible, sch_programmemembership },
    { { 1, 92 }, { 2, 1 }, sch_intangible, sch_property },
    { { 3, 5 }, { 0, 0 }, sch_intangible, sch_property },
    { { 1, 92 }, { 0, 0 }, sch_intangible, sch_propertyvaluespecification },
    { { 0, 10 }, { 0, 0 }, sch_intangible, sch_quantity },
    { { 0, 10 }, { 0, 0 }, sch_intangible, sch_rating },
    { { 1, 10 }, { 0, 0 }, sch_intangible, sch_reservation },
    { { 1, 92 }, { 0, 0 }, sch_intangible, sch_role },
    { { 3, 6 }, { 0, 0 }, sch_intangible, sch_schedule },
    { { 1, 20 }, { 0, 0 }, sch_intangible, sch_seat },
    { { 3, 4 }, { 0, 0 }, sch_intangible, sch_series },
    { { 1, 3 }, { 0, 0 }, sch_intangible, sch_service },
    { { 1, 3 }, { 0, 0 }, sch_intangible, sch_servicechannel },
    { { 3, 5 }, { 0, 0 }, sch_intangible, sch_speakablespecification },
    { { 3, 9 }, { 0, 0 }, sch_intangible, sch_statisticalpopulation },
    { { 0, 10 }, { 0, 0 }, sch_intangible, sch_structuredvalue },
    { { 1, 20 }, { 0, 0 }, sch_intangible, sch_ticket },
    { { 1, 20 }, { 3, 3 }, sch_intangible, sch_traintrip },
    { { 3, 4 }, { 0, 0 }, sch_intangible, sch_trip },
    { { 3, 7 }, { 0, 0 }, sch_intangible, sch_virtuallocation },
    { { 1, 2 }, { 0, 0 }, sch_interactaction, sch_befriendaction },
    { { 1, 2 }, { 0, 0 }, sch_interactaction, sch_communicateaction },
    { { 1, 2 }, { 0, 0 }, sch_interactaction, sch_followaction },
    { { 1, 2 }, { 0, 0 }, sch_interactaction, sch_joinaction },
    { { 1, 2 }, { 0, 0 }, sch_interactaction, sch_leaveaction },
    { { 1, 2 }, { 0, 0 }, sch_interactaction, sch_marryaction },
    { { 1, 2 }, { 0, 0 }, sch_interactaction, sch_registeraction },
    { { 1, 2 }, { 0, 0 }, sch_interactaction, sch_subscribeaction },
    { { 1, 2 }, { 0, 0 }, sch_interactaction, sch_unregisteraction },
    { { 3, 5 }, { 0, 0 }, sch_investmentordeposit, sch_brokerageaccount },
    { { 3, 1 }, { 0, 0 }, sch_investmentordeposit, sch_depositaccount },
    { { 3, 5 }, { 0, 0 }, sch_investmentordeposit, sch_investmentfund },
    { { 12, 0 }, { 0, 0 }, sch_itemavailability, sch_backorder },
    { { 0, 10 }, { 0, 0 }, sch_itemavailability, sch_discontinued },
    { { 0, 10 }, { 0, 0 }, sch_itemavailability, sch_instock },
    { { 0, 10 }, { 0, 0 }, sch_itemavailability, sch_instoreonly },
    { { 3, 5 }, { 0, 0 }, sch_itemavailability, sch_limitedavailability },
    { { 0, 10 }, { 0, 0 }, sch_itemavailability, sch_onlineonly },
    { { 0, 10 }, { 0, 0 }, sch_itemavailability, sch_outofstock },
    { { 0, 10 }, { 0, 0 }, sch_itemavailability, sch_preorder },
    { { 3, 5 }, { 0, 0 }, sch_itemavailability, sch_presale },
    { { 3, 5 }, { 0, 0 }, sch_itemavailability, sch_soldout },
    { { 1, 93 }, { 0, 0 }, sch_itemlist, sch_breadcrumblist },
    { { 3, 3 }, { 0, 0 }, sch_itemlist, sch_howtosection },
    { { 3, 3 }, { 0, 0 }, sch_itemlist, sch_howtostep },
    { { 2, 2 }, { 0, 0 }, sch_itemlist, sch_offercatalogue },
    { { 1, 92 }, { 0, 0 }, sch_itemlistordertype, sch_itemlistorderascending },
    { { 1, 92 }, { 0, 0 }, sch_itemlistordertype, sch_itemlistorderdescending },
    { { 1, 92 }, { 0, 0 }, sch_itemlistordertype, sch_itemlistunordered },
    { { 0, 10 }, { 0, 0 }, sch_landform, sch_bodyofwater },
    { { 0, 10 }, { 0, 0 }, sch_landform, sch_continent },
    { { 0, 10 }, { 0, 0 }, sch_landform, sch_mountain },
    { { 0, 10 }, { 0, 0 }, sch_landform, sch_volcano },
    { { 13, 0 }, { 0, 0 }, sch_language, poetry_dialect },
    { { 9, 0 }, { 0, 0 }, sch_learningresource, sch_course },
    { { 9, 0 }, { 0, 0 }, sch_learningresource, sch_quiz },
    { { 3, 5 }, { 0, 0 }, sch_legalforcestatus, sch_inforce },
    { { 3, 5 }, { 0, 0 }, sch_legalforcestatus, sch_notinforce },
    { { 3, 5 }, { 0, 0 }, sch_legalforcestatus, sch_partiallyinforce },
    { { 3, 5 }, { 0, 0 }, sch_legalvaluelevel, sch_authoritativelegalvalue },
    { { 3, 5 }, { 0, 0 }, sch_legalvaluelevel, sch_definitivelegalvalue },
    { { 3, 5 }, { 0, 0 }, sch_legalvaluelevel, sch_officiallegalvalue },
    { { 3, 5 }, { 0, 0 }, sch_legalvaluelevel, sch_unofficiallegalvalue },
    { { 3, 5 }, { 0, 0 }, sch_legislation, sch_legislationobject },
    { { 0, 95 }, { 0, 0 }, sch_lifestylemodification, sch_diet },
    { { 0, 95 }, { 0, 0 }, sch_lifestylemodification, sch_physicalactivity },
    { { 3, 3 }, { 0, 0 }, sch_listitem, sch_howtodirection },
    { { 3, 3 }, { 0, 0 }, sch_listitem, sch_howtoitem },
    { { 3, 4 }, { 0, 0 }, sch_listitem, sch_howtosection },
    { { 3, 4 }, { 0, 0 }, sch_listitem, sch_howtostep },
    { { 3, 3 }, { 0, 0 }, sch_listitem, sch_howtotip },
    { { 3, 0 }, { 0, 0 }, sch_loanorcredit, sch_creditcard },
    { { 3, 5 }, { 0, 0 }, sch_loanorcredit, sch_mortgageloan },
    { { 0, 10 }, { 0, 0 }, sch_localbusiness, sch_animalshelter },
    { { 3, 5 }, { 0, 0 }, sch_localbusiness, sch_archiveorganisation },
    { { 2, 2 }, { 0, 0 }, sch_localbusiness, sch_attorney },
    { { 0, 10 }, { 0, 0 }, sch_localbusiness, sch_automotivebusiness },
    { { 0, 10 }, { 0, 0 }, sch_localbusiness, sch_childcare },
    { { 3, 1 }, { 0, 0 }, sch_localbusiness, sch_dentist },
    { { 0, 10 }, { 0, 0 }, sch_localbusiness, sch_drycleaningorlaundry },
    { { 0, 10 }, { 0, 0 }, sch_localbusiness, sch_emergencyservice },
    { { 0, 10 }, { 0, 0 }, sch_localbusiness, sch_employmentagency },
    { { 0, 10 }, { 0, 0 }, sch_localbusiness, sch_entertainmentbusiness },
    { { 0, 10 }, { 0, 0 }, sch_localbusiness, sch_financialservice },
    { { 0, 10 }, { 0, 0 }, sch_localbusiness, sch_foodestablishment },
    { { 0, 10 }, { 0, 0 }, sch_localbusiness, sch_governmentoffice },
    { { 0, 10 }, { 0, 0 }, sch_localbusiness, sch_healthandbeautybusiness },
    { { 0, 10 }, { 0, 0 }, sch_localbusiness, sch_homeandconstructionbusiness },
    { { 0, 10 }, { 0, 0 }, sch_localbusiness, sch_internetcafe },
    { { 2, 2 }, { 0, 0 }, sch_localbusiness, sch_legalservice },
    { { 0, 10 }, { 0, 0 }, sch_localbusiness, sch_library },
    { { 0, 10 }, { 0, 0 }, sch_localbusiness, sch_lodgingbusiness },
    { { 0, 10 }, { 2, 1 }, sch_localbusiness, sch_medicalorganisation },
    { { 3, 0 }, { 3, 1 }, sch_localbusiness, sch_medicalbusiness },
    { { 3, 5 }, { 0, 0 }, sch_localbusiness, sch_medicalbusiness },
    { { 3, 0 }, { 0, 0 }, sch_localbusiness, sch_notary },
    { { 2, 0 }, { 0, 0 }, sch_localbusiness, sch_postoffice },
    { { 0, 10 }, { 0, 0 }, sch_localbusiness, sch_professionalservice },
    { { 0, 10 }, { 0, 0 }, sch_localbusiness, sch_radiostation },
    { { 0, 10 }, { 0, 0 }, sch_localbusiness, sch_realestateagent },
    { { 0, 10 }, { 0, 0 }, sch_localbusiness, sch_recyclingcentre },
    { { 0, 10 }, { 0, 0 }, sch_localbusiness, sch_selfstorage },
    { { 0, 10 }, { 0, 0 }, sch_localbusiness, sch_shoppingcentre },
    { { 0, 10 }, { 0, 0 }, sch_localbusiness, sch_sportsactivitylocation },
    { { 0, 10 }, { 0, 0 }, sch_localbusiness, sch_store },
    { { 0, 10 }, { 0, 0 }, sch_localbusiness, sch_televisionstation },
    { { 0, 10 }, { 0, 0 }, sch_localbusiness, sch_touristinformationcentre },
    { { 0, 10 }, { 0, 0 }, sch_localbusiness, sch_travelagency },
    { { 2, 0 }, { 0, 0 }, sch_localbusiness, sch_wholesalers },
    { { 0, 10 }, { 0, 0 }, sch_lodgingbusiness, sch_bedandbreakfast },
    { { 3, 1 }, { 0, 0 }, sch_lodgingbusiness, sch_campground },
    { { 0, 10 }, { 0, 0 }, sch_lodgingbusiness, sch_hostel },
    { { 0, 10 }, { 0, 0 }, sch_lodgingbusiness, sch_hotel },
    { { 0, 10 }, { 0, 0 }, sch_lodgingbusiness, sch_motel },
    { { 3, 1 }, { 0, 0 }, sch_lodgingbusiness, sch_resort },
    { { 1, 92 }, { 0, 0 }, sch_mapcategorytype, sch_parkingmap },
    { { 1, 92 }, { 0, 0 }, sch_mapcategorytype, sch_seatingmap },
    { { 1, 92 }, { 0, 0 }, sch_mapcategorytype, sch_transitmap },
    { { 1, 92 }, { 0, 0 }, sch_mapcategorytype, sch_venuemap },
    { { 12, 0 }, { 0, 0 }, sch_measurementtypeenumeration, sch_bodymeasurementtypeenumeration },
    { { 12, 0 }, { 0, 0 }, sch_measurementtypeenumeration, sch_wearablemeasurementtypeenumeration },
    { { 6, 0 }, { 0, 0 }, sch_mediagallery, sch_imagegallery },
    { { 6, 0 }, { 0, 0 }, sch_mediagallery, sch_videogallery },
    { { 7, 0 }, { 11, 0 }, sch_mediamanipulationratingenumeration, sch_authenticcontent },
    { { 12, 0 }, { 0, 0 }, sch_mediamanipulationratingenumeration, sch_decontextualisedcontent },
    { { 12, 0 }, { 0, 0 }, sch_mediamanipulationratingenumeration, sch_editedorcroppedcontent },
    { { 7, 0 }, { 11, 0 }, sch_mediamanipulationratingenumeration, sch_missingcontext },
    { { 12, 0 }, { 0, 0 }, sch_mediamanipulationratingenumeration, sch_originalmediacontent },
    { { 12, 0 }, { 0, 0 }, sch_mediamanipulationratingenumeration, sch_satireorparodycontent },
    { { 12, 0 }, { 0, 0 }, sch_mediamanipulationratingenumeration, sch_stagedcontent },
    { { 12, 0 }, { 0, 0 }, sch_mediamanipulationratingenumeration, sch_transformedcontent },
    { { 7, 3 }, { 0, 0 }, sch_mediaobject, sch_3dmodel },
    { { 0, 10 }, { 0, 0 }, sch_mediaobject, sch_audioobject },
    { { 1, 0 }, { 0, 0 }, sch_mediaobject, sch_datadownload },
    { { 0, 10 }, { 0, 0 }, sch_mediaobject, sch_imageobject },
    { { 3, 5 }, { 0, 0 }, sch_mediaobject, sch_legislationobject },
    { { 0, 10 }, { 0, 0 }, sch_mediaobject, sch_musicvideoobject },
    { { 0, 10 }, { 0, 0 }, sch_mediaobject, sch_videoobject },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicalaudience, sch_clinician },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicalaudience, sch_medicalresearcher },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicalaudience, sch_patient },
    { { 9, 0 }, { 0, 0 }, sch_medicalaudiencetype, sch_clinician },
    { { 9, 0 }, { 0, 0 }, sch_medicalaudiencetype, sch_medicalresearcher },
    { { 3, 5 }, { 0, 0 }, sch_medicalbusiness, sch_communityhealth },
    { { 3, 0 }, { 0, 0 }, sch_medicalbusiness, sch_dentist },
    { { 3, 5 }, { 0, 0 }, sch_medicalbusiness, sch_dermatology },
    { { 3, 5 }, { 0, 0 }, sch_medicalbusiness, sch_dietnutrition },
    { { 3, 5 }, { 0, 0 }, sch_medicalbusiness, sch_emergency },
    { { 3, 5 }, { 0, 0 }, sch_medicalbusiness, sch_geriatric },
    { { 3, 5 }, { 0, 0 }, sch_medicalbusiness, sch_gynaecologic },
    { { 3, 5 }, { 0, 0 }, sch_medicalbusiness, sch_medicalclinic },
    { { 3, 5 }, { 0, 0 }, sch_medicalbusiness, sch_midwifery },
    { { 3, 5 }, { 0, 0 }, sch_medicalbusiness, sch_nursing },
    { { 3, 5 }, { 0, 0 }, sch_medicalbusiness, sch_obstetric },
    { { 3, 5 }, { 0, 0 }, sch_medicalbusiness, sch_oncologic },
    { { 3, 5 }, { 0, 0 }, sch_medicalbusiness, sch_optician },
    { { 3, 5 }, { 0, 0 }, sch_medicalbusiness, sch_optometric },
    { { 3, 5 }, { 0, 0 }, sch_medicalbusiness, sch_otolaryngologic },
    { { 3, 5 }, { 0, 0 }, sch_medicalbusiness, sch_paediatric },
    { { 3, 0 }, { 0, 0 }, sch_medicalbusiness, sch_pharmacy },
    { { 3, 0 }, { 0, 0 }, sch_medicalbusiness, sch_physician },
    { { 3, 5 }, { 0, 0 }, sch_medicalbusiness, sch_physiotherapy },
    { { 3, 5 }, { 0, 0 }, sch_medicalbusiness, sch_plasticsurgery },
    { { 3, 5 }, { 0, 0 }, sch_medicalbusiness, sch_podiatric },
    { { 3, 5 }, { 0, 0 }, sch_medicalbusiness, sch_primarycare },
    { { 3, 5 }, { 0, 0 }, sch_medicalbusiness, sch_psychiatric },
    { { 3, 5 }, { 0, 0 }, sch_medicalbusiness, sch_publichealth },
    { { 7, 0 }, { 0, 0 }, sch_medicalclinic, sch_covidtestingfacility },
    { { 0, 95 }, { 0, 0 }, sch_medicalcondition, sch_infectiousdisease },
    { { 3, 5 }, { 0, 0 }, sch_medicalcondition, sch_medicalsignorsymptom },
    { { 0, 95 }, { 0, 0 }, sch_medicaldevicepurpose, sch_diagnostic },
    { { 0, 95 }, { 0, 0 }, sch_medicaldevicepurpose, sch_therapeutic },
    { { 0, 95 }, { 0, 0 }, sch_medicalentity, sch_anatomicalstructure },
    { { 0, 95 }, { 0, 0 }, sch_medicalentity, sch_anatomicalsystem },
    { { 9, 0 }, { 0, 0 }, sch_medicalentity, sch_drugclass },
    { { 9, 0 }, { 0, 0 }, sch_medicalentity, sch_drugcost },
    { { 3, 5 }, { 0, 0 }, sch_medicalentity, sch_lifestylemodification },
    { { 0, 95 }, { 0, 0 }, sch_medicalentity, sch_medicalcause },
    { { 0, 95 }, { 0, 0 }, sch_medicalentity, sch_medicalcondition },
    { { 0, 95 }, { 0, 0 }, sch_medicalentity, sch_medicalcontraindication },
    { { 0, 95 }, { 0, 0 }, sch_medicalentity, sch_medicaldevice },
    { { 0, 95 }, { 0, 0 }, sch_medicalentity, sch_medicalguideline },
    { { 0, 95 }, { 0, 0 }, sch_medicalentity, sch_medicalindication },
    { { 0, 95 }, { 0, 0 }, sch_medicalentity, sch_medicalintangible },
    { { 0, 95 }, { 0, 0 }, sch_medicalentity, sch_medicalprocedure },
    { { 0, 95 }, { 0, 0 }, sch_medicalentity, sch_medicalriskestimator },
    { { 0, 95 }, { 0, 0 }, sch_medicalentity, sch_medicalriskfactor },
    { { 0, 95 }, { 2, 1 }, sch_medicalentity, sch_medicalsignorsymptom },
    { { 0, 95 }, { 0, 0 }, sch_medicalentity, sch_medicalstudy },
    { { 0, 95 }, { 0, 0 }, sch_medicalentity, sch_medicaltest },
    { { 0, 95 }, { 2, 1 }, sch_medicalentity, sch_medicaltherapy },
    { { 3, 5 }, { 0, 0 }, sch_medicalentity, sch_substance },
    { { 0, 95 }, { 0, 0 }, sch_medicalentity, sch_superficialanatomy },
    { { 3, 5 }, { 0, 0 }, sch_medicalenumeration, sch_drugclass },
    { { 3, 5 }, { 0, 0 }, sch_medicalenumeration, sch_drugcost },
    { { 0, 95 }, { 0, 0 }, sch_medicalenumeration, sch_drugcostcategory },
    { { 0, 95 }, { 0, 0 }, sch_medicalenumeration, sch_drugpregnancycategory },
    { { 0, 95 }, { 0, 0 }, sch_medicalenumeration, sch_drugprescriptionstatus },
    { { 0, 95 }, { 0, 0 }, sch_medicalenumeration, sch_infectiousagentclass },
    { { 0, 95 }, { 8, 0 }, sch_medicalenumeration, sch_medicalaudience },
    { { 9, 0 }, { 0, 0 }, sch_medicalenumeration, sch_medicalaudiencetype },
    { { 0, 95 }, { 0, 0 }, sch_medicalenumeration, sch_medicaldevicepurpose },
    { { 0, 95 }, { 0, 0 }, sch_medicalenumeration, sch_medicalevidencelevel },
    { { 0, 95 }, { 0, 0 }, sch_medicalenumeration, sch_medicalimagingtechnique },
    { { 0, 95 }, { 0, 0 }, sch_medicalenumeration, sch_medicalobservationalstudydesign },
    { { 0, 95 }, { 0, 0 }, sch_medicalenumeration, sch_medicalproceduretype },
    { { 0, 95 }, { 0, 0 }, sch_medicalenumeration, sch_medicalspeciality },
    { { 0, 95 }, { 0, 0 }, sch_medicalenumeration, sch_medicalstudystatus },
    { { 0, 95 }, { 0, 0 }, sch_medicalenumeration, sch_medicaltrialdesign },
    { { 0, 95 }, { 0, 0 }, sch_medicalenumeration, sch_medicinesystem },
    { { 0, 95 }, { 2, 1 }, sch_medicalenumeration, sch_physicalactivitycategory },
    { { 0, 95 }, { 0, 0 }, sch_medicalenumeration, sch_physicalexam },
    { { 0, 95 }, { 0, 0 }, sch_medicalevidencelevel, sch_evidencelevela },
    { { 0, 95 }, { 0, 0 }, sch_medicalevidencelevel, sch_evidencelevelb },
    { { 0, 95 }, { 0, 0 }, sch_medicalevidencelevel, sch_evidencelevelc },
    { { 0, 95 }, { 0, 0 }, sch_medicalguideline, sch_medicalguidelinecontraindication },
    { { 0, 95 }, { 0, 0 }, sch_medicalguideline, sch_medicalguidelinerecommendation },
    { { 0, 95 }, { 0, 0 }, sch_medicalimagingtechnique, sch_ct },
    { { 0, 95 }, { 0, 0 }, sch_medicalimagingtechnique, sch_mri },
    { { 0, 95 }, { 0, 0 }, sch_medicalimagingtechnique, sch_pet },
    { { 2, 0 }, { 0, 0 }, sch_medicalimagingtechnique, sch_radiography },
    { { 0, 95 }, { 0, 0 }, sch_medicalimagingtechnique, sch_ultrasound },
    { { 0, 95 }, { 0, 0 }, sch_medicalimagingtechnique, sch_xray },
    { { 0, 95 }, { 0, 0 }, sch_medicalindication, sch_approvedindication },
    { { 0, 95 }, { 0, 0 }, sch_medicalindication, sch_preventionindication },
    { { 0, 95 }, { 0, 0 }, sch_medicalindication, sch_treatmentindication },
    { { 0, 95 }, { 0, 0 }, sch_medicalintangible, sch_ddxelement },
    { { 0, 95 }, { 0, 0 }, sch_medicalintangible, sch_doseschedule },
    { { 0, 95 }, { 2, 1 }, sch_medicalintangible, sch_drugcost },
    { { 0, 95 }, { 0, 0 }, sch_medicalintangible, sch_druglegalstatus },
    { { 0, 95 }, { 0, 0 }, sch_medicalintangible, sch_drugstrength },
    { { 0, 95 }, { 0, 0 }, sch_medicalintangible, sch_medicalcode },
    { { 0, 95 }, { 0, 0 }, sch_medicalintangible, sch_medicalconditionstage },
    { { 0, 95 }, { 2, 1 }, sch_medicalintangible, sch_medicalenumeration },
    { { 0, 95 }, { 0, 0 }, sch_medicalobservationalstudydesign, sch_caseseries },
    { { 0, 95 }, { 0, 0 }, sch_medicalobservationalstudydesign, sch_cohortstudy },
    { { 0, 95 }, { 0, 0 }, sch_medicalobservationalstudydesign, sch_crosssectional },
    { { 0, 95 }, { 0, 0 }, sch_medicalobservationalstudydesign, sch_longitudinal },
    { { 0, 95 }, { 0, 0 }, sch_medicalobservationalstudydesign, sch_observational },
    { { 0, 95 }, { 0, 0 }, sch_medicalobservationalstudydesign, sch_registry },
    { { 0, 10 }, { 0, 0 }, sch_medicalorganisation, sch_dentist },
    { { 0, 95 }, { 2, 1 }, sch_medicalorganisation, sch_diagnosticlab },
    { { 3, 5 }, { 0, 0 }, sch_medicalorganisation, sch_diagnosticlab },
    { { 0, 10 }, { 0, 0 }, sch_medicalorganisation, sch_hospital },
    { { 0, 10 }, { 2, 1 }, sch_medicalorganisation, sch_medicalclinic },
    { { 0, 10 }, { 2, 1 }, sch_medicalorganisation, sch_optician },
    { { 0, 10 }, { 0, 0 }, sch_medicalorganisation, sch_pharmacy },
    { { 0, 10 }, { 0, 0 }, sch_medicalorganisation, sch_physician },
    { { 0, 10 }, { 2, 1 }, sch_medicalorganisation, sch_veterinarycare },
    { { 3, 5 }, { 0, 0 }, sch_medicalorganisation, sch_veterinarycare },
    { { 0, 95 }, { 0, 0 }, sch_medicalprocedure, sch_diagnosticprocedure },
    { { 0, 95 }, { 0, 0 }, sch_medicalprocedure, sch_palliativeprocedure },
    { { 3, 5 }, { 0, 0 }, sch_medicalprocedure, sch_physicalexam },
    { { 3, 5 }, { 0, 0 }, sch_medicalprocedure, sch_surgicalprocedure },
    { { 0, 95 }, { 0, 0 }, sch_medicalprocedure, sch_therapeuticprocedure },
    { { 0, 95 }, { 0, 0 }, sch_medicalproceduretype, sch_noninvasiveprocedure },
    { { 0, 95 }, { 0, 0 }, sch_medicalproceduretype, sch_percutaneousprocedure },
    { { 0, 95 }, { 0, 0 }, sch_medicalriskestimator, sch_medicalriskcalculator },
    { { 0, 95 }, { 0, 0 }, sch_medicalriskestimator, sch_medicalriskscore },
    { { 3, 5 }, { 0, 0 }, sch_medicalsign, sch_vitalsign },
    { { 0, 95 }, { 0, 0 }, sch_medicalsignorsymptom, sch_medicalsign },
    { { 0, 95 }, { 0, 0 }, sch_medicalsignorsymptom, sch_medicalsymptom },
    { { 0, 95 }, { 0, 0 }, sch_medicalspeciality, sch_anaesthesia },
    { { 0, 95 }, { 0, 0 }, sch_medicalspeciality, sch_cardiovascular },
    { { 0, 95 }, { 0, 0 }, sch_medicalspeciality, sch_communityhealth },
    { { 0, 95 }, { 0, 0 }, sch_medicalspeciality, sch_dentistry },
    { { 0, 95 }, { 0, 0 }, sch_medicalspeciality, sch_dermatology },
    { { 0, 95 }, { 0, 0 }, sch_medicalspeciality, sch_dietnutrition },
    { { 0, 95 }, { 0, 0 }, sch_medicalspeciality, sch_emergency },
    { { 0, 95 }, { 0, 0 }, sch_medicalspeciality, sch_endocrine },
    { { 0, 95 }, { 0, 0 }, sch_medicalspeciality, sch_gastroenterologic },
    { { 0, 95 }, { 0, 0 }, sch_medicalspeciality, sch_genetic },
    { { 0, 95 }, { 0, 0 }, sch_medicalspeciality, sch_geriatric },
    { { 0, 95 }, { 0, 0 }, sch_medicalspeciality, sch_gynaecologic },
    { { 0, 95 }, { 0, 0 }, sch_medicalspeciality, sch_haematologic },
    { { 0, 95 }, { 0, 0 }, sch_medicalspeciality, sch_infectious },
    { { 0, 95 }, { 0, 0 }, sch_medicalspeciality, sch_laboratoryscience },
    { { 0, 95 }, { 0, 0 }, sch_medicalspeciality, sch_midwifery },
    { { 0, 95 }, { 0, 0 }, sch_medicalspeciality, sch_musculoskeletal },
    { { 0, 95 }, { 0, 0 }, sch_medicalspeciality, sch_neurologic },
    { { 0, 95 }, { 0, 0 }, sch_medicalspeciality, sch_nursing },
    { { 0, 95 }, { 0, 0 }, sch_medicalspeciality, sch_obstetric },
    { { 0, 95 }, { 0, 0 }, sch_medicalspeciality, sch_oncologic },
    { { 0, 95 }, { 0, 0 }, sch_medicalspeciality, sch_optometric },
    { { 0, 95 }, { 0, 0 }, sch_medicalspeciality, sch_otolaryngologic },
    { { 0, 95 }, { 0, 0 }, sch_medicalspeciality, sch_pathology },
    { { 0, 95 }, { 0, 0 }, sch_medicalspeciality, sch_paediatric },
    { { 0, 95 }, { 0, 0 }, sch_medicalspeciality, sch_pharmacyspeciality },
    { { 0, 95 }, { 0, 0 }, sch_medicalspeciality, sch_physiotherapy },
    { { 0, 95 }, { 0, 0 }, sch_medicalspeciality, sch_plasticsurgery },
    { { 0, 95 }, { 0, 0 }, sch_medicalspeciality, sch_podiatric },
    { { 0, 95 }, { 0, 0 }, sch_medicalspeciality, sch_primarycare },
    { { 0, 95 }, { 0, 0 }, sch_medicalspeciality, sch_psychiatric },
    { { 0, 95 }, { 0, 0 }, sch_medicalspeciality, sch_publichealth },
    { { 0, 95 }, { 0, 0 }, sch_medicalspeciality, sch_pulmonary },
    { { 0, 95 }, { 0, 0 }, sch_medicalspeciality, sch_radiography },
    { { 0, 95 }, { 0, 0 }, sch_medicalspeciality, sch_renal },
    { { 0, 95 }, { 0, 0 }, sch_medicalspeciality, sch_respiratorytherapy },
    { { 0, 95 }, { 0, 0 }, sch_medicalspeciality, sch_rheumatologic },
    { { 0, 95 }, { 0, 0 }, sch_medicalspeciality, sch_speechpathology },
    { { 0, 95 }, { 0, 0 }, sch_medicalspeciality, sch_surgical },
    { { 0, 95 }, { 0, 0 }, sch_medicalspeciality, sch_toxicologic },
    { { 0, 95 }, { 0, 0 }, sch_medicalspeciality, sch_urologic },
    { { 0, 95 }, { 0, 0 }, sch_medicalstudy, sch_medicalobservationalstudy },
    { { 0, 95 }, { 0, 0 }, sch_medicalstudy, sch_medicaltrial },
    { { 0, 95 }, { 0, 0 }, sch_medicalstudystatus, sch_activenotrecruiting },
    { { 0, 95 }, { 0, 0 }, sch_medicalstudystatus, sch_completed },
    { { 0, 95 }, { 0, 0 }, sch_medicalstudystatus, sch_enrollingbyinvitation },
    { { 0, 95 }, { 0, 0 }, sch_medicalstudystatus, sch_notyetrecruiting },
    { { 0, 95 }, { 0, 0 }, sch_medicalstudystatus, sch_recruiting },
    { { 0, 95 }, { 0, 0 }, sch_medicalstudystatus, sch_resultsavailable },
    { { 0, 95 }, { 0, 0 }, sch_medicalstudystatus, sch_resultsnotavailable },
    { { 0, 95 }, { 0, 0 }, sch_medicalstudystatus, sch_suspended },
    { { 0, 95 }, { 0, 0 }, sch_medicalstudystatus, sch_terminated },
    { { 0, 95 }, { 0, 0 }, sch_medicalstudystatus, sch_withdrawn },
    { { 0, 95 }, { 0, 0 }, sch_medicaltest, sch_bloodtest },
    { { 0, 95 }, { 2, 1 }, sch_medicaltest, sch_diagnosticprocedure },
    { { 0, 95 }, { 0, 0 }, sch_medicaltest, sch_imagingtest },
    { { 0, 95 }, { 0, 0 }, sch_medicaltest, sch_medicaltestpanel },
    { { 0, 95 }, { 0, 0 }, sch_medicaltest, sch_pathologytest },
    { { 0, 95 }, { 2, 1 }, sch_medicaltherapy, sch_dietarysupplement },
    { { 0, 95 }, { 2, 1 }, sch_medicaltherapy, sch_drug },
    { { 0, 95 }, { 2, 1 }, sch_medicaltherapy, sch_drugclass },
    { { 0, 95 }, { 2, 1 }, sch_medicaltherapy, sch_lifestylemodification },
    { { 3, 5 }, { 0, 0 }, sch_medicaltherapy, sch_occupationaltherapy },
    { { 0, 95 }, { 0, 0 }, sch_medicaltherapy, sch_palliativeprocedure },
    { { 0, 95 }, { 0, 0 }, sch_medicaltherapy, sch_physicaltherapy },
    { { 0, 95 }, { 2, 1 }, sch_medicaltherapy, sch_psychologicaltreatment },
    { { 0, 95 }, { 0, 0 }, sch_medicaltherapy, sch_radiationtherapy },
    { { 3, 5 }, { 0, 0 }, sch_medicaltherapy, sch_respiratorytherapy },
    { { 0, 95 }, { 2, 1 }, sch_medicaltherapy, sch_therapeuticprocedure },
    { { 0, 95 }, { 0, 0 }, sch_medicaltrialdesign, sch_doubleblindedtrial },
    { { 0, 95 }, { 0, 0 }, sch_medicaltrialdesign, sch_internationaltrial },
    { { 0, 95 }, { 0, 0 }, sch_medicaltrialdesign, sch_multicentretrial },
    { { 0, 95 }, { 0, 0 }, sch_medicaltrialdesign, sch_opentrial },
    { { 0, 95 }, { 0, 0 }, sch_medicaltrialdesign, sch_placebocontrolledtrial },
    { { 0, 95 }, { 0, 0 }, sch_medicaltrialdesign, sch_randomisedtrial },
    { { 0, 95 }, { 0, 0 }, sch_medicaltrialdesign, sch_singleblindedtrial },
    { { 0, 95 }, { 0, 0 }, sch_medicaltrialdesign, sch_singlecentretrial },
    { { 0, 95 }, { 0, 0 }, sch_medicaltrialdesign, sch_tripleblindedtrial },
    { { 0, 95 }, { 0, 0 }, sch_medicinesystem, sch_ayurvedic }, // magic
    { { 0, 95 }, { 0, 0 }, sch_medicinesystem, sch_chiropractic }, // magic
    { { 0, 95 }, { 0, 0 }, sch_medicinesystem, sch_homeopathic }, // magic
    { { 0, 95 }, { 0, 0 }, sch_medicinesystem, sch_osteopathic }, // well...
    { { 0, 95 }, { 0, 0 }, sch_medicinesystem, sch_traditionalchinese }, // magic and environmental harm
    { { 0, 95 }, { 0, 0 }, sch_medicinesystem, sch_westernconventional },  // empirical (reality based)
    { { 6, 0 }, { 0, 0 }, sch_merchantreturnenumeration, sch_merchantreturnfinitereturnwindow  },
    { { 6, 0 }, { 0, 0 }, sch_merchantreturnenumeration, sch_merchantreturnnotpermitted  },
    { { 6, 0 }, { 0, 0 }, sch_merchantreturnenumeration, sch_merchantreturnunlimitedwindow  },
    { { 6, 0 }, { 0, 0 }, sch_merchantreturnenumeration, sch_merchantreturnunspecified  },
    { { 3, 0 }, { 0, 0 }, sch_message, sch_emailmessage },
    { { 1, 2 }, { 0, 0 }, sch_moveaction, sch_arriveaction },
    { { 1, 2 }, { 0, 0 }, sch_moveaction, sch_departaction },
    { { 1, 2 }, { 0, 0 }, sch_moveaction, sch_travelaction },
    { { 1, 92 }, { 0, 0 }, sch_musicalbumproductiontype, sch_compilationalbum },
    { { 1, 92 }, { 0, 0 }, sch_musicalbumproductiontype, sch_demoalbum },
    { { 1, 92 }, { 0, 0 }, sch_musicalbumproductiontype, sch_djmixalbum },
    { { 1, 92 }, { 0, 0 }, sch_musicalbumproductiontype, sch_livealbum },
    { { 1, 92 }, { 0, 0 }, sch_musicalbumproductiontype, sch_mixtapealbum },
    { { 1, 92 }, { 0, 0 }, sch_musicalbumproductiontype, sch_remixalbum },
    { { 1, 92 }, { 0, 0 }, sch_musicalbumproductiontype, sch_soundtrackalbum },
    { { 1, 92 }, { 0, 0 }, sch_musicalbumproductiontype, sch_spokenwordalbum },
    { { 1, 92 }, { 0, 0 }, sch_musicalbumproductiontype, sch_studioalbum },
    { { 1, 92 }, { 0, 0 }, sch_musicalbumreleasetype, sch_albumrelease },
    { { 1, 92 }, { 0, 0 }, sch_musicalbumreleasetype, sch_broadcastrelease },
    { { 1, 92 }, { 0, 0 }, sch_musicalbumreleasetype, sch_eprelease },
    { { 1, 92 }, { 0, 0 }, sch_musicalbumreleasetype, sch_singlerelease },
    { { 0, 10 }, { 0, 0 }, sch_musicplaylist, sch_musical_bum },
    { { 2, 0 }, { 0, 0 }, sch_musicplaylist, sch_musicrelease },
    { { 3, 5 }, { 0, 0 }, sch_musicreleaseformattype, sch_cdformat },
    { { 3, 5 }, { 0, 0 }, sch_musicreleaseformattype, sch_cassetteformat },
    { { 3, 5 }, { 0, 0 }, sch_musicreleaseformattype, sch_dvdformat },
    { { 3, 5 }, { 0, 0 }, sch_musicreleaseformattype, sch_digitalaudiotapeformat },
    { { 3, 5 }, { 0, 0 }, sch_musicreleaseformattype, sch_digitalformat },
    { { 3, 5 }, { 0, 0 }, sch_musicreleaseformattype, sch_laserdiscformat },
    { { 3, 5 }, { 0, 0 }, sch_musicreleaseformattype, sch_vinylformat },
    { { 3, 5 }, { 0, 0 }, sch_newsarticle, sch_analysisnewsarticle },
    { { 3, 5 }, { 0, 0 }, sch_newsarticle, sch_askpublicnewsarticle },
    { { 3, 5 }, { 0, 0 }, sch_newsarticle, sch_backgroundnewsarticle },
    { { 3, 5 }, { 0, 0 }, sch_newsarticle, sch_opinionnewsarticle },
    { { 3, 5 }, { 0, 0 }, sch_newsarticle, sch_reportagenewsarticle },
    { { 3, 5 }, { 0, 0 }, sch_newsarticle, sch_reviewnewsarticle },
    { { 9, 0 }, { 0, 0 }, sch_nlnonprofittype, sch_nonprofitanbi },
    { { 9, 0 }, { 0, 0 }, sch_nlnonprofittype, sch_nonprofitsbbi },
    { { 8, 0 }, { 0, 0 }, sch_nonprofittype, sch_nlnonprofittype },
    { { 8, 0 }, { 0, 0 }, sch_nonprofittype, sch_uknonprofittype },
    { { 8, 0 }, { 0, 0 }, sch_nonprofittype, sch_usnonprofittype },
    { { 0, 10 }, { 0, 0 }, sch_number, sch_float },
    { { 0, 10 }, { 0, 0 }, sch_number, sch_integer },
    { { 0, 10 }, { 0, 0 }, sch_offer, sch_aggregateoffer },
    { { 4, 0 }, { 0, 0 }, sch_offer, sch_offerforlease },
    { { 4, 0 }, { 0, 0 }, sch_offer, sch_offerforpurchase },
    { { 0, 10 }, { 0, 0 }, sch_offeritemcondition, sch_damagedcondition },
    { { 0, 10 }, { 0, 0 }, sch_offeritemcondition, sch_newcondition },
    { { 0, 10 }, { 0, 0 }, sch_offeritemcondition, sch_refurbishedcondition },
    { { 0, 10 }, { 0, 0 }, sch_offeritemcondition, sch_usedcondition },
    { { 14, 0 }, { 0, 0 }, sch_onlinebusiness, sch_onlinestore },
    { { 1, 92 }, { 0, 0 }, sch_orderstatus, sch_ordercancelled },
    { { 1, 92 }, { 0, 0 }, sch_orderstatus, sch_orderdelivered },
    { { 1, 92 }, { 0, 0 }, sch_orderstatus, sch_orderintransit },
    { { 1, 92 }, { 0, 0 }, sch_orderstatus, sch_orderpaymentdue },
    { { 1, 92 }, { 0, 0 }, sch_orderstatus, sch_orderpickupavailable },
    { { 1, 92 }, { 0, 0 }, sch_orderstatus, sch_orderproblem },
    { { 1, 92 }, { 0, 0 }, sch_orderstatus, sch_orderprocessing },
    { { 1, 92 }, { 0, 0 }, sch_orderstatus, sch_orderreturned },
    { { 1, 20 }, { 0, 0 }, sch_organisation, sch_airline },
    { { 3, 5 }, { 0, 0 }, sch_organisation, sch_consortium },
    { { 0, 10 }, { 2, 1 }, sch_organisation, sch_corporation },
    { { 3, 1 }, { 0, 0 }, sch_organisation, sch_corporation },
    { { 0, 10 }, { 0, 0 }, sch_organisation, sch_educationorganisation },
    { { 3, 5 }, { 0, 0 }, sch_organisation, sch_fundingscheme },
    { { 2, 0 }, { 2, 1 }, sch_organisation, sch_governmentorganisation },
    { { 3, 1 }, { 0, 0 }, sch_organisation, sch_governmentorganisation },
    { { 3, 5 }, { 0, 0 }, sch_organisation, sch_librarysystem },
    { { 0, 10 }, { 0, 0 }, sch_organisation, sch_localbusiness },
    { { 3, 0 }, { 0, 0 }, sch_organisation, sch_medicalorganisation },
    { { 0, 10 }, { 2, 1 }, sch_organisation, sch_ngo },
    { { 3, 1 }, { 0, 0 }, sch_organisation, sch_ngo },
    { { 3, 5 }, { 0, 0 }, sch_organisation, sch_newsmediaorganisation },
    { { s_odrl, 1, 0 }, { 0, 0 }, sch_organisation, odrl_party },
    { { 14, 0 }, { 0, 0 }, sch_organisation, sch_onlinebusiness },
    { { 0, 10 }, { 0, 0 }, sch_organisation, sch_performinggroup },
    { { 3, 5 }, { 0, 0 }, sch_organisation, sch_project },
    { { 14, 0 }, { 0, 0 }, sch_organisation, sch_searchrescueorganisation },
    { { 0, 10 }, { 1, 10 }, sch_organisation, sch_sportsteam },
    { { 1, 11 }, { 0, 0 }, sch_organisation, sch_sportsorganisation },
    { { 3, 5 }, { 0, 0 }, sch_organisation, sch_workersunion },
    { { 1, 2 }, { 0, 0 }, sch_organiseaction, sch_allocateaction },
    { { 1, 2 }, { 0, 0 }, sch_organiseaction, sch_applyaction },
    { { 1, 2 }, { 0, 0 }, sch_organiseaction, sch_bookmarkaction },
    { { 1, 2 }, { 0, 0 }, sch_organiseaction, sch_planaction },
    { { 1, 92 }, { 0, 0 }, sch_organisationrole, sch_employeerole },
    { { 3, 0 }, { 0, 0 }, sch_paymentcard, sch_creditcard },
    { { 0, 99 }, { 0, 0 }, sch_paymentmethod, gr_bybanktransferinadvance },
    { { 0, 99 }, { 0, 0 }, sch_paymentmethod, gr_byinvoice },
    { { 0, 99 }, { 0, 0 }, sch_paymentmethod, gr_cash },
    { { 0, 99 }, { 0, 0 }, sch_paymentmethod, gr_checkinadvance },
    { { 0, 99 }, { 0, 0 }, sch_paymentmethod, gr_cod },
    { { 0, 99 }, { 2, 1 }, sch_paymentmethod, sch_creditcard },
    { { 0, 99 }, { 0, 0 }, sch_paymentmethod, gr_directdebit },
    { { 0, 99 }, { 0, 0 }, sch_paymentmethod, gr_googlecheckout },
    { { 3, 0 }, { 0, 0 }, sch_paymentmethod, sch_paymentcard },
    { { 0, 99 }, { 0, 0 }, sch_paymentmethod, gr_paymentmethodcreditcard },
    { { 0, 99 }, { 0, 0 }, sch_paymentmethod, gr_paypal },
    { { 0, 99 }, { 0, 0 }, sch_paymentmethod, gr_payswarm },
    { { 3, 5 }, { 0, 0 }, sch_paymentstatustype, sch_paymentautomaticallyapplied },
    { { 3, 5 }, { 0, 0 }, sch_paymentstatustype, sch_paymentcomplete },
    { { 3, 5 }, { 0, 0 }, sch_paymentstatustype, sch_paymentdeclined },
    { { 3, 5 }, { 0, 0 }, sch_paymentstatustype, sch_paymentdue },
    { { 3, 5 }, { 0, 0 }, sch_paymentstatustype, sch_paymentpastdue },
    { { 1, 0 }, { 2, 1 }, sch_peopleaudience, sch_medicalaudience },
    { { 3, 5 }, { 0, 0 }, sch_peopleaudience, sch_medicalaudience },
    { { 1, 0 }, { 0, 0 }, sch_peopleaudience, sch_parentaudience },
    { { 0, 10 }, { 0, 0 }, sch_performinggroup, sch_dancegroup },
    { { 0, 10 }, { 0, 0 }, sch_performinggroup, sch_musicgroup },
    { { 0, 10 }, { 0, 0 }, sch_performinggroup, sch_theatregroup },
    { { 1, 90 }, { 0, 0 }, sch_periodical, sch_blog },
    { { 3, 5 }, { 0, 0 }, sch_periodical, sch_comicseries },
    { { 3, 5 }, { 0, 0 }, sch_periodical, sch_newspaper },
    { { 1, 4 }, { 2, 1 }, sch_permit, sch_governmentpermit },
    { { 3, 1 }, { 0, 0 }, sch_permit, sch_governmentpermit },
    { { 3, 5 }, { 0, 0 }, sch_person, sch_patient },
    { { s_odrl, 1, 0 }, { 0, 0 }, sch_person, odrl_party },
    { { 0, 95 }, { 0, 0 }, sch_physicalactivity, sch_exerciseplan },
    { { 0, 95 }, { 0, 0 }, sch_physicalactivitycategory, sch_aerobicactivity },
    { { 0, 95 }, { 0, 0 }, sch_physicalactivitycategory, sch_anaerobicactivity },
    { { 0, 95 }, { 0, 0 }, sch_physicalactivitycategory, sch_balance },
    { { 0, 95 }, { 0, 0 }, sch_physicalactivitycategory, sch_flexibility },
    { { 0, 95 }, { 0, 0 }, sch_physicalactivitycategory, sch_leisuretimeactivity },
    { { 0, 95 }, { 0, 0 }, sch_physicalactivitycategory, sch_occupationalactivity },
    { { 0, 95 }, { 0, 0 }, sch_physicalactivitycategory, sch_strengthtraining },
    { { 0, 95 }, { 0, 0 }, sch_physicalexam, sch_abdomen },
    { { 0, 95 }, { 0, 0 }, sch_physicalexam, sch_appearance },
    { { 0, 95 }, { 0, 0 }, sch_physicalexam, sch_cardiovascularexam },
    { { 0, 95 }, { 0, 0 }, sch_physicalexam, sch_ear },
    { { 0, 95 }, { 0, 0 }, sch_physicalexam, sch_eye },
    { { 0, 95 }, { 0, 0 }, sch_physicalexam, sch_genitourinary },
    { { 0, 95 }, { 0, 0 }, sch_physicalexam, sch_head },
    { { 0, 95 }, { 0, 0 }, sch_physicalexam, sch_lung },
    { { 0, 95 }, { 0, 0 }, sch_physicalexam, sch_musculoskeletalexam },
    { { 0, 95 }, { 0, 0 }, sch_physicalexam, sch_neck },
    { { 0, 95 }, { 0, 0 }, sch_physicalexam, sch_neuro },
    { { 0, 95 }, { 0, 0 }, sch_physicalexam, sch_nose },
    { { 0, 95 }, { 0, 0 }, sch_physicalexam, sch_skin },
    { { 0, 95 }, { 0, 0 }, sch_physicalexam, sch_throat },
    { { 3, 1 }, { 0, 0 }, sch_place, sch_accommodation },
    { { 0, 10 }, { 0, 0 }, sch_place, sch_administrativearea },
    { { 0, 10 }, { 0, 0 }, sch_place, sch_civicstructure },
    { { 0, 10 }, { 0, 0 }, sch_place, sch_landform },
    { { 0, 10 }, { 0, 0 }, sch_place, sch_landmarksorhistoricalbuildings },
    { { 0, 10 }, { 0, 0 }, sch_place, sch_localbusiness },
    { { 0, 10 }, { 0, 0 }, sch_place, sch_residence },
    { { 0, 10 }, { 0, 0 }, sch_place, sch_touristattraction },
    { { 3, 5 }, { 0, 0 }, sch_place, sch_touristdestination },
    { { 0, 10 }, { 0, 0 }, sch_placeofworship, sch_buddhisttemple },
    { { 0, 10 }, { 3, 5 }, sch_placeofworship, sch_catholicchurch },
    { { 0, 10 }, { 0, 0 }, sch_placeofworship, sch_church },
    { { 0, 10 }, { 0, 0 }, sch_placeofworship, sch_hindutemple },
    { { 0, 10 }, { 0, 0 }, sch_placeofworship, sch_mosque },
    { { 0, 10 }, { 0, 0 }, sch_placeofworship, sch_synagogue },
    { { 1, 2 }, { 0, 0 }, sch_planaction, sch_cancelaction },
    { { 1, 2 }, { 0, 0 }, sch_planaction, sch_reserveaction },
    { { 1, 2 }, { 0, 0 }, sch_planaction, sch_scheduleaction },
    { { 1, 2 }, { 0, 0 }, sch_playaction, sch_exerciseaction },
    { { 1, 2 }, { 0, 0 }, sch_playaction, sch_performaction },
    { { 11, 0 }, { 0, 0 }, sch_pricecomponenttypeenumeration, sch_activationfee },
    { { 11, 0 }, { 0, 0 }, sch_pricecomponenttypeenumeration, sch_cleaningfee },
    { { 11, 0 }, { 0, 0 }, sch_pricecomponenttypeenumeration, sch_distancefee },
    { { 11, 0 }, { 0, 0 }, sch_pricecomponenttypeenumeration, sch_downpayment },
    { { 11, 0 }, { 0, 0 }, sch_pricecomponenttypeenumeration, sch_installment },
    { { 11, 0 }, { 0, 0 }, sch_pricecomponenttypeenumeration, sch_subscription },
    { { 11, 0 }, { 0, 0 }, sch_pricetypeenumeration, sch_invoiceprice },
    { { 11, 0 }, { 0, 0 }, sch_pricetypeenumeration, sch_listprice },
    { { 11, 0 }, { 0, 0 }, sch_pricetypeenumeration, sch_msrp },
    { { 11, 0 }, { 0, 0 }, sch_pricetypeenumeration, sch_minimumadvertisedprice },
    { { 11, 0 }, { 0, 0 }, sch_pricetypeenumeration, sch_srp },
    { { 11, 0 }, { 0, 0 }, sch_pricetypeenumeration, sch_saleprice },
    { { 3, 0 }, { 0, 0 }, sch_pricespecification, sch_compoundpricespecification },
    { { 0, 99 }, { 0, 0 }, sch_pricespecification, sch_deliverychargespecification },
    { { 0, 99 }, { 0, 0 }, sch_pricespecification, sch_paymentchargespecification },
    { { 0, 99 }, { 0, 0 }, sch_pricespecification, sch_unitpricespecification },
    { { 0, 99 }, { 0, 0 }, sch_product, sch_individualproduct },
    { { 9, 0 }, { 0, 0 }, sch_product, sch_productcollection },
    { { 9, 0 }, { 0, 0 }, sch_product, sch_productgroup },
    { { 0, 99 }, { 0, 0 }, sch_product, sch_productmodel },
    { { 0, 99 }, { 0, 0 }, sch_product, sch_someproducts },
    { { 1, 20 }, { 0, 0 }, sch_product, sch_vehicle },
    { { 2, 0 }, { 0, 0 }, sch_product, gr_productorservicemodel },
    { { 3, 8 }, { 0, 0 }, sch_productreturnenumeration, sch_productreturnfinitereturnwindow },
    { { 3, 8 }, { 0, 0 }, sch_productreturnenumeration, sch_productreturnnotpermitted },
    { { 3, 8 }, { 0, 0 }, sch_productreturnenumeration, sch_productreturnunlimitedwindow },
    { { 3, 8 }, { 0, 0 }, sch_productreturnenumeration, sch_productreturnunspecified },
    { { 0, 10 }, { 2, 1 }, sch_professionalservice, sch_accountingservice },
    { { 0, 10 }, { 2, 1 }, sch_professionalservice, sch_attorney },
    { { 0, 10 }, { 3, 0 }, sch_professionalservice, sch_dentist },
    { { 0, 10 }, { 2, 1 }, sch_professionalservice, sch_electrician },
    { { 0, 10 }, { 2, 1 }, sch_professionalservice, sch_generalcontractor },
    { { 0, 10 }, { 2, 1 }, sch_professionalservice, sch_housepainter },
    { { 0, 10 }, { 2, 1 }, sch_professionalservice, sch_locksmith},
    { { 0, 10 }, { 2, 1 }, sch_professionalservice, sch_notary },
    { { 0, 10 }, { 2, 1 }, sch_professionalservice, sch_plumber },
    { { 0, 10 }, { 2, 1 }, sch_professionalservice, sch_roofingcontractor },
    { { 3, 5 }, { 0, 0 }, sch_project, sch_fundingagency },
    { { 3, 5 }, { 0, 0 }, sch_project, sch_researchproject },
    { { 1, 3 }, { 0, 0 }, sch_publicationevent, sch_broadcastevent },
    { { 1, 3 }, { 0, 0 }, sch_publicationevent, sch_ondemandevent },
    { { 2, 0 }, { 0, 0 }, sch_publicationissue, sch_comicissue },
    { { 3, 5 }, { 0, 0 }, sch_qualitativevalue, sch_bedtype },
    { { 2, 0 }, { 2, 1 }, sch_qualitativevalue, sch_carusagetype },
    { { 3, 5 }, { 8, 0 }, sch_qualitativevalue, sch_carusagetype },
    { { 2, 0 }, { 0, 0 }, sch_qualitativevalue, sch_drivewheelconfigurationvalue },
    { { 12, 0 }, { 0, 0 }, sch_qualitativevalue, sch_sizespecification },
    { { 2, 1 }, { 0, 0 }, sch_qualitativevalue, sch_steeringpositionvalue },
    { { 0, 10 }, { 0, 0 }, sch_quantity, sch_distance },
    { { 0, 10 }, { 0, 0 }, sch_quantity, sch_duration },
    { { 0, 15 }, { 0, 0 }, sch_quantity, sch_energy },
    { { 0, 10 }, { 0, 0 }, sch_quantity, sch_mass },
    { { 3, 5 }, { 0, 0 }, sch_quantitativevaluedistribution, sch_monetaryamountdistribution },
    { { 3, 5 }, { 0, 0 }, sch_radiochannel, sch_amradiochannel },
    { { 3, 5 }, { 0, 0 }, sch_radiochannel, sch_fmradiochannel },
    { { 0, 10 }, { 0, 0 }, sch_rating, sch_aggregaterating },
    { { 3, 5 }, { 0, 0 }, sch_rating, sch_endorsementrating },
    { { 1, 2 }, { 0, 0 }, sch_reactaction, sch_agreeaction },
    { { 1, 2 }, { 0, 0 }, sch_reactaction, sch_disagreeaction },
    { { 1, 2 }, { 0, 0 }, sch_reactaction, sch_dislikeaction },
    { { 1, 2 }, { 0, 0 }, sch_reactaction, sch_endorseaction },
    { { 1, 2 }, { 0, 0 }, sch_reactaction, sch_likeaction },
    { { 1, 2 }, { 0, 0 }, sch_reactaction, sch_wantaction },
    { { 3, 8 }, { 0, 0 }, sch_refundtypeenumeration, sch_exchangerefund },
    { { 3, 8 }, { 0, 0 }, sch_refundtypeenumeration, sch_fullrefund },
    { { 3, 8 }, { 0, 0 }, sch_refundtypeenumeration, sch_storecreditrefund },
    { { 9, 0 }, { 0, 0 }, sch_reservation, sch_boatreservation },
    { { 1, 20 }, { 2, 1 }, sch_reservation, sch_busreservation },
    { { 3, 1 }, { 0, 0 }, sch_reservation, sch_busreservation },
    { { 1, 20 }, { 2, 1 }, sch_reservation, sch_eventreservation },
    { { 3, 1 }, { 0, 0 }, sch_reservation, sch_eventreservation },
    { { 1, 20 }, { 0, 0 }, sch_reservation, sch_flightreservation },
    { { 1, 20 }, { 0, 0 }, sch_reservation, sch_foodestablishmentreservation },
    { { 1, 20 }, { 0, 0 }, sch_reservation, sch_lodgingreservation },
    { { 1, 20 }, { 0, 0 }, sch_reservation, sch_rentalcarreservation },
    { { 1, 20 }, { 2, 1 }, sch_reservation, sch_reservationpackage },
    { { 3, 1 }, { 0, 0 }, sch_reservation, sch_reservationpackage },
    { { 1, 20 }, { 2, 1 }, sch_reservation, sch_taxireservation },
    { { 3, 1 }, { 0, 0 }, sch_reservation, sch_taxireservation },
    { { 1, 20 }, { 0, 0 }, sch_reservation, sch_trainreservation },
    { { 1, 92 }, { 0, 0 }, sch_reservationstatustype, sch_reservationcancelled },
    { { 1, 92 }, { 0, 0 }, sch_reservationstatustype, sch_reservationconfirmed },
    { { 1, 92 }, { 0, 0 }, sch_reservationstatustype, sch_reservationhold },
    { { 1, 92 }, { 0, 0 }, sch_reservationstatustype, sch_reservationpending },
    { { 0, 10 }, { 0, 0 }, sch_residence, sch_apartmentcomplex },
    { { 0, 10 }, { 0, 0 }, sch_residence, sch_gatedresidencecommunity },
    { { 0, 10 }, { 3, 0 }, sch_residence, sch_singlefamilyresidence },
    { { 10, 0 }, { 0, 0 }, sch_resort, sch_skiresort },
    { { 3, 5 }, { 0, 0 }, sch_restricteddiet, sch_diabeticdiet },
    { { 3, 5 }, { 0, 0 }, sch_restricteddiet, sch_glutenfreediet },
    { { 3, 5 }, { 0, 0 }, sch_restricteddiet, sch_halaldiet },
    { { 3, 5 }, { 0, 0 }, sch_restricteddiet, sch_hindudiet },
    { { 3, 5 }, { 0, 0 }, sch_restricteddiet, sch_kosherdiet },
    { { 3, 5 }, { 0, 0 }, sch_restricteddiet, sch_lowcaloriediet },
    { { 3, 5 }, { 0, 0 }, sch_restricteddiet, sch_lowfatdiet },
    { { 3, 5 }, { 0, 0 }, sch_restricteddiet, sch_lowlactosediet },
    { { 3, 5 }, { 0, 0 }, sch_restricteddiet, sch_lowsaltdiet },
    { { 3, 5 }, { 0, 0 }, sch_restricteddiet, sch_vegandiet },
    { { 3, 5 }, { 0, 0 }, sch_restricteddiet, sch_vegetariandiet },
    { { 13, 0 }, { 0, 0 }, sch_returnfeesenumeration, sch_freereturn },
    { { 3, 8 }, { 0, 0 }, sch_returnfeesenumeration, sch_originalshippingfees },
    { { 3, 8 }, { 0, 0 }, sch_returnfeesenumeration, sch_restockingfees },
    { { 13, 0 }, { 0, 0 }, sch_returnfeesenumeration, sch_returnfeescustomerresponsibility },
    { { 3, 8 }, { 0, 0 }, sch_returnfeesenumeration, sch_returnshippingfees },
    { { 13, 0 }, { 0, 0 }, sch_returnlabelsourceenumeration, sch_returnlabelcustomerresponsibility },
    { { 13, 0 }, { 0, 0 }, sch_returnlabelsourceenumeration, sch_returnlabeldownloadandprint },
    { { 13, 0 }, { 0, 0 }, sch_returnlabelsourceenumeration, sch_returnlabelinbox },
    { { 13, 0 }, { 0, 0 }, sch_returnmethodenumeration, sch_returnatkiosk },
    { { 13, 0 }, { 0, 0 }, sch_returnmethodenumeration, sch_returnbymail },
    { { 13, 0 }, { 0, 0 }, sch_returnmethodenumeration, sch_returninstore },
    { { 3, 2 }, { 0, 0 }, sch_review, sch_claimreview },
    { { 3, 5 }, { 0, 0 }, sch_review, sch_criticreview },
    { { 3, 5 }, { 0, 0 }, sch_review, sch_employerreview },
    { { 7, 0 }, { 0, 0 }, sch_review, sch_mediareview },
    { { 6, 0 }, { 0, 0 }, sch_review, sch_recommendation },
    { { 3, 5 }, { 0, 0 }, sch_review, sch_userreview },
    { { 3, 5 }, { 0, 0 }, sch_role, sch_linkrole },
    { { 1, 92 }, { 0, 0 }, sch_role, sch_organisationrole },
    { { 1, 92 }, { 0, 0 }, sch_role, sch_performancerole },
    { { 3, 1 }, { 0, 0 }, sch_room, sch_hotelroom },
    { { 3, 1 }, { 0, 0 }, sch_room, sch_meetingroom },
    { { 1, 9 }, { 0, 0 }, sch_rsvpresponsetype, sch_rsvpresponsemaybe },
    { { 1, 9 }, { 0, 0 }, sch_rsvpresponsetype, sch_rsvpresponseno },
    { { 1, 9 }, { 0, 0 }, sch_rsvpresponsetype, sch_rsvpresponseyes },
    { { 0, 95 }, { 0, 0 }, sch_scholarlyarticle, sch_medicalscholarlyarticle },
    { { 1, 3 }, { 1, 93 }, sch_season, sch_radioseason },
    { { 1, 3 }, { 1, 93 }, sch_season, sch_tvseason },
    { { 3, 4 }, { 0, 0 }, sch_series, sch_creativeworkseries },
    { { 3, 5 }, { 0, 0 }, sch_series, sch_eventseries },
    { { 1, 3 }, { 1, 93 }, sch_series, sch_radioseries },
    { { 1, 3 }, { 1, 93 }, sch_series, sch_tvseries },
    { { 2, 1 }, { 0, 0 }, sch_service, sch_broadcastservice },
    { { 2, 0 }, { 0, 0 }, sch_service, sch_cableorsatelliteservice },
    { { 3, 0 }, { 0, 0 }, sch_service, sch_financialproduct },
    { { 3, 1 }, { 0, 0 }, sch_service, sch_foodservice },
    { { 1, 3 }, { 0, 0 }, sch_service, sch_governmentservice },
    { { 1, 20 }, { 2, 1 }, sch_service, sch_taxi },
    { { 2, 0 }, { 0, 0 }, sch_service, sch_taxiservice },
    { { 3, 5 }, { 0, 0 }, sch_service, sch_webapi },
    { { 12, 0 }, { 0, 0 }, sch_sizegroupenumeration, sch_wearablesizegroupenumeration },
    { { 12, 0 }, { 0, 0 }, sch_sizesystemenumeration, sch_sizesystemimperial },
    { { 12, 0 }, { 0, 0 }, sch_sizesystemenumeration, sch_sizesystemmetric },
    { { 12, 0 }, { 0, 0 }, sch_sizesystemenumeration, sch_wearablesizesystemenumeration },
    { { 2, 1 }, { 2, 2 }, sch_socialmediaposting, sch_blogposting },
    { { 3, 1 }, { 0, 0 }, sch_socialmediaposting, sch_blogposting },
    { { 2, 1 }, { 0, 0 }, sch_socialmediaposting, sch_discussionforumposting },
    { { 0, 95 }, { 0, 0 }, sch_softwareapplication, sch_mobileapplication },
    { { 2, 0 }, { 0, 0 }, sch_softwareapplication, sch_videogame },
    { { 0, 95 }, { 0, 0 }, sch_softwareapplication, sch_webapplication },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_speciality, sch_medicalspeciality },
    { { 0, 10 }, { 0, 0 }, sch_sportsactivitylocation, sch_bowlingalley },
    { { 0, 10 }, { 0, 0 }, sch_sportsactivitylocation, sch_exercisegym },
    { { 0, 10 }, { 0, 0 }, sch_sportsactivitylocation, sch_golfcourse },
    { { 0, 10 }, { 0, 0 }, sch_sportsactivitylocation, sch_healthclub },
    { { 0, 10 }, { 0, 0 }, sch_sportsactivitylocation, sch_publicswimmingpool },
    { { 0, 10 }, { 9, 0 }, sch_sportsactivitylocation, sch_skiresort },
    { { 10, 0 }, { 0, 0 }, sch_sportsactivitylocation, sch_resort },
    { { 0, 10 }, { 0, 0 }, sch_sportsactivitylocation, sch_sportsclub },
    { { 0, 10 }, { 0, 0 }, sch_sportsactivitylocation, sch_stadiumorarena },
    { { 0, 10 }, { 0, 0 }, sch_sportsactivitylocation, sch_tenniscomplex },
    { { 2, 0 }, { 0, 0 }, sch_sportsorganisation, sch_sportsteam },
    { { 9, 0 }, { 0, 0 }, sch_statusenumeration, sch_actionstatustype },
    { { 9, 0 }, { 0, 0 }, sch_statusenumeration, sch_eventstatustype },
    { { 9, 0 }, { 0, 0 }, sch_statusenumeration, sch_gameserverstatus },
    { { 9, 0 }, { 0, 0 }, sch_statusenumeration, sch_legalforcestatus },
    { { 9, 0 }, { 0, 0 }, sch_statusenumeration, sch_orderstatus },
    { { 9, 0 }, { 0, 0 }, sch_statusenumeration, sch_paymentstatustype },
    { { 9, 0 }, { 0, 0 }, sch_statusenumeration, sch_reservationstatustype },
    { { 3, 6 }, { 5, 0 }, sch_steeringpositionvalue, sch_lefthanddriving },
    { { 3, 6 }, { 5, 0 }, sch_steeringpositionvalue, sch_righthanddriving },
    { { 9, 0 }, { 0, 0 }, sch_steeringpositionvalue, sch_lefthanddriving },
    { { 9, 0 }, { 0, 0 }, sch_steeringpositionvalue, sch_righthanddriving },
    { { 0, 10 }, { 0, 0 }, sch_store, sch_carpartsshop },
    { { 0, 10 }, { 0, 0 }, sch_store, sch_bikeshop },
    { { 0, 10 }, { 0, 0 }, sch_store, sch_bookshop },
    { { 0, 10 }, { 0, 0 }, sch_store, sch_clothesshop },
    { { 0, 10 }, { 0, 0 }, sch_store, sch_computershop },
    { { 0, 10 }, { 0, 0 }, sch_store, sch_convenienceshop },
    { { 0, 10 }, { 0, 0 }, sch_store, sch_departmentstore },
    { { 0, 10 }, { 0, 0 }, sch_store, sch_electronicsshop },
    { { 0, 10 }, { 0, 0 }, sch_store, sch_florist },
    { { 0, 10 }, { 0, 0 }, sch_store, sch_furnitureshop },
    { { 0, 10 }, { 0, 0 }, sch_store, sch_gardencentre },
    { { 0, 10 }, { 0, 0 }, sch_store, sch_grocers },
    { { 0, 10 }, { 0, 0 }, sch_store, sch_hardwareshop },
    { { 0, 10 }, { 0, 0 }, sch_store, sch_hobbyshop },
    { { 0, 10 }, { 0, 0 }, sch_store, sch_householdgoodsshop },
    { { 0, 10 }, { 0, 0 }, sch_store, sch_jewellers },
    { { 0, 10 }, { 0, 0 }, sch_store, sch_offlicence },
    { { 0, 10 }, { 0, 0 }, sch_store, sch_tailors },
    { { 0, 10 }, { 0, 0 }, sch_store, sch_mobilephoneshop },
    { { 0, 10 }, { 0, 0 }, sch_store, sch_filmhireshop },
    { { 0, 10 }, { 0, 0 }, sch_store, sch_musicshop },
    { { 0, 10 }, { 0, 0 }, sch_store, sch_officeshop },
    { { 0, 10 }, { 0, 0 }, sch_store, sch_outletstore },
    { { 0, 10 }, { 0, 0 }, sch_store, sch_pawnshop },
    { { 0, 10 }, { 0, 0 }, sch_store, sch_petshop },
    { { 0, 10 }, { 0, 0 }, sch_store, sch_shoeshop },
    { { 0, 10 }, { 0, 0 }, sch_store, sch_sportsshop },
    { { 0, 10 }, { 0, 0 }, sch_store, sch_tyreshop },
    { { 0, 10 }, { 0, 0 }, sch_store, sch_toyshop },
    { { 0, 10 }, { 0, 0 }, sch_store, sch_wholesalers },
    { { 7, 3 }, { 0, 0 }, sch_structuredvalue, sch_cdcpmdrecord },
    { { 0, 10 }, { 0, 0 }, sch_structuredvalue, sch_contactpoint },
    { { 1, 92 }, { 2, 1 }, sch_structuredvalue, sch_datedmoneyspecification },
    { { 7, 4 }, { 2, 1 }, sch_structuredvalue, sch_definedregion },
    { { 8, 0 }, { 2, 1 }, sch_structuredvalue, sch_deliverytimesettings },
    { { 2, 0 }, { 0, 0 }, sch_structuredvalue, sch_enginespecification },
    { { 3, 5 }, { 0, 0 }, sch_structuredvalue, sch_exchangeratespecification },
    { { 0, 10 }, { 0, 0 }, sch_structuredvalue, sch_geocoordinates },
    { { 0, 30 }, { 0, 0 }, sch_structuredvalue, sch_geoshape },
    { { 2, 1 }, { 0, 0 }, sch_structuredvalue, sch_interactioncounter },
    { { 3, 0 }, { 0, 0 }, sch_structuredvalue, sch_monetaryamount },
    { { 0, 10 }, { 0, 0 }, sch_structuredvalue, sch_nutritioninformation },
    { { 7, 4 }, { 0, 0 }, sch_structuredvalue, sch_offershippingdetails },
    { { 0, 99 }, { 0, 0 }, sch_structuredvalue, sch_openinghoursspecification },
    { { 0, 99 }, { 0, 0 }, sch_structuredvalue, sch_ownershipinfo },
    { { 0, 99 }, { 0, 0 }, sch_structuredvalue, sch_pricespecification },
    { { 8, 0 }, { 0, 0 }, sch_structuredvalue, sch_postalcoderangespecification },
    { { 2, 0 }, { 0, 0 }, sch_structuredvalue, sch_propertyvalue },
    { { 0, 99 }, { 0, 0 }, sch_structuredvalue, sch_quantitativevalue },
    { { 3, 5 }, { 0, 0 }, sch_structuredvalue, sch_quantitativevaluedistribution },
    { { 3, 5 }, { 0, 0 }, sch_structuredvalue, sch_repaymentspecification },
    { { 8, 0 }, { 0, 0 }, sch_structuredvalue, sch_shippingdeliverytime },
    { { 8, 0 }, { 0, 0 }, sch_structuredvalue, sch_shippingratesettings },
    { { 0, 99 }, { 0, 0 }, sch_structuredvalue, sch_typeandquantitynode },
    { { 0, 99 }, { 0, 0 }, sch_structuredvalue, sch_warrantypromise },
    { { 3, 5 }, { 0, 0 }, sch_substance, sch_dietarysupplement },
    { { 3, 5 }, { 0, 0 }, sch_substance, sch_drug },
    { { 1, 0 }, { 0, 0 }, sch_techarticle, sch_apireference },
    { { 2, 0 }, { 0, 0 }, sch_text, sch_cssselectortype },
    { { 2, 0 }, { 0, 0 }, sch_text, sch_pronounceabletext },
    { { 0, 10 }, { 0, 0 }, sch_text, sch_url },
    { { 2, 0 }, { 0, 0 }, sch_text, sch_xpathtype },
    { { 3, 5 }, { 0, 0 }, sch_therapeuticprocedure, sch_medicaltherapy },
    { { 3, 5 }, { 0, 0 }, sch_therapeuticprocedure, sch_psychologicaltreatment },
    { { 2, 0 }, { 0, 0 }, sch_thing, sch_action },
    { { 13, 0 }, { 0, 0 }, sch_thing, sch_biochementity },
    { { 1, 3 }, { 2, 0 }, sch_thing, sch_broadcastservice },
    { { 1, 0 }, { 1, 91 }, sch_thing, sch_class },
    { { 0, 10 }, { 0, 0 }, sch_thing, sch_creativework },
    { { 0, 10 }, { 0, 0 }, sch_thing, sch_event },
    { { 0, 10 }, { 0, 0 }, sch_thing, sch_intangible },
    { { 0, 95 }, { 2, 1 }, sch_thing, sch_medicalentity },
    { { 3, 5 }, { 0, 0 }, sch_thing, sch_medicalentity },
    { { 12, 0 }, { 0, 0 }, sch_thing, sch_occupationalexperiencerequirements },
    { { 0, 10 }, { 0, 0 }, sch_thing, sch_organisation },
    { { 0, 10 }, { 0, 0 }, sch_thing, sch_person },
    { { 0, 10 }, { 0, 0 }, sch_thing, sch_place },
    { { 0, 10 }, { 0, 0 }, sch_thing, sch_product },
    { { 1, 0 }, { 1, 91 }, sch_thing, sch_property },
    { { 13, 0 }, { 0, 0 }, sch_thing, sch_taxon },
    { { 1, 2 }, { 0, 0 }, sch_tradeaction, sch_buyaction },
    { { 1, 2 }, { 0, 0 }, sch_tradeaction, sch_donateaction },
    { { 1, 2 }, { 0, 0 }, sch_tradeaction, sch_orderaction },
    { { 1, 2 }, { 0, 0 }, sch_tradeaction, sch_payaction },
    { { 3, 5 }, { 0, 0 }, sch_tradeaction, sch_preorderaction },
    { { 1, 2 }, { 0, 0 }, sch_tradeaction, sch_quoteaction },
    { { 1, 2 }, { 0, 0 }, sch_tradeaction, sch_rentaction },
    { { 1, 2 }, { 0, 0 }, sch_tradeaction, sch_sellaction },
    { { 1, 2 }, { 0, 0 }, sch_tradeaction, sch_tipaction },
    { { 1, 2 }, { 0, 0 }, sch_transferaction, sch_borrowaction },
    { { 1, 2 }, { 0, 0 }, sch_transferaction, sch_downloadaction },
    { { 1, 2 }, { 0, 0 }, sch_transferaction, sch_giveaction },
    { { 1, 2 }, { 0, 0 }, sch_transferaction, sch_lendaction },
    { { 4, 0 }, { 0, 0 }, sch_transferaction, sch_moneytransfer },
    { { 1, 2 }, { 0, 0 }, sch_transferaction, sch_receiveaction },
    { { 1, 2 }, { 0, 0 }, sch_transferaction, sch_returnaction },
    { { 1, 2 }, { 0, 0 }, sch_transferaction, sch_sendaction },
    { { 1, 2 }, { 0, 0 }, sch_transferaction, sch_takeaction },
    { { 9, 0 }, { 0, 0 }, sch_trip, sch_boattrip },
    { { 3, 4 }, { 0, 0 }, sch_trip, sch_bustrip },
    { { 3, 4 }, { 0, 0 }, sch_trip, sch_flight },
    { { 3, 5 }, { 0, 0 }, sch_trip, sch_touristtrip },
    { { 3, 4 }, { 0, 0 }, sch_trip, sch_traintrip },
    { { 1, 2 }, { 0, 0 }, sch_updateaction, sch_addaction },
    { { 1, 2 }, { 0, 0 }, sch_updateaction, sch_deleteaction },
    { { 1, 2 }, { 0, 0 }, sch_updateaction, sch_replaceaction },
    { { 9, 0 }, { 0, 0 }, sch_uknonprofittype, sch_charitableincorporatedorganization },
    { { 9, 0 }, { 0, 0 }, sch_uknonprofittype, sch_limitedbyguaranteecharity },
    { { 9, 0 }, { 0, 0 }, sch_uknonprofittype, sch_uktrust },
    { { 9, 0 }, { 0, 0 }, sch_uknonprofittype, sch_unincorporatedassociationcharity },
    { { 1, 2 }, { 0, 0 }, sch_useaction, sch_wearaction },
    { { 0, 10 }, { 3, 4 }, sch_userinteraction, sch_userblocks },
    { { 0, 10 }, { 3, 4 }, sch_userinteraction, sch_usercheckins },
    { { 0, 10 }, { 3, 4 }, sch_userinteraction, sch_usercomments },
    { { 0, 10 }, { 3, 4 }, sch_userinteraction, sch_userdownloads },
    { { 0, 10 }, { 3, 4 }, sch_userinteraction, sch_userlikes },
    { { 0, 10 }, { 3, 4 }, sch_userinteraction, sch_userpagevisits },
    { { 0, 10 }, { 3, 4 }, sch_userinteraction, sch_userplays },
    { { 0, 10 }, { 3, 4 }, sch_userinteraction, sch_userplusones },
    { { 0, 10 }, { 3, 4 }, sch_userinteraction, sch_usertweets },
    { { 9, 0 }, { 0, 0 }, sch_usnonprofittype, sch_nonprofit501a, },
    { { 9, 0 }, { 0, 0 }, sch_usnonprofittype, sch_nonprofit501c1, },
    { { 9, 0 }, { 0, 0 }, sch_usnonprofittype, sch_nonprofit501c10, },
    { { 9, 0 }, { 0, 0 }, sch_usnonprofittype, sch_nonprofit501c11, },
    { { 9, 0 }, { 0, 0 }, sch_usnonprofittype, sch_nonprofit501c12, },
    { { 9, 0 }, { 0, 0 }, sch_usnonprofittype, sch_nonprofit501c13, },
    { { 9, 0 }, { 0, 0 }, sch_usnonprofittype, sch_nonprofit501c14, },
    { { 9, 0 }, { 0, 0 }, sch_usnonprofittype, sch_nonprofit501c15, },
    { { 9, 0 }, { 0, 0 }, sch_usnonprofittype, sch_nonprofit501c16, },
    { { 9, 0 }, { 0, 0 }, sch_usnonprofittype, sch_nonprofit501c17, },
    { { 9, 0 }, { 0, 0 }, sch_usnonprofittype, sch_nonprofit501c18, },
    { { 9, 0 }, { 0, 0 }, sch_usnonprofittype, sch_nonprofit501c19, },
    { { 9, 0 }, { 0, 0 }, sch_usnonprofittype, sch_nonprofit501c2, },
    { { 9, 0 }, { 0, 0 }, sch_usnonprofittype, sch_nonprofit501c20, },
    { { 9, 0 }, { 0, 0 }, sch_usnonprofittype, sch_nonprofit501c21, },
    { { 9, 0 }, { 0, 0 }, sch_usnonprofittype, sch_nonprofit501c22, },
    { { 9, 0 }, { 0, 0 }, sch_usnonprofittype, sch_nonprofit501c23, },
    { { 9, 0 }, { 0, 0 }, sch_usnonprofittype, sch_nonprofit501c24, },
    { { 9, 0 }, { 0, 0 }, sch_usnonprofittype, sch_nonprofit501c25, },
    { { 9, 0 }, { 0, 0 }, sch_usnonprofittype, sch_nonprofit501c26, },
    { { 9, 0 }, { 0, 0 }, sch_usnonprofittype, sch_nonprofit501c27, },
    { { 9, 0 }, { 0, 0 }, sch_usnonprofittype, sch_nonprofit501c28, },
    { { 9, 0 }, { 0, 0 }, sch_usnonprofittype, sch_nonprofit501c3, },
    { { 9, 0 }, { 0, 0 }, sch_usnonprofittype, sch_nonprofit501c4, },
    { { 9, 0 }, { 0, 0 }, sch_usnonprofittype, sch_nonprofit501c5, },
    { { 9, 0 }, { 0, 0 }, sch_usnonprofittype, sch_nonprofit501c6, },
    { { 9, 0 }, { 0, 0 }, sch_usnonprofittype, sch_nonprofit501c7, },
    { { 9, 0 }, { 0, 0 }, sch_usnonprofittype, sch_nonprofit501c8, },
    { { 9, 0 }, { 0, 0 }, sch_usnonprofittype, sch_nonprofit501c9, },
    { { 9, 0 }, { 0, 0 }, sch_usnonprofittype, sch_nonprofit501d, },
    { { 9, 0 }, { 0, 0 }, sch_usnonprofittype, sch_nonprofit501e, },
    { { 9, 0 }, { 0, 0 }, sch_usnonprofittype, sch_nonprofit501f, },
    { { 9, 0 }, { 0, 0 }, sch_usnonprofittype, sch_nonprofit501k, },
    { { 9, 0 }, { 0, 0 }, sch_usnonprofittype, sch_nonprofit501n, },
    { { 9, 0 }, { 0, 0 }, sch_usnonprofittype, sch_nonprofit501q, },
    { { 9, 0 }, { 0, 0 }, sch_usnonprofittype, sch_nonprofit527, },
    { { 3, 5 }, { 0, 0 }, sch_vehicle, sch_busorcoach },
    { { 1, 20 }, { 0, 0 }, sch_vehicle, sch_car },
    { { 3, 5 }, { 0, 0 }, sch_vehicle, sch_motorcycle },
    { { 3, 5 }, { 0, 0 }, sch_vehicle, sch_motorisedbicycle },
    { { 0, 95 }, { 0, 0 }, sch_vessel, sch_artery },
    { { 0, 95 }, { 0, 0 }, sch_vessel, sch_lymphaticvessel },
    { { 0, 95 }, { 0, 0 }, sch_vessel, sch_vein },
    { { 13, 0 }, { 0, 0 }, sch_videoobject, sch_videoobjectsnapshot },
    { { 3, 5 }, { 0, 0 }, sch_visualartwork, sch_coverart },
    { { 0, 99 }, { 0, 0 }, sch_warrantyscope, gr_labour_bringin },
    { { 0, 99 }, { 0, 0 }, sch_warrantyscope, gr_partsandlabour_bringin },
    { { 0, 99 }, { 0, 0 }, sch_warrantyscope, gr_partsandlabour_pickup },
    { { 12, 0 }, { 0, 0 }, sch_wearablemeasurementtypeenumeration, sch_wearablemeasurementback },
    { { 12, 0 }, { 0, 0 }, sch_wearablemeasurementtypeenumeration, sch_wearablemeasurementchestorbust },
    { { 12, 0 }, { 0, 0 }, sch_wearablemeasurementtypeenumeration, sch_wearablemeasurementcollar },
    { { 12, 0 }, { 0, 0 }, sch_wearablemeasurementtypeenumeration, sch_wearablemeasurementcup },
    { { 12, 0 }, { 0, 0 }, sch_wearablemeasurementtypeenumeration, sch_wearablemeasurementheight },
    { { 12, 0 }, { 0, 0 }, sch_wearablemeasurementtypeenumeration, sch_wearablemeasurementhips },
    { { 12, 0 }, { 0, 0 }, sch_wearablemeasurementtypeenumeration, sch_wearablemeasurementinseam },
    { { 12, 0 }, { 0, 0 }, sch_wearablemeasurementtypeenumeration, sch_wearablemeasurementlength },
    { { 12, 0 }, { 0, 0 }, sch_wearablemeasurementtypeenumeration, sch_wearablemeasurementoutsideleg },
    { { 12, 0 }, { 0, 0 }, sch_wearablemeasurementtypeenumeration, sch_wearablemeasurementsleeve },
    { { 12, 0 }, { 0, 0 }, sch_wearablemeasurementtypeenumeration, sch_wearablemeasurementwaist },
    { { 12, 0 }, { 0, 0 }, sch_wearablemeasurementtypeenumeration, sch_wearablemeasurementwidth },
    { { 12, 0 }, { 0, 0 }, sch_wearablesizegroupenumeration, sch_wearablesizegroupbig },
    { { 12, 0 }, { 0, 0 }, sch_wearablesizegroupenumeration, sch_wearablesizegroupboys },
    { { 12, 0 }, { 0, 0 }, sch_wearablesizegroupenumeration, sch_wearablesizegroupextrashort },
    { { 12, 0 }, { 0, 0 }, sch_wearablesizegroupenumeration, sch_wearablesizegroupextratall },
    { { 12, 0 }, { 0, 0 }, sch_wearablesizegroupenumeration, sch_wearablesizegroupgirls },
    { { 12, 0 }, { 0, 0 }, sch_wearablesizegroupenumeration, sch_wearablesizegrouphusky },
    { { 12, 0 }, { 0, 0 }, sch_wearablesizegroupenumeration, sch_wearablesizegroupinfants },
    { { 12, 0 }, { 0, 0 }, sch_wearablesizegroupenumeration, sch_wearablesizegroupjuniors },
    { { 12, 0 }, { 0, 0 }, sch_wearablesizegroupenumeration, sch_wearablesizegroupmaternity },
    { { 12, 0 }, { 0, 0 }, sch_wearablesizegroupenumeration, sch_wearablesizegroupmens },
    { { 12, 0 }, { 0, 0 }, sch_wearablesizegroupenumeration, sch_wearablesizegroupmisses },
    { { 12, 0 }, { 0, 0 }, sch_wearablesizegroupenumeration, sch_wearablesizegrouppetite },
    { { 12, 0 }, { 0, 0 }, sch_wearablesizegroupenumeration, sch_wearablesizegroupplus },
    { { 12, 0 }, { 0, 0 }, sch_wearablesizegroupenumeration, sch_wearablesizegroupregular },
    { { 12, 0 }, { 0, 0 }, sch_wearablesizegroupenumeration, sch_wearablesizegroupshort },
    { { 12, 0 }, { 0, 0 }, sch_wearablesizegroupenumeration, sch_wearablesizegrouptall },
    { { 12, 0 }, { 0, 0 }, sch_wearablesizegroupenumeration, sch_wearablesizegroupwomens },
    { { 12, 0 }, { 0, 0 }, sch_wearablesizesystemenumeration, sch_wearablesizesystemau },
    { { 12, 0 }, { 0, 0 }, sch_wearablesizesystemenumeration, sch_wearablesizesystembr },
    { { 12, 0 }, { 0, 0 }, sch_wearablesizesystemenumeration, sch_wearablesizesystemcn },
    { { 12, 0 }, { 0, 0 }, sch_wearablesizesystemenumeration, sch_wearablesizesystemcontinental },
    { { 12, 0 }, { 0, 0 }, sch_wearablesizesystemenumeration, sch_wearablesizesystemde },
    { { 12, 0 }, { 0, 0 }, sch_wearablesizesystemenumeration, sch_wearablesizesystemen13402 },
    { { 12, 0 }, { 0, 0 }, sch_wearablesizesystemenumeration, sch_wearablesizesystemeurope },
    { { 12, 0 }, { 0, 0 }, sch_wearablesizesystemenumeration, sch_wearablesizesystemfr },
    { { 12, 0 }, { 0, 0 }, sch_wearablesizesystemenumeration, sch_wearablesizesystemgs1 },
    { { 12, 0 }, { 0, 0 }, sch_wearablesizesystemenumeration, sch_wearablesizesystemit },
    { { 12, 0 }, { 0, 0 }, sch_wearablesizesystemenumeration, sch_wearablesizesystemjp },
    { { 12, 0 }, { 0, 0 }, sch_wearablesizesystemenumeration, sch_wearablesizesystemmx },
    { { 12, 0 }, { 0, 0 }, sch_wearablesizesystemenumeration, sch_wearablesizesystemuk },
    { { 12, 0 }, { 0, 0 }, sch_wearablesizesystemenumeration, sch_wearablesizesystemus },
    { { 5, 0 }, { 0, 0 }, sch_webcontent, sch_healthtopiccontent },
    { { 0, 10 }, { 0, 0 }, sch_webpage, sch_aboutpage },
    { { 0, 10 }, { 0, 0 }, sch_webpage, sch_checkoutpage },
    { { 0, 10 }, { 0, 0 }, sch_webpage, sch_collectionpage },
    { { 0, 10 }, { 0, 0 }, sch_webpage, sch_contactpage },
    { { 3, 5 }, { 0, 0 }, sch_webpage, sch_faqpage },
    { { 0, 10 }, { 0, 0 }, sch_webpage, sch_itempage },
    { { 0, 95 }, { 2, 1 }, sch_webpage, sch_medicalwebpage },
    { { 3, 5 }, { 0, 0 }, sch_webpage, sch_medicalwebpage },
    { { 0, 10 }, { 0, 0 }, sch_webpage, sch_profilepage },
    { { 2, 0 }, { 0, 0 }, sch_webpage, sch_qapage },
    { { 4, 0 }, { 0, 0 }, sch_webpage, sch_realestatelisting },
    { { 0, 10 }, { 0, 0 }, sch_webpage, sch_searchresultspage },
    { { 0, 10 }, { 0, 0 }, sch_webpageelement, sch_sitenavigationelement },
    { { 0, 10 }, { 0, 0 }, sch_webpageelement, sch_table },
    { { 0, 10 }, { 0, 0 }, sch_webpageelement, sch_wpadblock },
    { { 0, 10 }, { 0, 0 }, sch_webpageelement, sch_wpfooter },
    { { 0, 10 }, { 0, 0 }, sch_webpageelement, sch_wpheader },
    { { 0, 10 }, { 0, 0 }, sch_webpageelement, sch_wpsidebar },

    // sd
    { { s_sd, 1, 0 }, { 0, 0 }, sd_feature, sd_aggregate },
    { { s_sd, 1, 0 }, { 0, 0 }, sd_feature, sd_basicfederatedquery },
    { { s_sd, 1, 0 }, { 0, 0 }, sd_feature, sd_dereferencesuris },
    { { s_sd, 1, 0 }, { 0, 0 }, sd_feature, sd_emptygraphs },
    { { s_sd, 1, 0 }, { 0, 0 }, sd_feature, sd_entailmentprofile },
    { { s_sd, 1, 0 }, { 0, 0 }, sd_feature, sd_entailmentregime },
    { { s_sd, 1, 0 }, { 0, 0 }, sd_feature, sd_function },
    { { s_sd, 1, 0 }, { 0, 0 }, sd_feature, sd_language },
    { { s_sd, 1, 0 }, { 0, 0 }, sd_feature, sd_requiresdataset },
    { { s_sd, 1, 0 }, { 0, 0 }, sd_feature, sd_uniondefaultgraph },

    { { s_sd, 1, 0 }, { 0, 0 }, sd_graphcollection, sd_dataset },

    { { s_sd, 1, 0 }, { 0, 0 }, sd_uniondefaultgraph, sd_sparql10query },
    { { s_sd, 1, 0 }, { 0, 0 }, sd_uniondefaultgraph, sd_sparql11query },
    { { s_sd, 1, 0 }, { 0, 0 }, sd_uniondefaultgraph, sd_sparql11update },

    // sioc
    { { s_sioc, 1, 0 }, { 0, 0 }, sioc_container, sioc_forum },
    { { s_sioc, 1, 0 }, { 0, 0 }, sioc_container, sioc_thread },

    { { s_sioc, 1, 0 }, { 0, 0 }, sioc_item, sioc_post },

    { { s_sioc, 1, 0 }, { 0, 0 }, sioc_space, sioc_site },

    // skos
    { { s_dcat, 2, 0 }, { 0, 0 }, skos_collection, skos_orderedcollection },

    { { s_dqv, 1, 0 }, { 0, 0 }, skos_concept, dqv_category },
    { { s_dqv, 1, 0 }, { 0, 0 }, skos_concept, dqv_dimension },
    { { s_frbr, 1, 0 }, { 0, 0 }, skos_concept, fs_concept },
    { { s_oa, 1, 0 }, { 0, 0 }, skos_concept, oa_motivation },
    { { s_skos, 1, 0 }, { 0, 0 }, skos_concept, dca_role },
    { { s_org, 1, 0 }, { 0, 0 }, skos_concept, org_role },

    // sosa
    { { s_ssn, 1, 0 }, { 0, 0 }, sosa_featureofinterest, sosa_sample },
    { { s_ssn, 1, 0 }, { 0, 0 }, sosa_result, sosa_sample },

    // ssn
    { { s_ssn, 1, 0 }, { 0, 0 }, ssn_property, sosa_actuatableproperty },
    { { s_ssn, 1, 0 }, { 0, 0 }, ssn_property, sosa_observableproperty },

    { { s_ssn, 1, 0 }, { 0, 0 }, ssn_system, sosa_actuator },
    { { s_ssn, 1, 0 }, { 0, 0 }, ssn_system, sosa_sampler },
    { { s_ssn, 1, 0 }, { 0, 0 }, ssn_system, sosa_sensor },

    // time
    { { s_time, 1, 0 }, { 0, 0 }, time_datetimedescription, time_april, },
    { { s_time, 1, 0 }, { 0, 0 }, time_datetimedescription, time_august, },
    { { s_time, 1, 0 }, { 0, 0 }, time_datetimedescription, time_february, },
    { { s_time, 1, 0 }, { 0, 0 }, time_datetimedescription, time_december, },
    { { s_time, 1, 0 }, { 0, 0 }, time_datetimedescription, time_january, },
    { { s_time, 1, 0 }, { 0, 0 }, time_datetimedescription, time_july, },
    { { s_time, 1, 0 }, { 0, 0 }, time_datetimedescription, time_june, },
    { { s_time, 1, 0 }, { 0, 0 }, time_datetimedescription, time_march, },
    { { s_time, 1, 0 }, { 0, 0 }, time_datetimedescription, time_may, },
    { { s_time, 1, 0 }, { 0, 0 }, time_datetimedescription, time_monthofyear, },
    { { s_time, 1, 0 }, { 0, 0 }, time_datetimedescription, time_november, },
    { { s_time, 1, 0 }, { 0, 0 }, time_datetimedescription, time_october, },
    { { s_time, 1, 0 }, { 0, 0 }, time_datetimedescription, time_september, },

    { { s_time, 1, 0 }, { 0, 0 }, time_dayofweek, time_friday },
    { { s_time, 1, 0 }, { 0, 0 }, time_dayofweek, time_monday },
    { { s_time, 1, 0 }, { 0, 0 }, time_dayofweek, time_thursday },
    { { s_time, 1, 0 }, { 0, 0 }, time_dayofweek, time_tuesday },
    { { s_time, 1, 0 }, { 0, 0 }, time_dayofweek, time_saturday },
    { { s_time, 1, 0 }, { 0, 0 }, time_dayofweek, time_sunday },
    { { s_time, 1, 0 }, { 0, 0 }, time_dayofweek, time_wednesday },

    { { s_time, 1, 0 }, { 0, 0 }, time_durationdescription, time_year },

    { { s_time, 1, 0 }, { 0, 0 }, time_generaldatetimedescription, time_datetimedescription },

    { { s_time, 1, 0 }, { 0, 0 }, time_generaldurationdescription, time_durationdescription },

    { { s_time, 1, 0 }, { 0, 0 }, time_interval, time_properinterval },

    { { s_time, 1, 0 }, { 0, 0 }, time_properinterval, time_datetimeinterval },

    { { s_time, 1, 0 }, { 0, 0 }, time_temporalduration, time_duration },
    { { s_time, 1, 0 }, { 0, 0 }, time_temporalduration, time_generaldurationdescription },
    { { s_time, 1, 0 }, { 0, 0 }, time_temporalduration, time_temporalunit },

    { { s_time, 1, 0 }, { 0, 0 }, time_temporalentity, time_instant },
    { { s_time, 1, 0 }, { 0, 0 }, time_temporalentity, time_interval },

    { { s_time, 1, 0 }, { 0, 0 }, time_temporalposition, time_generaldatetimedescription },
    { { s_time, 1, 0 }, { 0, 0 }, time_temporalposition, time_position },

    { { s_time, 1, 0 }, { 0, 0 }, time_temporalunit, time_unitday },
    { { s_time, 1, 0 }, { 0, 0 }, time_temporalunit, time_unithour },
    { { s_time, 1, 0 }, { 0, 0 }, time_temporalunit, time_unitminute },
    { { s_time, 1, 0 }, { 0, 0 }, time_temporalunit, time_unitmonth },
    { { s_time, 1, 0 }, { 0, 0 }, time_temporalunit, time_unitsecond },
    { { s_time, 1, 0 }, { 0, 0 }, time_temporalunit, time_unitweek },
    { { s_time, 1, 0 }, { 0, 0 }, time_temporalunit, time_unityear },

    // vcard
    { { s_odrl, 1, 0 }, { 0, 0 }, vcard_cagent, odrl_party },
    { { s_odrl, 1, 0 }, { 0, 0 }, vcard_individual, odrl_party },
    { { s_odrl, 1, 0 }, { 0, 0 }, vcard_organisation, odrl_party },

    { { s_vcard, 2, 0 }, { s_vcard, 3, 0 }, vcard_address, vcard_dom },
    { { s_vcard, 2, 0 }, { s_vcard, 3, 0 }, vcard_address, vcard_home },
    { { s_vcard, 2, 0 }, { s_vcard, 3, 0 }, vcard_address, vcard_intl },
    { { s_vcard, 2, 0 }, { s_vcard, 3, 0 }, vcard_address, vcard_parcel },
    { { s_vcard, 2, 0 }, { s_vcard, 3, 0 }, vcard_address, vcard_postal },
    { { s_vcard, 2, 0 }, { s_vcard, 3, 0 }, vcard_address, vcard_pref },
    { { s_vcard, 2, 0 }, { s_vcard, 3, 0 }, vcard_address, vcard_work },

    { { s_vcard, 1, 0 }, { s_vcard, 1, 0 }, vcard_adrtypes, vcard_dom },
    { { s_vcard, 1, 0 }, { s_vcard, 1, 0 }, vcard_adrtypes, vcard_home },
    { { s_vcard, 1, 0 }, { s_vcard, 1, 0 }, vcard_adrtypes, vcard_intl },
    { { s_vcard, 1, 0 }, { s_vcard, 1, 0 }, vcard_adrtypes, vcard_label },
    { { s_vcard, 1, 0 }, { s_vcard, 1, 0 }, vcard_adrtypes, vcard_parcel },
    { { s_vcard, 1, 0 }, { s_vcard, 1, 0 }, vcard_adrtypes, vcard_postal },
    { { s_vcard, 1, 0 }, { s_vcard, 1, 0 }, vcard_adrtypes, vcard_pref },
    { { s_vcard, 1, 0 }, { s_vcard, 1, 0 }, vcard_adrtypes, vcard_work },

    { { s_vcard, 3, 0 }, { s_vcard, 3, 0 }, vcard_email, vcard_internet },
    { { s_vcard, 3, 0 }, { s_vcard, 3, 0 }, vcard_email, vcard_x400 },
    { { s_vcard, 3, 0 }, { s_vcard, 3, 0 }, vcard_email, vcard_pref },

    { { s_vcard, 1, 0 }, { s_vcard, 1, 0 }, vcard_emailtypes, vcard_internet },
    { { s_vcard, 1, 0 }, { s_vcard, 1, 0 }, vcard_emailtypes, vcard_x400 },
    { { s_vcard, 1, 0 }, { s_vcard, 1, 0 }, vcard_emailtypes, vcard_pref },

    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_cgender, vcard_female },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_cgender, vcard_male },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_cgender, vcard_cother },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_cgender, vcard_none },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_cgender, vcard_unknown },

    { { s_vcard, 3, 0 }, { s_vcard, 3, 0 }, vcard_label, vcard_dom },
    { { s_vcard, 3, 0 }, { s_vcard, 3, 0 }, vcard_label, vcard_home },
    { { s_vcard, 3, 0 }, { s_vcard, 3, 0 }, vcard_label, vcard_intl },
    { { s_vcard, 3, 0 }, { s_vcard, 3, 0 }, vcard_label, vcard_parcel },
    { { s_vcard, 3, 0 }, { s_vcard, 3, 0 }, vcard_label, vcard_postal },
    { { s_vcard, 3, 0 }, { s_vcard, 3, 0 }, vcard_label, vcard_pref },
    { { s_vcard, 3, 0 }, { s_vcard, 3, 0 }, vcard_label, vcard_work },

    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_relatedtype, vcard_cacquaintance },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_relatedtype, vcard_cagent },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_relatedtype, vcard_cchild },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_relatedtype, vcard_ccolleague },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_relatedtype, vcard_ccontact },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_relatedtype, vcard_ccoresident },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_relatedtype, vcard_ccoworker },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_relatedtype, vcard_ccrush },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_relatedtype, vcard_cdate },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_relatedtype, vcard_cemergency },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_relatedtype, vcard_cfriend },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_relatedtype, vcard_ckin },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_relatedtype, vcard_cme },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_relatedtype, vcard_cmet },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_relatedtype, vcard_cmuse },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_relatedtype, vcard_cneighbour },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_relatedtype, vcard_cparent },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_relatedtype, vcard_csibling },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_relatedtype, vcard_cspouse },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_relatedtype, vcard_csweetheart },

    { { s_vcard, 3, 0 }, { s_vcard, 3, 0 }, vcard_tel, vcard_bbs },
    { { s_vcard, 3, 0 }, { s_vcard, 3, 0 }, vcard_tel, vcard_car },
    { { s_vcard, 3, 0 }, { s_vcard, 3, 0 }, vcard_tel, vcard_fax },
    { { s_vcard, 3, 0 }, { s_vcard, 3, 0 }, vcard_tel, vcard_home },
    { { s_vcard, 3, 0 }, { s_vcard, 3, 0 }, vcard_tel, vcard_isdn },
    { { s_vcard, 3, 0 }, { s_vcard, 3, 0 }, vcard_tel, vcard_mobile },
    { { s_vcard, 3, 0 }, { s_vcard, 3, 0 }, vcard_tel, vcard_modem },
    { { s_vcard, 3, 0 }, { s_vcard, 3, 0 }, vcard_tel, vcard_msg },
    { { s_vcard, 3, 0 }, { s_vcard, 3, 0 }, vcard_tel, vcard_pager },
    { { s_vcard, 3, 0 }, { s_vcard, 3, 0 }, vcard_tel, vcard_pcs },
    { { s_vcard, 3, 0 }, { s_vcard, 3, 0 }, vcard_tel, vcard_pref },
    { { s_vcard, 3, 0 }, { s_vcard, 3, 0 }, vcard_tel, vcard_video },
    { { s_vcard, 3, 0 }, { s_vcard, 3, 0 }, vcard_tel, vcard_voice },
    { { s_vcard, 3, 0 }, { s_vcard, 3, 0 }, vcard_tel, vcard_work },

    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_ctype, vcard_dom },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_ctype, vcard_home },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_ctype, vcard_isdn },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_ctype, vcard_internet },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_ctype, vcard_intl },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_ctype, vcard_label },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_ctype, vcard_parcel },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_ctype, vcard_postal },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_ctype, vcard_pref },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_ctype, vcard_work },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_ctype, vcard_x400 },

    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_telephonetype, vcard_bbs },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_telephonetype, vcard_car },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_telephonetype, vcard_ctext },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_telephonetype, vcard_ctextphone },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_telephonetype, vcard_fax },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_telephonetype, vcard_home },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_telephonetype, vcard_isdn },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_telephonetype, vcard_mobile },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_telephonetype, vcard_modem },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_telephonetype, vcard_msg },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_telephonetype, vcard_pager },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_telephonetype, vcard_pcs },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_telephonetype, vcard_pref },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_telephonetype, vcard_video },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_telephonetype, vcard_voice },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_telephonetype, vcard_work },

    { { s_vcard, 1, 0 }, { s_vcard, 1, 0 }, vcard_teltypes, vcard_bbs },
    { { s_vcard, 1, 0 }, { s_vcard, 1, 0 }, vcard_teltypes, vcard_car },
    { { s_vcard, 1, 0 }, { s_vcard, 1, 0 }, vcard_teltypes, vcard_fax },
    { { s_vcard, 1, 0 }, { s_vcard, 1, 0 }, vcard_teltypes, vcard_home },
    { { s_vcard, 1, 0 }, { s_vcard, 1, 0 }, vcard_teltypes, vcard_isdn },
    { { s_vcard, 1, 0 }, { s_vcard, 1, 0 }, vcard_teltypes, vcard_mobile },
    { { s_vcard, 1, 0 }, { s_vcard, 1, 0 }, vcard_teltypes, vcard_modem },
    { { s_vcard, 1, 0 }, { s_vcard, 1, 0 }, vcard_teltypes, vcard_msg },
    { { s_vcard, 1, 0 }, { s_vcard, 1, 0 }, vcard_teltypes, vcard_pager },
    { { s_vcard, 1, 0 }, { s_vcard, 1, 0 }, vcard_teltypes, vcard_pcs },
    { { s_vcard, 1, 0 }, { s_vcard, 1, 0 }, vcard_teltypes, vcard_pref },
    { { s_vcard, 1, 0 }, { s_vcard, 1, 0 }, vcard_teltypes, vcard_video },
    { { s_vcard, 1, 0 }, { s_vcard, 1, 0 }, vcard_teltypes, vcard_work },

    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_vcard, vcard_cgroup },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_vcard, vcard_individual },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_vcard, vcard_organisation },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_vcard, vcard_location },

    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_vkind, vcard_cgroup },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_vkind, vcard_individual },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_vkind, vcard_organisation },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_vkind, vcard_location },

    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_vcardtype, vcard_home },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_vcardtype, vcard_work },

    // wordnet
//    { { s_doap, 1, 0 }, { 0, 0 }, wordnet_project, doap_project },

    // XML schema types
    { { s_xsd, 1, 0 }, { 0, 0 }, xsd_anytype, xsd_anysimpletype },

    { { s_xsd, 1, 1 }, { s_xsd, 1, 0 }, xsd_anyatomictype, xsd_anyuri },
    { { s_xsd, 1, 1 }, { s_xsd, 1, 0 }, xsd_anyatomictype, xsd_date },
    { { s_xsd, 1, 1 }, { s_xsd, 1, 0 }, xsd_anyatomictype, xsd_base64binary },
    { { s_xsd, 1, 1 }, { s_xsd, 1, 0 }, xsd_anyatomictype, xsd_boolean },
    { { s_xsd, 1, 1 }, { s_xsd, 1, 0 }, xsd_anyatomictype, xsd_datetime },
    { { s_xsd, 1, 1 }, { s_xsd, 1, 0 }, xsd_anyatomictype, xsd_decimal },
    { { s_xsd, 1, 1 }, { s_xsd, 1, 0 }, xsd_anyatomictype, xsd_double },
    { { s_xsd, 1, 1 }, { s_xsd, 1, 0 }, xsd_anyatomictype, xsd_duration },
    { { s_xsd, 1, 1 }, { s_xsd, 1, 0 }, xsd_anyatomictype, xsd_float },
    { { s_xsd, 1, 1 }, { s_xsd, 1, 0 }, xsd_anyatomictype, xsd_hexbinary },
    { { s_xsd, 1, 1 }, { s_xsd, 1, 0 }, xsd_anyatomictype, xsd_notation },
    { { s_xsd, 1, 1 }, { s_xsd, 1, 0 }, xsd_anyatomictype, xsd_qname },
    { { s_xsd, 1, 1 }, { s_xsd, 1, 0 }, xsd_anyatomictype, xsd_string },

    { { s_xsd, 1, 1 }, { 0, 0 }, xsd_anysimpletype, xsd_anyatomictype },
    { { s_xsd, 1, 1 }, { 0, 0 }, xsd_anysimpletype, xsd_entities },
    { { s_xsd, 1, 1 }, { 0, 0 }, xsd_anysimpletype, xsd_idrefs },
    { { s_xsd, 1, 1 }, { 0, 0 }, xsd_anysimpletype, xsd_nmtokens },

    { { s_xsd, 1, 0 }, { s_xsd, 1, 0 }, xsd_anysimpletype, xsd_anyuri },
    { { s_xsd, 1, 0 }, { s_xsd, 1, 0 }, xsd_anysimpletype, xsd_date },
    { { s_xsd, 1, 0 }, { s_xsd, 1, 0 }, xsd_anysimpletype, xsd_base64binary },
    { { s_xsd, 1, 0 }, { s_xsd, 1, 0 }, xsd_anysimpletype, xsd_boolean },
    { { s_xsd, 1, 0 }, { s_xsd, 1, 0 }, xsd_anysimpletype, xsd_datetime },
    { { s_xsd, 1, 0 }, { s_xsd, 1, 0 }, xsd_anysimpletype, xsd_decimal },
    { { s_xsd, 1, 0 }, { s_xsd, 1, 0 }, xsd_anysimpletype, xsd_double },
    { { s_xsd, 1, 0 }, { s_xsd, 1, 0 }, xsd_anysimpletype, xsd_duration },
    { { s_xsd, 1, 0 }, { s_xsd, 1, 0 }, xsd_anysimpletype, xsd_float },
    { { s_xsd, 1, 0 }, { s_xsd, 1, 0 }, xsd_anysimpletype, xsd_hexbinary },
    { { s_xsd, 1, 0 }, { s_xsd, 1, 0 }, xsd_anysimpletype, xsd_notation },
    { { s_xsd, 1, 0 }, { s_xsd, 1, 0 }, xsd_anysimpletype, xsd_qname },
    { { s_xsd, 1, 0 }, { s_xsd, 1, 0 }, xsd_anysimpletype, xsd_string },

    { { s_xsd, 1, 1 }, { 0, 0 }, xsd_datetime, xsd_datetimestamp },

    { { s_xsd, 1, 1 }, { 0, 0 }, xsd_duration, xsd_datetimeduration },
    { { s_xsd, 1, 1 }, { 0, 0 }, xsd_duration, xsd_yearmonthduration },

    { { s_xsd, 1, 0 }, { s_xsd, 1, 0 }, xsd_entity, xsd_entities },

    { { s_xsd, 1, 0 }, { s_xsd, 1, 0 }, xsd_idref, xsd_idrefs },

    { { s_xsd, 1, 0 }, { 0, 0 }, xsd_long, xsd_short },

    { { s_xsd, 1, 0 }, { 0, 0 }, xsd_integer, xsd_long },
    { { s_xsd, 1, 0 }, { 0, 0 }, xsd_integer, xsd_nonnegativeinteger },
    { { s_xsd, 1, 0 }, { 0, 0 }, xsd_integer, xsd_nonpositiveinteger },

    { { s_xsd, 1, 0 }, { 0, 0 }, xsd_long, xsd_int },

    { { s_xsd, 1, 0 }, { 0, 0 }, xsd_name, xsd_ncname },

    { { s_xsd, 1, 0 }, { 0, 0 }, xsd_ncname, xsd_entity },
    { { s_xsd, 1, 0 }, { 0, 0 }, xsd_ncname, xsd_id },
    { { s_xsd, 1, 0 }, { 0, 0 }, xsd_ncname, xsd_idref },

    { { s_xsd, 1, 0 }, { s_xsd, 1, 0 }, xsd_nmtoken, xsd_nmtokens },

    { { s_xsd, 1, 0 }, { 0, 0 }, xsd_nonnegativeinteger, xsd_positiveinteger },
    { { s_xsd, 1, 0 }, { 0, 0 }, xsd_nonnegativeinteger, xsd_unsignedlong },

    { { s_xsd, 1, 0 }, { 0, 0 }, xsd_nonpositiveinteger, xsd_negativeinteger },

    { { s_xsd, 1, 0 }, { 0, 0 }, xsd_token, xsd_language },
    { { s_xsd, 1, 0 }, { 0, 0 }, xsd_token, xsd_name },
    { { s_xsd, 1, 0 }, { 0, 0 }, xsd_token, xsd_nmtoken },

    { { s_xsd, 1, 0 }, { 0, 0 }, xsd_normalisedstring, xsd_token },

    { { s_xsd, 1, 0 }, { 0, 0 }, xsd_long, xsd_byte },

    { { s_xsd, 1, 1 }, { 0, 0 }, xsd_string, xsd_normalisedstring },

    { { s_xsd, 1, 0 }, { 0, 0 }, xsd_unsignedint, xsd_unsignedshort },
    { { s_xsd, 1, 0 }, { 0, 0 }, xsd_unsignedlong, xsd_unsignedint },
    { { s_xsd, 1, 0 }, { 0, 0 }, xsd_unsignedshort, xsd_unsignedbyte },

    { { 0, 0 }, { 0, 0 }, sty_illegal, sty_illegal } };

typedef ::std::multimap < e_schema_type, microdata_hierachy* > vmap_t;
typedef ::std::multimap < e_schema_type, e_schema_type > vss_t;
vmap_t hierarchy;
vss_t generalisations;

#ifdef _MSC_VER
#pragma warning (push, 3)
#pragma warning (disable : 26446 26482) // Suggested solution breaks the compilation, plus ::std::array can't be length initialised by the initiliser
#endif // _MSC_VER
void hierarchy_init (nitpick& nits)
{   PRESUME (hierarchy.empty (), __FILE__, __LINE__);
    for (::std::size_t x = 0; schema_hierarchy [x].general_ != sty_illegal; ++x)
    {   hierarchy.insert (vmap_t::value_type (schema_hierarchy [x].general_, &schema_hierarchy [x]));
        if (generalisations.find (schema_hierarchy [x].specific_) != generalisations.cend ())
            for (vmap_t::const_iterator vi = hierarchy.find (schema_hierarchy [x].general_); (vi != hierarchy.cend ()) && (vi -> first == schema_hierarchy [x].general_); ++vi)
                if (vi -> second != nullptr)
                    if ((vi -> second -> specific_ == schema_hierarchy [x].specific_) && (vi -> second -> general_ != schema_hierarchy [x].general_))
                        if (overlap (schema_hierarchy [x].from_, schema_hierarchy [x].to_, vi -> second -> from_, vi -> second -> to_))
                            nits.pick (nit_schema_hierarchy, es_catastrophic, ec_schema, "multiple generalisations for ", sch::name (schema_hierarchy [x].specific_), " (", schema_hierarchy [x].specific_, ")");
        if (context.tell (es_all)) nits.pick (nit_all, es_all, ec_schema, sch::name (schema_hierarchy [x].general_), " : ", sch::name (schema_hierarchy [x].specific_)); 
        generalisations.emplace (vss_t::value_type (schema_hierarchy [x].specific_, schema_hierarchy [x].general_)); } }
#ifdef _MSC_VER
#pragma warning (pop)
#endif // _MSC_VER

void int_generalise  (const e_schema_type s, ssch_t& ssch)
{   if (ssch.find (s) == ssch.cend ()) ssch.insert (s);
    for (vss_t::const_iterator i = generalisations.find (s); (i != generalisations.cend ()) && (i -> first == s); ++i)
        for (vmap_t::const_iterator vi = hierarchy.find (i -> second); (vi != hierarchy.cend ()) && (vi -> first == i -> second); ++vi)
            if (vi -> second != nullptr)
                if (vi -> second -> specific_ == s)
                    if (does_apply < schema_version > (get_default_schema_version (vi -> second -> from_.root ()), vi -> second -> from_, vi -> second -> to_))
                        int_generalise (i -> second, ssch); }

ssch_t generalise (const e_schema_type s)
{   ssch_t res;
    int_generalise (s, res);
    return res; }

bool is_specific_type_of (const e_schema_type general, const e_schema_type specific)
{   for (vss_t::const_iterator x = generalisations.find (specific); x != generalisations.cend (); ++x)
    {   if (x -> first != specific) break;
        if (x -> second == general) return true;
        if (is_specific_type_of (general, x -> second)) return true; }
    return false; }

